/**
 * Window-Manager + Window + Dialog + Drag&Drop
 */

function jkqWindowInfo() {

	this.win = null;
	this.ziWindow = 0;
	this.ziModal = 0;
	this.id = 0;

}

function jkqWindowManagerObj() {

	this.zindex = 90;
	this.windowId = 0;

	this.windows = [];

	/**
	 * register a new window
	 *
	 * @returns {jkqWindowInfo}
	 */
	this.addWindow = function(win) {

		var winInfo = new jkqWindowInfo();
		winInfo.win = win;
		winInfo.ziWindow = this.zindex + 1;
		winInfo.ziModal = this.zindex;
		winInfo.id = this.windowId;

		this.windows[this.windowId] = winInfo;

		this.windowId++;
		this.zindex += 2;

		return winInfo;

	};

	this.removeWindow = function(win) {

		var ix = -1;
		this.zindex = 90;
		var i = 0;

		for (var id in this.windows) {

			if (this.windows[id].win == win) {
				ix = i;
			} else if (this.windows[id].ziWindow > this.zindex) {
				this.zindex = this.windows[id].ziWindow + 1;
			}
			i++;

		}

		if (ix > -1) {
			this.windows.splice(ix, 1);
		}

	};

}
jkqWindowManager = new jkqWindowManagerObj();

function AdjustProps(props) {
	this.hAlign = 'center';
	this.vAlign = 'center';
	this.effect = '';
	this.duration = 400;

	$.extend(this, props);
}

function jkqWindow() {

	/**
	 * default Window-Template
	 */
	this.template =
		'<div class="jkqWindow jk-round">' +
			'<div class="jkqWindowTitleBar">' +
				'<div class="jkqWindowIcon"></div>' +
				'<div class="jkqWindowClose" title="close"></div>' +
				'<div class="jkqWindowTitle jkqWindowMovable"></div>' +
			'</div>' +
			'<div class="jkqWindowContent"></div>' +
		'</div>';

	/**
	 * Window-Element
	 */
	this.container	= null;

	this.width		= 'auto';			// width and ...
	this.height		=    0;				// height of the container
	this.minheight	=  150;				// estimated height of the container (to calculate its position)
	this.pos_x		=    0;				// position of
	this.pos_y		=    0;				// the container
	this.distX		=   20;				// horizontal and ...
	this.distY		=   10;				// vertical distanc to mousepointer
	/** @returns {jkqWindowInfo} */
	this.ctx		= null;				// data for this window, filled by jkqWindowManager

	/**
	 * Callback-Function: window gets closed
	 */
	this.onClose   = null;

	/**
	 * inserted area for modal windows
	 */
	this.$modal_canvas = null;

	/**
	 * stores the event position delivered to this.show() for usage in this.ajustPos()
	 * {x, y} or null
	 *
	 * @type {x, y}
	 */
	var showEventPos = null;

	/**
	 * create the window element
	 *
	 * @param e			jqEvent:	Position can be calculated by mouse pointer position (=> adjustPos)
	 * 					null:		Position cannot be calculated by mouse pointer position (=> adjustPos)
	 * @param modal		boolean (optional)
	 */
	this.show = function(e, modal) {

		var self = this;

		this.close(e);

		this.ctx = jkqWindowManager.addWindow(this);

		if (modal) {

			var modal_height = Math.max($(document).outerHeight(true), $(window).height() );
			var modal_width = Math.max($(document).outerWidth(true), $(window).width() );

			this.$modal_canvas = $('<div class="jkqModalBg"/>').css({
				width : modal_width,
				height : modal_height,
				zIndex : this.ctx.ziModal  // < jkqWindow.z-index < drag-canvas.z-index
			}).click(function(e) {
				e.stopPropagation();
			})
			.appendTo('BODY');

		}

		//var pixelRatio = (window.devicePixelRatio) ? window.devicePixelRatio : 1;

		// mobile test
		//var mobile = false;
		//if ((screen.width / pixelRatio <= 480) || (screen.height / pixelRatio <= 480)) {
		//	mobile = true;
		//}

		if (e && !e.isTrigger && (e.clientX > 0 || e.clientY > 0)) {

			showEventPos = {x: e.pageX, y: e.pageY};

		} else {

			showEventPos = null;

		}

		this.container = $(this.template)
		.css({
			display	: 'none',
			//left	: this.pos_x + "px",
			//top		: this.pos_y + "px",
			width	: this.width + (isNaN(this.width) ? "" : "px"),
			zIndex	: this.ctx.ziWindow
		})
		.appendTo($('body')); //(parent);

		if (modal) {
			// modal windows should not propagate the click event to html
			this.container.click(function(e) {
				e.stopPropagation();
			});
		}

		// close
		$(this.container).find('.jkqWindowClose').click(function (e) {
			e.stopPropagation();
			self.close(e);
		});

		// move
		dragndrop = new jkqDragDrop();
		$(this.container).find('.jkqWindowMovable').mousedown(function (e) {
			dragndrop._start(e, self.container);
		});

	};

	/**
	 * enables closing the window, if the user clicks outside the window
	 *
	 * @param {jqEvent}	e
	 */
	this.closeOnOutsideClick = function() {

		var self = this;

		window.setTimeout(function()  {
			$('html').one('click', function(event) {
				self.close(event);
			});
		}, 100);

		this.container.click(function(event){
			event.stopPropagation();
		});

	};

	/**
	 * sets the position in respect of the real width and height of the window
	 * should be called, after the content is inserted
	 *
	 * @param {AdjustProps}	properties	object with properties [hAlign],[vAlign],[effect],[duration] <br>
	 * <br>
	 * properties: <br>
	 * {String}	hAlign		left, right, center (default), e.left, e.right, e.center <br>
	 * {String}	vAlign		top, bottom, center (default), e.top, e.bottom, e.center <br>
	 *						the 'e.xyz' values means position relative to the mouse position <br>
	 * {String}	effect		slide-rtl <br>
	 * {Number}	duration	duration of the effect animation in ms (default = 400) <br>
	 *
	 */
	this.adjustPos = function(properties) {

		var props = new AdjustProps( properties );

		// remove 'e.' if position is not available
		if (!showEventPos) {
			props.hAlign = props.hAlign.replace(/e\./, '');
			props.vAlign = props.vAlign.replace(/e\./, '');
		}

		var scrollLeft	= $(document).scrollLeft();
		var scrollTop	= $(document).scrollTop();

		var w = this.container.outerWidth();
		var h = this.container.outerHeight();

		switch (props.hAlign) {
			case 'left':
				this.pos_x = scrollLeft + this.distX;
				break;

			case 'right':
				this.pos_x = scrollLeft + $(window).width() - w - this.distX;
				break;

			case 'e.right':
				this.pos_x = showEventPos.x + this.distX;
				break;

			case 'e.left':
				this.pos_x = showEventPos.x - this.distX - w;
				break;

			case 'e.center':
				this.pos_x = showEventPos.x - parseInt(w / 2);
				break;

			default: // center
				this.pos_x = parseInt(($(window).width() - w) / 2) + scrollLeft;
				break;
		}

		if (
				(!props.hAlign) &&
				( this.pos_x + w > scrollLeft + $(window).width() )
		) {
			this.pos_x = scrollLeft + $(window).width() - w;
		}
		if ( this.pos_x < scrollLeft ) this.pos_x = scrollLeft;

		switch (props.vAlign) {
			case 'top':
				this.pos_y = scrollTop + this.distY;
				break;

			case 'bottom':
				this.pos_y = scrollTop + $(window).height() - h - this.distY;
				break;

			case 'e.bottom':
				this.pos_y = showEventPos.y + this.distY;
				break;

			case 'e.top':
				this.pos_y = showEventPos.y - this.distY - h;
				break;

			case 'e.center':
				this.pos_y = showEventPos.y - parseInt(h / 2);
				break;

			default: // center
				this.pos_y = parseInt(($(window).height() - h) / 2) + scrollTop;
				break;
		}


		if (
				( props.vAlign.indexOf('-') < 0 ) &&
				( this.pos_y + h > scrollTop + $(window).height() )
		) {
			this.pos_y = scrollTop + $(window).height() - h;
		}
		if ( this.pos_y < scrollTop ) this.pos_y = scrollTop;

		var posX = this.pos_x;
		var posY = this.pos_y;

		// effect
		var anim = null;
		switch (props.effect) {
			case 'slide-rtl':

				var outsideRight = scrollLeft + $(window).width();
				anim = { left: '-=' + (outsideRight - posX) };
				posX = outsideRight;
				break;

			default:
				break;
		}

		// display
		this.container.css({
			display	: 'block',
			left	: posX + "px",
			top		: posY + "px",
		});

		if (anim) {
			this.container.animate( anim, props.duration );
		}

	};

	/**
	 * shortcut: show modal window
	 *
	 * @method showModal
	 * @param e		jQuery-Event
	 */
	this.showModal = function (e) {
		this.show(e, true);
	};

	/**
	 * closes the window and removes the container
	 *
	 * @param {Function} onCloseHandler_or_Event or {jqEvent} event
	 * @returns {jkqWindow}
	 */
	this.close = function (onCloseHandler_or_Event) {

		if (typeof onCloseHandler_or_Event == 'function') {

			this.onClose = onCloseHandler_or_Event;

		} else {

			var event = (typeof onCloseHandler_or_Event == 'object') ? onCloseHandler_or_Event : null;

			if (this.container) {

				var doClose = true;

				// old onClose concept
				if (this.onClose) {
					var res = this.onClose(event);
					if ((typeof res == 'boolean') && !res) {
						doClose = false;
					}
				}

				// new onClose concept
				if ( $(this).triggerHandler('close') === false) {
					// an event handler does not want this wundow to be closed
					doClose = false;
				}

				if (doClose) {

					$(this.container).remove();

					this.container = null;

					// remove Modal Canvas
					if (this.$modal_canvas) {

						this.$modal_canvas.remove();

						this.$modal_canvas = null;

					}

					jkqWindowManager.removeWindow(this);

				}

			}

		}

		return this;

	};

	/**
	 * set title text
	 *
	 * @param {String}	title
	 * @returns {jkqWindow}
	 */
	this.setTitle = function(title) {

		$(this.container).find('DIV.jkqWindowTitle').html(title);

		return this;

	};

	/**
	 * shortcut for xy.container.on(eventname, handler)
	 *
	 * @param {String}		eventname
	 * @param {Function}	handler
	 * @returns {jkqWindow}
	 */
	this.on = function(eventname, handler) {

		this.container.on(eventname, handler);

		return this;

	};

}

jkqDialog.prototype = new jkqWindow();
/**
 * dialog window <br>
 * call it with: new jkqDialog({ <br>
 * {String}	title:					window title <br>
 * {String}	text:					dialog text <br>
 * {Object}	buttons:				{ButtonName: ButtonCaption, ... } <br>
 * {String}	css_class:				(optional) classname added to div.jkqDialogContent <br>
 * {Bool}	cancelButton:			(optional default=true) add a cancel button (jkqDialogCancel), if not defined in buttons <br>
 * {Bool}	closeOnOutsideClick:	(optional default=false) close the dialog, if user clicks outside the window. Needs event parameter in confirm()
 * }).confirm(function(button) { ... }, [e])
 *
 * @param {Object} properties
 * @returns {jkqDialog}
 */
function jkqDialog(properties) {

	this.properties = {
			title:				'',
			css_class:			'',
			text:				'',
			cancelButton:		true,
			icon:				'',
			buttons:			{},
			closeOnOutsideClick:false,
			autoSkipText:		'',
			autoSkipButton:		'',
	};

	this.autoSkipEnabled = function() {
		return (
			(amplify) &&
			(this.properties.autoSkipButton) &&
			(this.properties.autoSkipText)
		);
	};

	/**
	 * @deprecated
	 */
	this.not = function (property) {
		return (typeof property	== 'undefined');
	};

	/**
	 * initialize the dialog properties
	 *
	 * @param {Object}	properties	Members: <br>
	 * <br>
	 * {String}	title:					window title <br>
	 * {String}	text:					dialog text <br>
	 * {Object}	buttons:				{ButtonName: ButtonCaption, ... } <br>
	 * {String}	css_class:				(optional) classname added to div.jkqDialogContent <br>
	 * {Bool}	cancelButton:			(optional default=true) add a cancel button (jkqDialogCancel), if not defined in buttons <br>
	 * {Bool}	closeOnOutsideClick:	(optional default=false) close the dialog, if user clicks outside the window. Needs event parameter in confirm()
	 */
	this.init = function (properties) {

		/*/ set defaults
		if (this.not(properties.title))					properties.title = '';
		if (this.not(properties.css_class))				properties.css_class = '';
		if (this.not(properties.text))					properties.text = '';
		if (this.not(properties.cancelButton))			properties.cancelButton	= true;
		//if (this.not(properties.icon))					properties.icon = '/pic/exclame_e0.gif';
		if (this.not(properties.buttons))				properties.buttons = {};
		if (this.not(properties.closeOnOutsideClick))	properties.closeOnOutsideClick = false;
		*/

		$.extend(this.properties, properties);

	};

	this.addButton = function(key, caption) {

		if (this.properties) {
			if (!this.properties.buttons[key]) {
				this.properties.buttons[key] = caption;
			}
		}

	};

	this.getButtonHtml = function() {

		var html = '';
		if (this.properties.buttons) {
			$.each(this.properties.buttons, function(name, caption) {
				html += '<button type="button" name="' + name + '" class="jkqDialogButton">' + caption + '</button>';
			});
		}

		return html;

	};

	/**
	 * Dialog-Template
	 */
	this.getTemplate = function() {

		var autoSkipHtml = '';

		if (this.autoSkipEnabled()) {
			autoSkipHtml = (
					'<p class="jkqDialogAutoSkip">' +
						'<input type="checkbox" name="jkqDialogAutoSkip" id="jkqDialogAutoSkip"> ' +
						'<label for="jkqDialogAutoSkip">{0}</label>' +
					'</p>'
			).format( this.properties.autoSkipText );
		}

		var template = (
			'<div class="jkqWindow jk-round">' +
				'<div class="jkqWindowTitleBar">' +
					'<div class="jkqWindowClose" title="close"></div>' +
					'<div class="jkqWindowTitle">{properties.title}</div>' +
				'</div>' +
				'<div class="jkqDialogContent {properties.css_class}">' +
					'<p class="jkqDialogText">{properties.text}</p>' +
					'<p class="jkqDialogButtons">{buttons}</p>' +
					'{autoSkipHtml}' +
				'</div>' +
			'</div>'
		).format( {
			properties:	this.properties,
			buttons:	this.getButtonHtml(),
			autoSkipHtml:	autoSkipHtml,
		} );

		return template;

	};

	/**
	 * displays the Confirm-Dialog
	 *
	 * @param {Function}	arg1 (optional) Callback: function(button_name) not called on cancel
	 * @param {jqEvent}		arg2 (optional) needed for closeOnOutsideClick, if set
	 * @returns {jkqDialog}
	 */
	this.confirm = function(arg1, arg2) {

		var handler = null, event = null;
		if (typeof arg1 == 'function')	handler = arg1;
		if (typeof arg2 == 'function')	handler = arg2;
		if (typeof arg1 == 'object')	event = arg1;
		if (typeof arg2 == 'object')	event = arg2;

		if (
				this.autoSkipEnabled() &&
				amplify.store( 'jkq-dialog-autoskip_' + this.properties.autoSkipButton )
		) {

			if (handler) $.proxy(handler, self)(this.properties.autoSkipButton);

		} else {

			this.showDialog(null, handler, true);

			if ((event) && this.properties.closeOnOutsideClick) {
				this.closeOnOutsideClick(event);
			}

			$(this).triggerHandler('confirm');

		}

		return this;

	};

	/**
	 * zeigt den Dialog an
	 *
	 * @param {Event}		e		jQuery-Event
	 * @param {Function}	handler Callback der Form function(button_name) wird aufgerufen, außer bei cancel
	 * @param {Boolean}		modal	soll der Dialog modal angezeigt werden
	 * @returns {jkqDialog}
	 */
	this.showDialog = function(e, handler, modal) {

		this.template = this.getTemplate();

		this.show(e, modal);
		this.adjustPos();

		// Button-Handler
		var self = this;
		$(this.container).find('BUTTON').on('click', function (e) {

			e.stopPropagation();

			var button_name = $(this).attr('name');
			if (button_name != 'jkqDialogCancel') {
				if (handler) $.proxy(handler, self)(button_name);

				if (
						self.autoSkipEnabled() &&
						(button_name == self.properties.autoSkipButton) &&
						$(self.container).find('INPUT#jkqDialogAutoSkip').is(':checked')
				) {
					amplify.store( 'jkq-dialog-autoskip_' + self.properties.autoSkipButton, true );
				}
			}

			self.close();

		});

		return this;

	};

	/**
	 * adds a cancel handler
	 *
	 * @param {Function} onCancel Callback: function() called on 'cancel' or 'close'
	 * @returns {jkqDialog}
	 */
	this.cancel = function(onCancel) {

		var self = this;

		// replace prev cancel handler
		$(this.container).find('BUTTON[name="jkqDialogCancel"]')
		.off('click')
		.on('click', function (e) {

			if (onCancel) onCancel('jkqDialogCancel');

			self.close(e);

		});

		// replace prev close handler
		$(this.container).find('DIV.jkqWindowClose')
		.off('click')
		.on('click', function (e) {

			if (onCancel) onCancel('jkqWindowClose');

			self.close(e);

		});

		return this;

	};

	if (properties) {

		// jkqDialog is created directly ( not from a child object, e.g. jkqAlert)

		this.init(properties);

		// add cancel button
		if (this.properties.cancelButton) {
			this.addButton('jkqDialogCancel', 'cancel');
		}

	}

}

jkqAlert.prototype = new jkqDialog();
/**
 * Message-Window, like jkqDialog, with a predefined OK-Button <br>
 * call: new jkqAlert({ <br>
 * {String}	title:		window title <br>
 * {String}	text:		dialog text <br>
 * }).confirm([e]);
 *
 * @param {Object} properties
 * @returns {jkqDialog}
 */
function jkqAlert(properties) {

	if (this.not(properties.closeOnOutsideClick)) {
		properties.closeOnOutsideClick = true;
	}
	if (this.not(properties.buttons)) {
		properties.buttons = {jkqDialogOk: 'ok'};
	}
	this.init(properties);

	// add Ok Button
	//this.addButton('jkqDialogOk', 'ok');

}

jkqInputDialog.prototype = new jkqDialog();
/**
 * Input Dialog-Window, like jkqDialog, with a text input field <br>
 * call: new jkqInputDialog({ <br>
 *  title:		window title <br>
 *  text:		dialog text <br>
 *  inputValue: predefined value of the input field <br>
 * }).confirm();
 *
 * @param {Object} properties
 * @returns {jkqDialog}
 */
function jkqInputDialog(properties) {

	var self = this;

	/**
	 * Input Dialog-Template
	 */
	this.getTemplate = function() {

		var template =
			'<div class="jkqWindow jk-round">' +
				'<div class="jkqWindowTitleBar">' +
					'<div class="jkqWindowIcon"></div>' +
					'<div class="jkqWindowClose" title="close"></div>' +
					'<div class="jkqWindowTitle">' + this.properties.title + '</div>' +
				'</div>' +
				'<div class="jkqDialogContent ' + this.properties.css_class + '">' +
					'<p class="jkqDialogText">' +
						this.properties.text +
					'</p>' +
					'<p class="jkqDialogInput">' +
						'<input type="text" name="jkqInputDialogValue" value="' + this.properties.inputValue + '">' +
					'</p>' +
					'<p class="jkqDialogButtons">' +
						this.getButtonHtml() +
					'</p>' +
				'</div>' +
			'</div>';

		return template;

	};

	this.value = function() {
		return this.container.find('INPUT[name="jkqInputDialogValue"]').val();
	};

	if (!properties.inputValue)	properties.inputValue = '';
	this.init(properties);

	// add Ok Button
	this.addButton('jkqDialogOk', 'ok');
	// add cancel Button
	this.addButton('jkqDialogCancel', 'cancel');

	$(this).on('confirm', function() {

		// add enter => ok
		self.container.find('INPUT[name="jkqInputDialogValue"]').on('keyup', function(e) {

			if (e.which == 13) {
				self.container.find('BUTTON[name="jkqDialogOk"]').click();
			}

		}).focus().select();

	});

}

/**
 * Drag & Drop Object
 */

jkqDragDrop.prototype = {};
/**
 * Drag & drop Object
 *
 * @param {String}		drop_target_selector
 * @param {Function}	onDragDrop
 * 		callback: fn(target,source) called, if drag-Objekt is dropped on a drop-target
 * @param {Function}	onDragOver
 * 		null (default behavior),
 * 		or callback: fn(target, source, xOffs, yOffs):bool ,
 * 		called, if mouse is over a drop-Object
 * 		returnes if a source Object may be dropped
 * @param {Function}	onDragOut
 * 		null (default behavior),
 * 		oder callback: fn(target) called, if mouse leaves drop-Object
 * @returns {jkqDragDrop}
 */
function jkqDragDrop(drop_target_selector, onDragDrop, onDragOver, onDragOut) {

	var self = this;

	/**
	 * div-Element (with class "jkDrag"), contains a copy of the drag-source
	 * @returns {jQueryObject}
	 */
	this.$container = $();

	/**
	 * div-Element to display the drag-status
	 * @private
	 */
	this.$drag_status = null;

	/**
	 * drag-source
	 * @returns {jQueryObject}
	 */
	this.$source = null;

	/**
	 * current drop-target (if exists)
	 * @private
	 */
	this.target = null;

	/**
	 * registred drop-targets
	 * @private
	 */
	this.drop_targets = [];

	/**
	 * default behavior of onDragOver
	 */
	this.dragOver = function(target) {
		self.$container.addClass('jkDropable');
		$(target).addClass('jkDragOver');
		return true;
	};

	/**
	 * callback: fn(target, source, xOffs, yOffs):bool
	 * called, if mouse is over a drop-Object
	 * returnes if a source Object can be dropped
	 */
	if (typeof onDragOver == 'function') {
		// custom onDragOver function
		this.onDragOver = onDragOver;
	} else if (drop_target_selector) {
		// default onDragOver function
		this.onDragOver = this.dragOver;
	}

	/**
	 * default behavior of onDragOut
	 */
	this.dragOut = function(target) {
		self.$container.removeClass('jkDropable');
		$(target).removeClass('jkDragOver');
	};

	/**
	 * callback: fn(target)
	 * called, if mouse leaves drop-Object
	 */
	if (typeof onDragOut == 'function') {
		// custom onDragOut function
		this.onDragOut = onDragOut;
	} else if (drop_target_selector) {
		// default onDragOut function
		this.onDragOut = this.dragOut;
	}

	/**
	 * inserted area to catch moue movement
	 */
	this.drag_canvas = null;

	this.start_timer = null;

	/**
	 * will be set to $(drop_target_selector) in _start
	 */
	this.drop_targets = null;

	/**
	 * starts a drag-action with drag_source as element to drag
	 * (call it in a mousedown/touchstart event)
	 *
	 * @param {jQueryEvent} 	e
	 * @param {jQueryObject}	drag_source		HTML-Element
	 * @param {Number}			delay			delay, until drag-action starts, 0 = immediately
	 */
	this.start = function(e, drag_source, delay) {

		if (delay) {

			var self = this;

			this.start_timer = window.setTimeout(function() {
				$(drag_source).off('mouseup touchend');
				self._start(e, drag_source);
			}, delay);

			$(drag_source).one('mouseup touchend', function() {
				window.clearTimeout(self.start_timer);
			});

		} else {

			// start immediately
			this._start(e, drag_source);

		}

	};

	/**
	 * starts a drag-action with drag_source as element to drag immediately
	 * (call it in a onmousedown/ontouchstart)
	 *
	 * @param e				jQuery-Event
	 * @param drag_source	HTML-Element
	 */
	this._start = function(e, drag_source) {

		var self = this;

		this.$source = $(drag_source);

		this.drag_canvas = document.createElement('div');

		this.drag_canvas.id = 'drag_canvas';
		$(this.drag_canvas).css({
			'position' : 'absolute',
			'left' : "0px",
			'top'  : "0px",
			'width' : "100%",
			'height' : Math.max($('body').outerHeight(), $(window).height()) + "px",
			'backgroundColor' : "transparent",
			'backgroundImage' : 'none',
			'overflow' : 'hidden',
			'zIndex' : jkqWindowManager.zindex
		});

		$("BODY").append(this.drag_canvas);

		// prevent selection
		document.onselectstart = function() { return false; };

		if (drop_target_selector) {

			// if Drag&Drop
			this.drop_targets = $(drop_target_selector);

			// remember coordinates
			this.drop_targets.each(function () {

				var p = $(this).offset();

				this.rect = {
						'l'	: p.left,
						't'	: p.top,
						'r' : p.left + $(this).outerWidth(),
						'b' : p.top + $(this).outerHeight()
				};

			});

		}

		this.target = null;

		if (e.type == 'touchstart') {

			e.preventDefault(); // prevent scrolling in a scrollbox (FF)
			// convert coordinates for onDragStart
			var touchstart = e.originalEvent.touches[0] || e.originalEvent.changedTouches[0];
			e.pageX = touchstart.pageX;
			e.pageY = touchstart.pageY;

			$(document).on('touchmove', function (e) {

				e.preventDefault();

				var touchmove = e.originalEvent.touches[0] || e.originalEvent.changedTouches[0];

				$.proxy(self.dragmove, self)(touchmove.pageX, touchmove.pageY);

			}).on('touchend', function (e) {

				$(document).off('touchmove');
				$(document).off('touchend');

				self.dragend();

			});

		} else {

			$(this.drag_canvas).on('mousemove', function (e) {

				e.preventDefault();

				// prevent text selection while moving
				if (window.getSelection) {
					var sel = window.getSelection();
					sel.removeAllRanges();
				}

				$.proxy(self.dragmove, self)(e.pageX, e.pageY);

			}).on('mouseup', function(e) {

				// close drag-action
				self.dragend();

				// restore selection
				document.onselectstart = function() { return true; };

			});

		}

		// create container
		if (this.onDragStart) {
			$.proxy(this.onDragStart, this)(e, drag_source);
		}

	};

	/**
	 * processes the drag-move
	 *
	 * 'this' is the scope of the current dragndrop Object
	 */
	this.dragmove = function(pageX, pageY) {

		if (!this.$container) {
			this.target = null;
			this.dragend();
			return;
		}

		// drag an absolute positioned element
		this.$container.offset({
			left: (pageX + this.drag_x_rel),
			top	: (pageY + this.drag_y_rel)
		});

		$(this.$drag_status).offset({
			left: (pageX + 8),
			top	: (pageY + 8)
		});

		// drag only (move) ...
		if (drop_target_selector) {
			// ... or drag&drop

			var over_target = null;

			var self = this;
			this.drop_targets.each(function () {

				if (this != self.$source) {

					if (	(pageX > this.rect.l) &&
							(pageX < this.rect.r) &&
							(pageY > this.rect.t) &&
							(pageY < this.rect.b) )
					{
						// call dragover-function with dragndrop objects scope
						if ($.proxy(self.onDragOver, self)(this, self.$source, pageX - this.rect.l, pageY - this.rect.t)) {
							over_target = this;
							// break
							return false;
						}
					}

				}

			});

			if (over_target) {

				if ((this.target) && (this.target != over_target)) {
					this.onDragOut(this.target);
				}

				this.target = over_target;
				$(this.$drag_status).hide();

			} else {

				if (this.target) {
					this.onDragOut(this.target);
					this.target = null;
					$(this.$drag_status).show();
				}

			}

		}

		if (this.onDragMove) {
			var p = this.$container.position();
			$.proxy(this.onDragMove, this)(p.left, p.top, this.$container.outerWidth(), this.$container.outerHeight());
		}

	};

	/**
	 * processes the drag-end
	 */
	this.dragend = function() {

		if (this.target) {

			try {
				this.onDragOut(this.target);
				onDragDrop(this.target, this.$source);
			} catch (e) {
				// TODO: handle exception
			}
			this.target = null;

		}

		$(this.drag_canvas).remove();

		this.drag_canvas = null;
		this.$container = null;

	};

	/**
	 * default drag-start-action with drag_source as dragging element
	 *
	 * 'this' is the scope of the current dragndrop Object
	 *
	 * @param e				jQuery-Event
	 * @param drag_source	HTML-Element
	 */
	this.defaultDragStart = function(e, drag_source) {

		// remember relative position to mouse
		var p = this.$source.offset();
		this.drag_x_rel = p.left - e.pageX;
		this.drag_y_rel = p.top  - e.pageY;

		if (!drop_target_selector) {

			// drag only (move) ...
			this.$container = this.$source;

		} else {
			// ... or drag&drop

			// create container
			this.$container = $('<div />');

			if (this.$source.is('LI')) {
				// first the parent element
				//var p = this.$container.appendChild(this.$source.parentNode.cloneNode(false));
				//p.appendChild(this.$source.cloneNode(true));
			} else {
				this.$container.append(this.$source.clone(true));
			}

			this.$container
			.css({
				'position' : 'absolute',
				'left' : p.left + "px",
				'top' : p.top + "px",
				'width' : this.$source.outerWidth() + "px",
				'height' : 'auto',
				'opacity' : '0.6'
			})
			.addClass('jkDrag')
			.appendTo(this.drag_canvas)
			;

			// Drag-Status
			this.$drag_status = $('<i class="fa fa-ban" />')
			.css({
				position	: 'absolute',
				left		: (e.pageX + 8) + "px",
				top			:  (e.pageY + 8) + "px",
				width		: "16px",
				height		: '16px',
				color		: 'red',
				//'backgroundImage': 'url(no-drop.png)',
				//'backgroundRepeat': 'no-repeat'
			})
			.appendTo(this.drag_canvas)
			;

		}

	};

	/**
	 * callback: fn(event, drag_source) DragStart-action with drag_source as element to drag
	 *
	 * the callback is called with 'this' as the scope of the current dragndrop Object
	 *
	 * @param drag_source HTML-Element
	 */
	this.onDragStart = this.defaultDragStart;

	/**
	 * callback: fn(left, top)
	 *
	 * 'this' is the scope of the current dragndrop Object
	 */
	this.onDragMove = null;

	// distance of the drag-object to mouse position
	this.drag_x_rel = 0;
	this.drag_y_rel = 0;

}

jkqResizeLeft.prototype = new jkqDragDrop();
function jkqResizeLeft() {

	this.drag_org_right = 0;

	this.onDragStart = function(e) {

		// remember relative position to mouse and right position
		var p = this.$source.position();
		this.drag_x_rel = p.left - e.pageX;
		this.drag_org_right = p.left + this.$source.outerWidth();

	};

	/**
	 * processes the drag-move
	 * overrides jkqDragDrop.dragmove
	 *
	 * 'this' is the scope of the current dragndrop Object
	 */
	this.dragmove = function(x, y) {

		var left = (x + this.drag_x_rel);
		var width = Math.max(0, this.drag_org_right - left);
		this.$source.css({
			left	: left,
			width	: width,
		});

		if (this.onDragMove) {
			$.proxy(this.onDragMove, this)(left, 0, width, 0);
		}

	};

}

jkqResizeRight.prototype = new jkqDragDrop();
function jkqResizeRight() {

	this.onDragStart = function(e) {

		// remember relative position to mouse and left position
		var p = this.$source.position();
		this.drag_x_rel = p.left + this.$source.outerWidth() - e.pageX;

	};

	/**
	 * processes the drag-move
	 * overrides jkqDragDrop.dragmove
	 *
	 * 'this' is the scope of the current dragndrop Object
	 */
	this.dragmove = function(x, y) {

		var right = (x + this.drag_x_rel);
		var left = Math.min(right, this.$source.position().left);
		var width = right - left;
		this.$source.css({
			width	: width,
		});

		if (this.onDragMove) {
			$.proxy(this.onDragMove, this)(left, 0, width, 0);
		}

	};

}

// jQuery extensions
(function ( $ ) {

	/**
	 * checks, if a jQuery-Objekt reacts on a defined event
	 */
	$.fn.hasEvent = function( event ) {

		var result = false;

		var eventSplit = event.split('.');
		var eventName = eventSplit[0];

		if (
				($._data(this.get(0), 'events')) &&
				($._data(this.get(0), 'events')[eventName])
		) {

			var dataEvent = $._data(this.get(0), 'events')[eventName];

			if ((eventSplit[1]) && (dataEvent.length > 0)) {

				// if namespace given: compare
				result = (dataEvent[0].namespace == eventSplit[1]);

			} else {

				// else: eventhandler exists
				result = true;

			}

		}

		return result;

	};

}( jQuery ));

/**
 * String format extension
 * http://stackoverflow.com/questions/610406/javascript-equivalent-to-printf-string-format
 * Example:
 * "{0} is dead, but {1} is alive! {0} {2}".format("ASP", "ASP.NET")
 * outputs
 * ASP is dead, but ASP.NET is alive! ASP {2}
 *
 * JK: extended by keys through object members:
 * "His name is {firstname}, {lastname}.".format({firstname: "John", lastname: "Doe"});
 */
if (!String.prototype.format) {
	String.prototype.format = function() {
		var args = arguments;
		if ( (args.length == 1) && (typeof args[0] == 'object') ) {
			args = args[0];
		}
		return this.replace(/{([^\}]+)}/g, function(match, key) {

			var value = args;
			var keys = key.split('.');
			for( var i = 0; i < keys.length - 1; i++) {

				var ikey = keys[i];
				if ( typeof value[ikey] == 'undefined' ) {
					break;
				}
				value = value[ikey];
				key = keys[i + 1];

			}

			if ( typeof value[key] == 'function' ) {

				return value[key]();

			} else if ( typeof value[key] != 'undefined' ) {

				return value[key];

			}

			return match;

		});
	};
}
