function tmaFeature(name) {

	this.STATE_STOP = 0;
	this.STATE_RUN = 1;
	this.STATE_PAUSE = 2;

	var state = this.STATE_STOP;
	var nextObjIndex = 0;
	var handlerObjects = [];
	var handlerArgs = [];
	var deferred = null;

	this.name = name;
	this.result = undefined;

	/**
	 * add a modul handler to the chain
	 *
	 * @param {Object}		handlerObj
	 */
	this.addFeature = function(handlerObj) {

		handlerObjects.push(handlerObj);

	};

	this.start = function(args) {

		if (tma.debug) tma.log('featureChain.start(' + this.name + ')');

		if (tma.debug) tma.log('featureChain.start: typeof args=' + typeof(args));
		if (tma.debug && args) tma.log('featureChain.start: args.length=' + args.length);

		// building new array this way, because iPad1 has problems else
		handlerArgs = [];
		handlerArgs.push(this);
		if (args) {
			for (var i=0; i<args.length; i++) {
				handlerArgs.push(args[i]);
			}
		}

		if (tma.debug) tma.log('featureChain.start: typeof handlerArgs=' + typeof(handlerArgs));
		if (tma.debug) tma.log('featureChain.start: handlerArgs.length=' + handlerArgs.length);
		if (tma.debug)
			for (var n=0; n<handlerArgs.length; n++) {
				tma.log('featureChain.start: typeof handlerArgs[' + n + ']=' + typeof(handlerArgs[n]));
			}
		nextObjIndex = 0;
		state = this.STATE_RUN;
		deferred = null;

		return this.run();

	};

	/**
	 * stops the feature chain
	 *
	 * @param {mixed}	result	optional return value, especially for async return values
	 */
	this.stop = function(result) {

		nextObjIndex = 0;
		state = this.STATE_STOP;
		if (typeof result != 'undefined') {
			this.result = result;
		}
		if (deferred) {
			deferred.resolve(this.result);
		}

	};

	this.pause = function() {

		if (!deferred) {
			deferred = $.Deferred();
		}

		state = this.STATE_PAUSE;

	};

	this.run = function() {

		if (state == this.STATE_PAUSE) {
			// continue
			state = this.STATE_RUN;
		}

		while ( (nextObjIndex < handlerObjects.length) && (state == this.STATE_RUN) ) {

			if (tma.debug) tma.log('featureChain.run(' + this.name + ') [' + nextObjIndex + ']');

			var obj = handlerObjects[nextObjIndex];
			nextObjIndex++;
			var result = obj[this.name].apply(obj, handlerArgs);
			if (typeof result != 'undefined') {
				this.result = result;
			}

		}

		if (
				(state == this.STATE_RUN) &&
				(nextObjIndex == handlerObjects.length)
		) {
			nextObjIndex = 0;
			state = this.STATE_STOP;
			if (deferred) return;
		}

		if (!deferred) {
			if (state == this.STATE_STOP) {
				return this.result;
			}

			deferred = $.Deferred();
		}
		return deferred.promise();

	};

}
