/**
 * encapsulates EPG-Genre object of json response
 *
 * @param {TmaGenre}	genre
 */
function TmaGenre(genre) {

	var data = {
			genre_code	: "00",
			genre_value : 0,
			genre_text	: "",
			text_color	: "",
			background	: "",
	};

	// apply initial data
	if (genre) {
		$.extend(true, data, genre);

		var defaultBgr = '';
		if (tma.genres.returned_source == 'au') {
			switch (data.genre_value) {
				case 0x20:	defaultBgr = '#87caf3'; break;	//AU News
				case 0x30:	defaultBgr = '#fcc45a'; break;	//AU Entertainment
				case 0x40:	defaultBgr = '#f08787'; break;	//AU Sports
				case 0x50:	defaultBgr = '#e8e753'; break;	//AU children's
				case 0x90:	defaultBgr = '#ff22aa'; break;	//AU Educational
				case 0xC0:	defaultBgr = '#ff9999'; break;	//AU Comedy
				case 0xD0:	defaultBgr = '#b8eb58'; break;	//AU Drama
				case 0xE0:	defaultBgr = '#babafe'; break;	//AU Documentary
			}
		} else {
			switch (data.genre_value) {
				case 0x10:	defaultBgr = '#b8eb58'; break;	//ETSI Movie/Drama
				case 0x20:	defaultBgr = '#87caf3'; break;	//ETSI News/Current Affairs
				case 0x30:	defaultBgr = '#fcc45a'; break;	//ETSI Show/Game Show
				case 0x40:	defaultBgr = '#f08787'; break;	//ETSI Sports
				case 0x50:	defaultBgr = '#e8e753'; break;	//ETSI Children's/Youth Programmes
				//case 0x60:	defaultBgr = '#ff9999'; break;	//ETSI Music/Ballet/Dance
				case 0x70:	defaultBgr = '#80f6de'; break;	//ETSI Arts/Culture
				case 0x80:	defaultBgr = '#babafe'; break;	//ETSI Social/Political Issues/Economics
				case 0x90:	defaultBgr = '#ff22aa'; break;	//ETSI Education/Science/Factual Topics
				//case 0xA0:	defaultBgr = '#ff22aa'; break;	//ETSI Leisure Hobbies
			}
		}

		// read color settings
		data.text_color = tma.setting( 'gc_txt_' + data.genre_code, '' );
		data.background = tma.setting( 'gc_bgr_' + data.genre_code, defaultBgr );
		data.coloring	= tma.setting( 'gc_ena_' + data.genre_code, 'true' ) == 'true';

	}

	/**
	 * TODO: convert to Getters in future
	 */

	this.genre_code		= data.genre_code;
	this.genre_value	= data.genre_value;
	this.genre_text		= data.genre_text;

	this.text_color = function(set) {
		if (set !== undefined) {
			data.text_color = set;
			// save color settings
			tma.setSetting( 'gc_txt_' + data.genre_code, set );
		} else {
			return data.text_color;
		}
	};

	this.background = function(set) {
		if (set !== undefined) {
			data.background = set;
			// save color settings
			tma.setSetting( 'gc_bgr_' + data.genre_code, set );
		} else {
			return data.background;
		}
	};

	this.coloring = function(set) {
		if (set !== undefined) {
			data.coloring = (set === true);
			// save 'enable' settings
			tma.setSetting( 'gc_ena_' + data.genre_code, data.coloring );
		} else {
			return data.coloring;
		}
	};

	/**
	 * returns the style string, if genre-colouring is enabled
	 * if this is a sub-genre, and there are no colours set,
	 * the colours of the parent genre is returned
	 *
	 * @param {Boolean}	coloring	should genre colouring be used?
	 */
	this.style = function( coloring ) {
		var result = '';
		if ( coloring ) {
			result = this.styleString();
//			if (!result && ( data.genre_value & 15 )) {
//				// try parent genre
//				var genre = tma.genres.genre(data.genre_value & 240);
//				if (genre) {
//					result = genre.styleString();
//				}
//			}
		}
		return result;
	};

	/**
	 * creates the style string based on defined colours
	 * e.g. color:#000000;background-color:#FF0000;
	 */
	this.styleString = function() {
		var result = '';
		if (data.coloring && (data.text_color || data.background)) {
			if (data.text_color) {
				result += 'color:' + data.text_color + ';';
			} else if (data.background.match(/#[0-9a-fA-F]{6}/)) {
				/* calculate text colour if only background specified*/
				var r = parseInt(data.background.substr(1,2),16);
				var g = parseInt(data.background.substr(3,2),16);
				var b = parseInt(data.background.substr(5,2),16);
				// calculate luminance of the YIQ color space in percent
				var Y = ((r*299)+(g*587)+(b*114))/2550;
				result += 'color:' +( (Y >= 50) ? '#000' : '#fff' ) + ';';
			} else {
				/* Force black if no color specified*/
				result += 'color:#000;';
			}

			if (data.background) {
				result += 'background-color:' + data.background + ';';
			}
		} else if ( data.genre_value & 15 ) {
			// try parent genre
			var genre = tma.genres.genre(data.genre_value & 240);
			if (genre) {
				result = genre.styleString();
			}
		}

		return result;
	};

	/**
	 * get count of sub genres of this genre
	 */
	this.countSubGenres = function() {

		var subGenres = 0;

		if ( (data.genre_value & 0xF) === 0) {

			var genre_value = (data.genre_value & 0xF0) + 1;
			while (genre_value < (data.genre_value & 0xF0) + 16) {

				var genre = tma.genres.genre(genre_value);
				if (genre) {
					if (!genre.genre_text.match(/^Genre [0-9a-fA-F]{2}$/)) {
						subGenres++;
					}
				}

				genre_value++;
			}

		}

		return subGenres;

	};

	/**
	 * get all sub genres of this genre
	 */
	this.subGenres = function() {

		var subGenres = {};

		if ( (data.genre_value & 0xF) === 0) {

			var genre_value = (data.genre_value & 0xF0) + 1;
			while (genre_value < (data.genre_value & 0xF0) + 16) {

				var genre = tma.genres.genre(genre_value);
				if (genre) {
					if (!genre.genre_text.match(/^Genre [0-9a-fA-F]{2}$/)) {
						subGenres[genre_value] = genre;
					}
				}

				genre_value++;
			}

		}

		return subGenres;

	};

}
