/**
 * encapsulates json response object of gere list
 *
 * @param {TmaGenres}	genres
 */
function TmaGenres(genres) {

	var data = {
			count				: 0,
			timestamp			: "",
			content				: "epg_genre",
			requested_source	: "",
			returned_source		: "auto",
			requested_language	: "",
			genres				: {},
	};

	// apply initial data
	if (genres) {

		var list = genres.genres;
		delete genres.genres;

		$.extend(data, genres);

		if (list) {
			for (var i = 0; i < list.length; i++) {
				var genre = new TmaGenre(list[i]);
				data.genres[genre.genre_value] = genre;
			}
		}

	}

	/**
	 * TODO: convert to Getters in future
	 */

	this.count				= data.count;
	this.timestamp			= data.timestamp;
	this.content			= data.content;
	this.genres				= data.genres;
	this.requested_source	= data.requested_source;
	this.returned_source	= data.returned_source;
	this.requested_language	= data.requested_language;

	/**
	 * loops through all genres
	 *
	 * @param {Function}	callback	callback function will be called for each genre with
	 * 									parameters ({Number} genre_value, {TmaGenre} genre)
	 */
	this.eachGenre = function( callback ) {

		for (var index in data.genres ) {
			if (callback.apply( data.genres[index], [index, data.genres[index]]) === false) {
				break;
			}
		}
	};

	/**
	 * returns a genre by genre_value or genre_code (hex)
	 *
	 * @param {mixed}	genre_code	String is interpreted as genre_code (hex), number as genre_value
	 * @returns {TmaGenre}	or null
	 */
	this.genre = function( genre_code ) {

		var genre_value = 0;

		if ( typeof genre_code == 'string' ) {
			genre_value = parseInt(genre_code, 16);
		} else {
			genre_value = genre_code;
		}

		if (data.genres[genre_value]) {
			return data.genres[genre_value];
		}

		return null;

	};

	/**
	 * get all main genres
	 */
	this.getMainGenres = function() {

		var mainGenres = {};

		var genre_value = 16;
		while (genre_value < 256) {

			if (data.genres[genre_value]) {
				if (!data.genres[genre_value].genre_text.match(/^Genre [0-9a-fA-F]{2}$/)) {
					mainGenres[genre_value] = data.genres[genre_value];
				}
			}

			genre_value += 16;
		}

		return mainGenres;

	};

}
