function tmaModuleData() {

	var STATE_AVAILABLE		= 0;
	var STATE_CREATED		= 1;
	var STATE_INITIALISED	= 2;

	this.name = '';
	this.nameHex = '';
	this.status = STATE_AVAILABLE;
	this.manifest = {};

	/** @returns tmaModule */
	this.module = {};

	this.active = false;

	/**
	 * load the manifest of this module
	 *
	 * @returns {Promise}
	 */
	this.loadManifest = function( ) {

		var self = this;

		var deferred = $.Deferred();

		var url = this.path() + 'manifest.json';

		if ($.isEmptyObject(this.manifest)) {

			// load manifest from server
			$.ajax({
				url: url,
				dataType: 'json',
				cache: false
			}).done(function(manifest) {

				self.manifest = manifest;
				deferred.resolve(manifest);

			}).fail(function(xhr, errorType, error) {

				deferred.reject(xhr, errorType, error);

			});

		} else {

			// manifest exists
			deferred.resolve(self.manifest);

		}

		return deferred.promise().done(function(manifest) {

			self.active =
				!( self.manifest.requiredTapId ) ||
				tma.taps.find( self.manifest.requiredTapId );

		}).fail(function(xhr, errorType, error) {

			tma.log({
				title: errorType + " in tma.loadManifest()",
				details: {
					error: error,
					xhrReadyState: xhr.readyState,
					xhrStatus: xhr.status,
					parameter: '',
					module: self.name,
					url: url,
				}
			});

		});

	};

	/**
	 * check, if manifest of the module is loaded
	 * @returns {Boolean}
	 */
	this.isManifestLoaded = function() {
		return !$.isEmptyObject(this.manifest);
	};

	/**
	 * set status to created
	 */
	this.setCreated = function() {
		this.status = STATE_CREATED;
	};

	/**
	 * set status to initialised
	 */
	this.setInitialised = function() {
		this.status = STATE_INITIALISED;
	};

	/**
	 * check status created
	 * @returns {Boolean}
	 */
	this.isCreated = function() {
		return this.status === STATE_CREATED;
	};

	/**
	 * check status initialised
	 * @returns {Boolean}
	 */
	this.isInitialised = function() {
		return this.status === STATE_INITIALISED;
	};

	/**
	 * get the modules path
	 */
	this.path = function() {
		return modulesPath + this.name + '/';
	};

}
