function TmaStatus( status ) {

	// empty default status
	var data = {
			content: "",
			state: 0,
			substate: 0,
			current_chtype: 0,
			current_chnum: 0,
			current_logoid: "",
			totaldisk: 0,
			freedisk: 0,
			heapsize: 0,
			freeheapsize: 0,
			availheapsize: 0,

			playstatus: 0,
			playmode: 0,
			trickmode: 0,
			speed: 0,
			svctype: 0,
			svcnum: 0,
			logoid: "",
			duration: 0,
			durationsec: 0,
			currentblock: 0,
			totalblock: 0,
			filename: "",
			filename_hex: "",
			absolutefilename: "",

			recording: [],

			drive_count: 0,
			drives: [],

			timestamp: '',
	};

	// apply initial data
	if (status) {

		$.extend(true, data, status);

		for (var i = 0; i < status.recording.length; i++) {
			data.recording[i] = new TmaStatusRecording(status.recording[i]);
		}

		if (status.drives) {
			for (var j = 0; j < status.drives.length; j++) {
				data.drives[j] = new TmaStatusDrive(status.drives[j]);
			}
		}

	}

	/**
	 * TODO: convert to Getters in future
	 */

	this.content =			data.content;
	this.state =			data.state;
	this.substate =			data.substate;
	this.current_chtype =	data.current_chtype;
	this.current_chnum =	data.current_chnum;
	this.current_logoid =	data.current_logoid;
	this.totaldisk =		data.totaldisk;
	this.freedisk =			data.freedisk;
	this.heapsize =			data.heapsize;
	this.freeheapsize =		data.freeheapsize;
	this.availheapsize =	data.availheapsize;

	this.playstatus =		data.playstatus;
	this.playmode =			data.playmode;
	this.trickmode =		data.trickmode;
	this.speed =			data.speed;
	this.svctype =			data.svctype;
	this.svcnum =			data.svcnum;
	this.logoid =			data.logoid;
	this.duration =			data.duration;
	this.durationsec =		data.durationsec;
	this.currentblock =		data.currentblock;
	this.totalblock =		data.totalblock;

	this.filename = 		data.filename;
	this.filename_hex = 	data.filename_hex;

	/**
	 * get file name of the current playback .inf file (non-UTF Toppy safe)
	 *
	 * @returns {String}
	 */
	this.getFilename = function() {

		return (tma.config.utftoppy || !data.filename_hex) ?
				data.filename
				: tma.decodeHexname(data.filename_hex);

	};

	this.absolutefilename =	data.absolutefilename;

	this.recording_count =	function () { return data.recording.length; };
	this.recording = 		data.recording;

	/**
	 * get data of a specific recoding slot by slot number or by logoid
	 *
	 * @param {any} slot_or_logoid		if type is string, search recording by logoid, else by slot number
	 * @returns {TmaStatusRecording}	or false, if there is no data for requested slot
	 */
	this.getRecording = function( slot_or_logoid ) {

		if (typeof slot_or_logoid == 'string') {
			for (var i = 0; i < data.recording.length; i++) {
				if ( (data.recording[i].rectype) && (data.recording[i].logoid == slot_or_logoid) ) {
					return data.recording[i];
				}
			}
		} else if (data.recording[slot]) {
			return data.recording[slot];
		}

		return false;

	};

	/**
	 * get the number of active recordings
	 *
	 * @returns {Number}
	 */
	this.activeRecordingCount = function() {

		var result = 0;
		this.eachRecording(function(slot, recording) {
			result++;
		});
		return result;

	};

	/**
	 * loops through all recordings
	 *
	 * @param {Function}	callback	callback function will be called for each recording with parameters (nr, recording)
	 * @param {Boolean}		activeOnly	optional, default = true, return only active recording slots
	 */
	this.eachRecording = function( callback, activeOnly ) {

		for (var i = 0; i < data.recording.length; i++ ) {
			if ( (activeOnly === false) || (data.recording[i].rectype)) {
				if (callback.apply( data.recording[i], [i, data.recording[i]]) === false) {
					break;
				}
			}
		}
	};

	/**
	 * number of external drives
	 */
	this.drive_count =	data.drive_count;
	this.drives = 		data.drives;

	/**
	 * loops through all drives
	 *
	 * @param {Function} callback		callback function will be called for each drive with parameters (nr, drive)
	 */
	this.eachDrive = function( callback ) {
		for (var i = 0; i < data.drives.length; i++ ) {
			if (callback.apply( data.drives[i], [i, data.drives[i]]) === false) {
				break;
			}
		}
	};

	/**
	 * get data of a specific drive
	 *
	 * @param {Number} nr
	 * @returns {TmaStatusDrive}	or false, if there is no data for requested drive
	 */
	this.drive =		function( nr ) {
		if (data.drives[nr]) {
			return data.drives[nr];
		} else {
			return false;
		}
	};

	this.timestamp =	data.timestamp;

}

function TmaStatusRecording( recording ) {

	// empty default recording
	var data = {
			slot:			0,
			rectype:		0,
			tuner:			0,
			svctype:		0,
			svcnum:			0,
			logoid:			"",
			duration:		0,
			starttime:		"",
			endtime:		"",
			recordedsec:	0,
			filename_hex:	"",
			filename:		""
	};

	// apply initial data
	if (recording) $.extend(true, data, recording);

	/**
	 * TODO: convert to Getters in future
	 */

	this.slot =			data.slot;
	this.rectype =		data.rectype;
	this.tuner =		data.tuner;
	this.svctype =		data.svctype;
	this.svcnum =		data.svcnum;
	this.logoid =		data.logoid;
	this.duration =		data.duration;
	this.starttime =	data.starttime;
	this.endtime =		data.endtime;
	this.recordedsec =	data.recordedsec;

	/** @TODO encapsulate */
	this.filename = 	data.filename;
	this.filename_hex = data.filename_hex;

	/**
	 * get file name of this recording slot (non-UTF Toppy safe)
	 *
	 * @param {Boolean]	removeExt	remove file extension? (default = false)
	 * @returns {String}
	 */
	this.getFilename = function(removeExt) {

		/** @returns {String} */
		var name = (tma.config.utftoppy || !data.filename_hex) ?
				data.filename
				: tma.decodeHexname(data.filename_hex);

		if (removeExt) {
			return name.replace(/\.[^.]+$/, '');
		} else {
			return name;
		}

	};

}

TmaStatusDrive = function(drive) {

	// empty default drive
	var data = {
			name:			"",
			path:			"",
			mountpath:		"",
			partitiontype:	0,
			free_mb:		0,
			size_mb:		0
	};

	// apply initial data
	if (drive) $.extend(true, data, drive);

	/**
	 * TODO: convert to Getters in future
	 */

	this.name =			data.name;
	this.path =			data.path;
	this.mountpath =	data.mountpath;
	this.partitiontype =data.partitiontype;
	this.free_mb =		data.free_mb;
	this.size_mb =		data.size_mb;

};
