/**
 * Utilities:
 */

/**
 * returns the html string for event start/end/duration
 *
 * @param {Date}	event_start
 * @param {Date}	event_end
 * @param {Number}	event_duration		in minutes
 * @param {Number}	infile_duration		(optional) default = event_duration
 * @returns {String}
 */
function getEventDateString(event_start, event_end, event_duration, infile_duration) {
	
	var partial = '';
	if ((infile_duration) && (infile_duration != event_duration)) {
		partial = tma.formatDuration(infile_duration).replace(" min","") + '/';
	}

	if (!infile_duration) infile_duration = event_duration;

	var s = (
			'<h3>{start} - {end} &nbsp; {partial}{duration}</h3>'
	).format({
		start:		tma.date( tma.i18.eventstart_fmt, event_start),
		end:		tma.date( 'H:i', event_end),
		partial:	partial,
		duration:	tma.formatDuration(event_duration),		
	});

	return s;

}

/**
 * returns the html string for a genre and parental text
 *
 * @param {TmaEpgEvent}	event		event object
 * @param {Boolean}		coloring	should genre colouring be used?
 * @returns {String}
 */
function getGenreAndParentalString(event, coloring) {

	var s = '';
	/** @returns {TmaGenre} */
	var genre = null;
	
	if ((event.genre) && (typeof event.genre == 'function')) {
		genre = event.genre();
	} else if ((event.genre) && (event.genre_text)) {
		genre = new TmaGenre({
			genre_value:	0,
			genre_code:		"00",
			genre_text:		event.genre_text
		});
	} else {
		genre = new TmaGenre();
	}	

	if ( 
			((genre) && (genre.genre_text)) || 
			(event.parental_text)
	) {
		s =	(
				'<h4 genre-value="{genre.genre_value}" style="{style}">{genre.genre_text}{parental}</h4>'
		).format({
			genre:		genre,
			style:		(coloring) ? genre.style(true) : '',
			parental:	(event.parental_text) ? ' (' + event.parental_text + ')' : ''
		});
	}

	return s;

}
