/**
 * channels module
 */
tma.registerModule( channelspage );
channelspage.prototype = new tmaModule();
function channelspage() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'channelspage';

	this.pageGroup = 'channels';

	var self = this;

	/**
	 * initialize module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'appendMenuTab' );
		tma.fn.addFeature( this, 'openChannelInfo' );

		return this.loadLanguage().done(function() {

			// extend TmaChannel
			TmaChannel.prototype.typeStr = function() {
				switch (this.chtype) {
					case  1: return self.radio		|| 'Radio';
					case -1: return self.analogue	|| 'Analogue';
					default: return self.tv			|| 'TV';
				}
			};

		});

	};

	/**
	 * append tab to menu
	 *
	 * @param {tmaFeature}	f
	 */
	this.appendMenuTab = function( f ) {

		tma.call('addPage', ['MainMenu', this]);

	};

	/**
	 * Display the channels obtained from the PVR in the browser window
	 */
	this.show = function() {

		this.init().done(function() {

			$("#timestamp").text(new Date(tma.channelsObj.timestamp).toLocaleString() );

			var template = (
					'<div class="channels-frame" />'
			);

			var $frame = $(template).appendTo('DIV#content');

			// show channels list

			self.showChannels();

			// add event listener to select channel
			$frame.on('click', 'DIV.channel .channel-name, DIV.channel .channellogo', function(e) {
				// 'this' is the clicked element
				self.selectCurrentChannel( $(this).parent() );
			});

			// add event listener to open more-fn
			$frame.on('click', 'DIV.channel-fn > I', function(e) {

				var channel = tma.channel( $(this).parents('[channelid]').attr('channelid') );

				// remove previous moreFn's
				$('DIV.morefn').fadeOut(function() {
					$(this).remove();
				});

				self.i18.menu = {info: 'Info'};

				var $moreFn = $('<div class="morefn" />');

				// close morefn
				$('<i class="fa fa-close morefn-close" />').appendTo($moreFn)
				.click(function() {
					$(this).parent().fadeOut(function() {
						$(this).remove();
					});
				});

				// show channel infos
				$('<button class="jk-but info"><i class="fa fa-info" /><span>'	+ self.i18.menu.info +	'</span></button>')
				.appendTo($moreFn)
				.click(function() {
					if ( channel ) {
						tma.call('openChannelInfo', [channel]);
					}
					// remove moreFn
					$('DIV.morefn').remove();
				});

				tma.call('channelCreateMoreFn', [$moreFn, channel]);

				$moreFn.appendTo( $(this).parent() )
				.animate({opacity:1}, {
					start: function() {
						var offs = $(this).offset();
						var w = $(this).outerWidth();
						var ww = $(window).width();
						if (offs.left + w >= ww ) {
							$(this).addClass('morefn-right');
						}
					}
				});

			});

			tma.call('addConfigItem', ['MainMenu', self]);

			self.insertConfig();

		});

	};

	/**
	 * this page gets removed, so hide the channels config
	this.cleanup = function() {

	};
	 */

	/**
	 * selects a channel and marks the element with css class 'current-channel'
	 *
	 * @param {jQueryObject} $channel	channel element with attribute channelid
	 */
	this.selectCurrentChannel = function ($channel) {

		var channel = tma.channel( $channel.attr('channelid') );
		if ( channel ) {
			tma.apiChannelSet( channel.chtype, channel.chnum ).done(function() {

				// set new current channel mark
				$('.current-channel').removeClass('current-channel');
				$channel.addClass('current-channel');
				// keep status up to date
				self.updateStatus();

			}).fail(function(xhr, errorType, error) {

				if (errorType == 'tma-error') {
					new jkqAlert({
						title:	tma.i18.error.dialog_title,
						text:	tma.i18.error.channels.set
					}).confirm();
				}

			});
		}

	};

	/**
	 * get current channel and current recording activity and mark
	 */
	this.updateStatus = function() {

		/** @param {TmaStatus} status */
		tma.apiGetStatus().done(function(status) {

			$('DIV.channel[channelid]').removeClass('current-channel rec-channel');

			$('DIV.channel[channelid="' + status.current_logoid + '"]')
				.addClass('current-channel');

			/** @param {TmaStatusRecording} recording */
			status.eachRecording( function(nr, recording) {
				$('DIV.channel[channelid="' + recording.logoid + '"]')
						.addClass('rec-channel');
			});

		});

	};

	this.showChannels = function () {

		var favGroup = prefs.getFavGroup();

		var $template = $(self.templates.channels.render({
			favgroup:		favGroupName(favGroup),
			displaymode:	prefs.getDisplayMode(),
			channels:		tma.getChannels(favGroup),
		}));

		var channels = tma.getChannels(favGroup);

		//DMC filter AV channels
		$template.find('.channel[channelid="ffffffffffffff02"]').hide();
		$template.find('.channel[channelid="ffffffffffffff04"]').hide();

		$template.appendTo( $('DIV.channels-frame').empty() );

		// get current channel and current recording activity and mark
		this.updateStatus();

		// apply the channelsacross setting
		applyChannelsAcross();

	};

	var applyChannelsAcross = function() {

		// apply the channelsacross setting
		$('DIV.channels').css( { maxWidth: "" } );
		var ca = prefs.getChannelsAcross();
		if (ca) {
			var $channels = $('DIV.channel');
			if ($channels.length > 1) {
				var left1 = $channels.eq(0).position().left;
				var left2 = $channels.eq(1).position().left;
				var w = parseInt(1 + left2 - left1) * ca;
				$('DIV.channels').css( { maxWidth: w } );
			}
		}

	};

	/**
	 * open popup with channel informations
	 *
	 * @param {tmaFeature}	f
	 * @param {TmaChannel}	channel
	 */
	this.openChannelInfo = function(f, channel) {

		/**
		 * since this contains asynchronous calls, pause the chain
		 * and decide with the result of the call, whether to stop,
		 * or to continue
		 */
		f.pause();

		this.init().done(function() {

			var popup = new jkqWindow();
			popup.width = 400;

			popup.show();
			popup.setTitle( channel.servicename );
			popup.closeOnOutsideClick();

			popup.container.find('DIV.jkqWindowContent').append(

					self.templates.channelinfo.render(channel)

			).find('SPAN[hex]').each(function() {
				// convert values with attribute 'hex' to hex-string
				// toHex(): https://gist.github.com/rubycon/1699103
				var p = parseInt($(this).attr('hex'));
				var value = parseInt($(this).text());
				p = (!p) ? 0 : p;
				var n = (value >>> 0).toString(16);
				while(n.length < p) n = '0' + n;
				$(this).text('0x' + n);
			});

			popup.adjustPos({
				hAlign:	'right',
				effect:	'slide-rtl'
			});

			// return popup instance
			f.stop(popup);

		});

	};

	this.insertConfig = function () {

		$('DIV#config').append($(this.templates.config.render({
			config:			this.i18.config,
			favgroups:		tma.i18.favgroups,
			channelsacross:	prefs.getChannelsAcross(),
		})));

		// change channelsacross setting
		$('DIV#config DIV.cfg-columns *.less').click(function() {

			var columns = prefs.getChannelsAcross() - 1;
			if (columns >= 0) {
				prefs.setChannelsAcross(columns);
				applyChannelsAcross();
				$('DIV#config DIV.cfg-columns *.value').text(columns);
			}

		});
		$('DIV#config DIV.cfg-columns *.more').click(function() {

			var columns = prefs.getChannelsAcross() + 1;
			prefs.setChannelsAcross(columns);
			applyChannelsAcross();
			$('DIV#config DIV.cfg-columns *.value').text(columns);

		});

		var $fav = $('DIV#config SELECT[name="favourites"]');
		/** @param {TmaFavouritesGroup} group */
		tma.favourites.eachGroup( function(nr, group) {

			if (group.count !== 0) {
				//Only list groups with members
				$fav.append(
						'<option value="{name}">{getName} ({count})</option>'.format(group)
				);
			}

		});
		$fav.val( prefs.getFavGroup() );
		$fav.change(function() {

			prefs.setFavGroup( $(this).val() );
			self.showChannels();

		});

		$('DIV#config INPUT[name="logotext"]')
			.click(function() {

				prefs.setDisplayMode( $('DIV#config INPUT[name="logotext"]:checked').val() );
				self.showChannels();

			})
			.filter('[value="' +  prefs.getDisplayMode() + '"]' ).attr('checked', true);

	};

	/**
	 * get a readable fav group name of the pseudo groups tv and radio
	 *
	 * @return {String}
	 */
	var favGroupName = function(favGroup) {

		var result = '';

		if (favGroup) {
			if (favGroup == '__tv') {
				result = 'TV';
			} else if (favGroup == '__radio') {
				result = 'Radio';
			} else {
				var group = tma.favourites.getGroup(favGroup);
				if (group) {
					result = group.getName();
				}
			}
		}

		return result;

	};

	/* Settings */

	var prefs = {

			/**
			 * get the defaultfavourites setting
			 *
			 * @return {String}
			 */
			getFavGroup: function() {
					return tma.localSetting( 'defaultfavourites', '' );
			},

			/**
			 * set the defaultfavourites setting
			 *
			 * @param {String}	set		new fav. group setting
			 */
			setFavGroup: function(set) {
				tma.setLocalSetting( 'defaultfavourites', set );
			},

			/**
			 * get the displayMode setting
			 *
			 * @return {Number}		1: display logo
			 * 						2: display name
			 * 						3: display both
			 */
			getDisplayMode: function() {
				return tma.localSetting( 'channel_display', 3 );
			},

			/**
			 * set the displayMode setting
			 *
			 * @param {Number}	set		new displayMode setting
			 * 							1: display logo
			 * 							2: display name
			 * 							3: display both
			 */
			setDisplayMode: function(set) {
				if ((set >= 1) && (set <= 3)) {
					tma.setLocalSetting( 'channel_display', set );
				}
			},

			/**
			 * get the channelsacross setting
			 *
			 * @return {Number}
			 */
			getChannelsAcross: function() {
				return parseInt(tma.localSetting( 'channelsacross', 0 ));
			},

			/**
			 * set the channelsacross setting
			 *
			 * @param {Number}	set		new channelsacross setting
			 */
			setChannelsAcross: function(set) {
				tma.setLocalSetting( 'channelsacross', parseInt(set) );
			},
	};

}
