/**
 * config module
 */
tma.registerModule( configpage );
configpage.prototype = new tmaModule();
function configpage() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'configpage';

	this.pageGroup = 'config';

	var installation = false;

	var self = this;

	/**
	 * initialize module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'appendMenuTab' );

		// set font size from setting
		var fs = parseFloat( tma.localSetting('fontsize', $('BODY').css('fontSize')) );
		$('BODY').css('fontSize', fs + 'px');

		if (tma.fn.newModules.length) {
			installation = true;
			this.installNewModules();
		}

		return this.loadLanguage();

	};

	/**
	 * append tab to menu
	 *
	 * @param {tmaFeature}	f
	 */
	this.appendMenuTab = function( f ) {

		tma.call('addPage', ['MainMenu', this]);

	};

	/**
	 * Display the channels obtained from the PVR in the browser window
	 */
	this.show = function() {

		this.init().done(function() {

			$('#config').addClass('globals-config');
			if (!installation) {
				self.insertConfig();
				$('#config').show();
			}

			$('#timestamp').hide();

		});

	};

	/**
	 * this page gets removed, so remove config class
	 */
	this.cleanup = function() {

		$('#config').removeClass('globals-config');
		$('#timestamp').show();

	};

	/**
	 * ensures, that a modules manifest and language is available
	 *
	 * @param	{tmaModuleData}	moduleData
	 * @param	{Function}		onLanguageAvailable		callback function()
	 */
	function getModuleLanguage(moduleData, onLanguageAvailable) {

		if (
				// module is in use
				moduleData.isInitialised() ||
				// module is not in use, but created
				( moduleData.isCreated() && moduleData.isManifestLoaded() )
		) {
			moduleData.module.loadLanguage().done(function() {
				onLanguageAvailable();
			});
		} else {
			// module is not in use and not created
			moduleData.loadManifest().done(function() {
				tma.fn.loadJs( moduleData.path() + moduleData.name + '.js' );
				$(moduleData).on('tmaModuleCreated', function(e, module) {
					module.loadLanguage().done(function() {
						onLanguageAvailable();
					});
				});
			});
		}

	}

	/**
	 * displays a popup with a module description
	 *
	 * @param {String}	modulename
	 */
	function showModuleInfo(modulename) {

		var moduleData = tma.fn.getModuleData(modulename);
		if (
				(moduleData) &&
				(moduleData.module.i18.module.info)
		) {

			self.getModuleDescription(moduleData).done(function(info) {

				var infoWindow = new jkqWindow();
				infoWindow.width = 450;

				infoWindow.showModal();
				infoWindow.setTitle( moduleData.module.i18.module.title );
				infoWindow.closeOnOutsideClick();

				infoWindow.container.find('DIV.jkqWindowContent')
				.append(
						'<div class="module-info">' +
						info +
						'</div>'
				);

				infoWindow.adjustPos();

			});

		}

	}

	this.insertConfig = function () {

		var template =
			'<div class="setting language-mode">' +
				'<span class="label">' + this.i18.language_mode + '</span>' +
				'<select name="language-mode" size="1">' +
					'<option value="pvr">' + this.i18.language_mode_pvr + ' (' + tma.config.language_code + ')</option>' +
				'</select>' +
				'<button class="button-save jk-but iconset ic-save"	title="' + self.i18.save_language + '"></button>' +
			'</div>' +
			'<div class="setting fontsize">' +
				'<span class="label">' + this.i18.font_size + '</span>' +
				'<button class="lessmore less" title="' + this.i18.dec_font_size + '">&#x25C0;</button>' +
				'<button class="lessmore more" title="' + this.i18.inc_font_size + '">&#x25B6;</button>' +
			'</div>' +
			'<div class="setting logos-aspect-ratio">' +
				'<span class="label">' + this.i18.logo_aspect_ratio + '</span>' +
				'<select name="logo-aspect-ratio" size="1">' +
					'<option value="54">54x34</option>' +
					'<option value="60">60x34</option>' +
					'<option value="78">78x34</option>' +
				'</select>' +
			'</div>' +
			//DMC Format durations
			'<div class="setting duration-format">' +
				'<span class="label">' + this.i18.duration_format + '</span>' +
				'<select name="duration-format" size="1">' +
					'<option value="0">1:23</option>' +
					'<option value="1">83 min</option>' +
				'</select>' +
			'</div>' +
			'<div class="setting reset-confirm-dialogs">' +
				'<span class="label">' + self.i18.confirm_dialogs + '</span>' +
				'<button class="button-dialog-reset jk-but" disabled>' + self.i18.reset_confirm_dialogs + '</button>' +
			'</div>' +

			'<div class="setting modules">' +
				'<span class="label">' + this.i18.modules + '</span>' +
				'<div class="modules-config-frame">' +
					'<div class="modules-frame used-modules">' +
						'<h1>' + this.i18.selected_modules + '</h1>' +
						'<div class="modules-list" />' +
					'</div>' +
					'<div class="module-buttons">' +
						'<button class="jk-but iconset ic-right"	disabled title="' + self.i18.remove_tooltip	+ '"></button>' +
						'<button class="jk-but iconset ic-up"		disabled title="' + self.i18.up_tooltip		+ '"></button>' +
						'<button class="jk-but iconset ic-down"		disabled title="' + self.i18.down_tooltip	+ '"></button>' +
						'<button class="jk-but iconset ic-left"		disabled title="' + self.i18.add_tooltip	+ '"></button>' +
						'<button class="jk-but iconset ic-save"				 title="' + self.i18.save_modules	+ '"></button>' +
						'<button class="jk-but iconset ic-home"		disabled title="' + self.i18.home_tooltip	+ '"></button>' +
						'<button class="jk-but iconset ic-reset"			 title="' + self.i18.reset_modules	+ '"></button>' +
					'</div>' +
					'<div class="modules-frame available-modules">' +
						'<h1>' + this.i18.available_modules + '</h1>' +
						'<div class="modules-list" />' +
					'</div>' +
				'</div>' +
			'</div>';

		$('DIV#config').append(template);

		/* modules config (device - all modules) */

		var moduleTemplate =
			'<div class="module jk-list-row" module="{name}" data-type="{moduletype}">' +
				'<input type="checkbox">' +
				'<span class="module-name">{caption}</span>' +
				'<button class="jk-but iconset ic-info" title="{info_tooltip}"></button>' +
			'</div>';

		// fill "available modules"
		var availableModules = tma.fn.getModules();
		$.each(availableModules, function( i, name ) {

			var moduleData = tma.fn.getModuleData(name);
			if (moduleData.manifest) {

				var $md = $(
						moduleTemplate.format({
							name:			moduleData.name,
							caption:		moduleData.name,
							moduletype:		moduleData.manifest.type,
							info_tooltip:	self.i18.info_tooltip,
						})
				);

				$md.appendTo('DIV#config DIV.available-modules DIV.modules-list');

			}

		});

		// fill "used modules"
		var usedModules = tma.modulesToLoad();
		$.each(usedModules, function( i, name ) {

			var moduleData = tma.fn.getModuleData(name);
			if (moduleData) {

				var $md = $(
						moduleTemplate.format({
							name:			moduleData.name,
							caption:		moduleData.name,
							moduletype:		moduleData.manifest.type,
							info_tooltip:	self.i18.info_tooltip,
						})
				);
				$md.appendTo('DIV#config DIV.used-modules DIV.modules-list');

				$('.available-modules .module[module="' + moduleData.name + '"]').hide();

			}

		});

		// load missing languages
		$.each(availableModules, function( i, name ) {

			var moduleData = tma.fn.getModuleData(name);
			if (moduleData.manifest) {

				getModuleLanguage(moduleData, function() {

					var $mds = $('.module[module="' + moduleData.name + '"]');
					$mds.find('SPAN').text(moduleData.module.i18.module.title);
					$mds.attr('data-type', moduleData.manifest.type);
					if (moduleData.module.i18.module.info) $mds.find('BUTTON.ic-info').show();

				});

			}

		});

		// set current startpage
		var startpage = tma.localSetting('startpage', 'none');
		$('DIV#config DIV.used-modules DIV.modules-list DIV.module[module="' + startpage + '"]').addClass('start-module');

		$('DIV#config DIV.modules-list').on('click', 'DIV.module', function(e) {

			// click on div.module works like a <select> list with multiple selections
			if (!e.metaKey && !e.ctrlKey) {

				$('DIV#config DIV.modules-list DIV.module-selected')
				.removeClass('module-selected')
				.find('INPUT').prop('checked', false);

			}

			$(this).find('INPUT').click();

		});

		$('DIV#config DIV.used-modules DIV.modules-list').on('click', 'DIV.module INPUT', function(e) {

			// click on used module deselects all available modules
			$('DIV#config DIV.available-modules DIV.module-selected').removeClass('module-selected');
			$('DIV#config DIV.available-modules INPUT:checked').prop('checked', false);

		});

		$('DIV#config DIV.available-modules DIV.modules-list').on('click', 'DIV.module INPUT', function(e) {

			// click on available module deselects all used modules
			$('DIV#config DIV.used-modules DIV.module-selected').removeClass('module-selected');
			$('DIV#config DIV.used-modules INPUT:checked').prop('checked', false);

		});

		$('DIV#config DIV.modules-list').on('click', 'DIV.module INPUT', function(e) {

			e.stopPropagation();

			// a module was (de-)selected
			$(this).parents('DIV.module').toggleClass('module-selected');

			var isUsedModule = $(this).parents('DIV.modules-frame').hasClass('used-modules');
			var $sm = $(this).parents('DIV.modules-list').find('DIV.module-selected');

			// set buttons visibility
			var $buttons = $('DIV#config DIV.module-buttons');
			$buttons.find('BUTTON.ic-right'	).attr( 'disabled', ($sm.length === 0) || !isUsedModule );
			$buttons.find('BUTTON.ic-left'	).attr( 'disabled', ($sm.length === 0) || isUsedModule );
			$buttons.find('BUTTON.ic-up'	).attr( 'disabled', ($sm.length === 0) || !isUsedModule || $sm.first().is(':first-child') );
			$buttons.find('BUTTON.ic-down'	).attr( 'disabled', ($sm.length === 0) || !isUsedModule || $sm.last().is(':last-child') );
			$buttons.find('BUTTON.ic-home'	).attr( 'disabled', ($sm.length !=  1) || !isUsedModule || !$sm.first().is('[data-type="page"]') );

		});

		// remove button
		$('DIV#config DIV.module-buttons BUTTON.ic-right').click(function() {

			// get all selected modules in 'used modules'
			var $selected = $('DIV#config DIV.used-modules DIV.module-selected');

			// show them in 'available modules'
			$selected.each(function() {

				var name = $(this).attr('module');
				$('DIV#config DIV.available-modules DIV.module[module="' + name + '"]').show();

			});

			$selected.remove();

			// update button's state
			$(this).siblings('.ic-up').attr( 'disabled', true );
			$(this).siblings('.ic-down').attr( 'disabled', true );
			$(this).siblings('.ic-right').attr( 'disabled', true );
			$(this).siblings('.ic-left').attr( 'disabled', true );

		});

		// add button
		$('DIV#config DIV.module-buttons BUTTON.ic-left').click(function() {

			// get all selected modules in 'available modules'
			var $selectedAvail = $('DIV#config DIV.available-modules DIV.module-selected');

			// add them to 'used modules'
			$selectedAvail.each(function() {

				$('DIV#config DIV.used-modules DIV.modules-list').append( $(this ).clone());

			});

			$selectedAvail.removeClass('module-selected').hide().find('INPUT').attr('checked', false);

			// update button's state
			var $selectedUsed = $('DIV#config DIV.used-modules DIV.module-selected');
			$(this).siblings('.ic-up').attr( 'disabled', $selectedUsed.first().is( ':first-child' ) );
			$(this).siblings('.ic-down').attr( 'disabled', $selectedUsed.last().is( ':last-child' ) );
			$(this).siblings('.ic-right').attr( 'disabled', false );
			$(this).siblings('.ic-left').attr( 'disabled', true );


		});

		// up button
		$('DIV#config DIV.module-buttons BUTTON.ic-up').click(function() {

			// get all selected modules
			var $selected = $('DIV#config DIV.used-modules DIV.modules-list DIV.module-selected');

			// move them one positiop up
			$selected.each(function() {

				$(this).prev().before( $(this) );

			});

			// update button's state
			$(this).attr( 'disabled', $selected.first().is( ':first-child' ) );
			$(this).siblings('.ic-down').attr( 'disabled', false );

		});

		// down button
		$('DIV#config DIV.module-buttons BUTTON.ic-down').click(function() {

			// get all selected modules
			var $selected = $('DIV#config DIV.used-modules DIV.modules-list DIV.module-selected');

			// move them one positiop down
			$selected.each(function() {

				$(this).next().after( $(this) );

			});

			// update button's state
			$(this).attr( 'disabled', $selected.last().is( ':last-child' ) );
			$(this).siblings('.ic-up').attr( 'disabled', false );

		});

		// home button
		$('DIV#config DIV.module-buttons BUTTON.ic-home').click(function() {

			// get first selected page module
			var $selected = $('DIV#config DIV.used-modules DIV.modules-list DIV.module-selected[data-type="page"]').first();

			if ( $selected.length ) {

				if ( $selected.hasClass('start-module') ) {
					$selected.removeClass('start-module');
				} else {

					// remove previous home
					$('DIV#config DIV.used-modules DIV.modules-list DIV.start-module').removeClass('start-module');
					// set home
					$selected.addClass('start-module');

				}

			}

		});

		// save button
		$('DIV#config DIV.module-buttons BUTTON.ic-save').click(function() {

			// get all 'used modules'
			var $selected = $('DIV#config DIV.used-modules DIV.modules-list DIV.module');
			// collect and save used module names
			var usedList = [];
			$selected.each(function() {
				usedList.push( $(this).attr('module') );
			});

			tma.setLocalSetting( 'modules', usedList.join(',') );

			// start page
			var home = $('DIV#config DIV.used-modules DIV.modules-list DIV.start-module').attr('module');
			if (home) {
				tma.setLocalSetting( 'startpage', home );
			} else {
				tma.setLocalSetting( 'startpage', null );
			}

			window.location.hash = '';
			window.location.reload();

		});

		// reset button
		$('DIV#config DIV.module-buttons BUTTON.ic-reset').click(function() {

			new jkqDialog({
				title: self.i18.reset_confirm_caption,
				text: self.i18.reset_confirm_text,
				buttons: {
					ok:				 tma.i18.respond_yes,
					jkqDialogCancel: tma.i18.respond_cancel
				}
			}).confirm(function(button) {

				if (button == 'ok') {

					location.replace('reset-modules.html');

				}

			});
		});

		// info button
		$('DIV#config DIV.modules-list BUTTON.ic-info').click(function(e) {

			e.stopPropagation();

			var modulename = $(this).parents('DIV.module').attr('module');
			showModuleInfo(modulename);

		});

		$('DIV#config SELECT[name="duration-format"]')		//DMC format duration
		.val( tma.localSetting('durationformat', 0) )
		.change(function() {

			tma.setLocalSetting( 'durationformat', $(this).val() );

		});


		/* logo aspect ratio setting (PVR - all modules) */

		if (tma.logos) {

			// change logos aspect ratio setting
			$('DIV#config SELECT[name="logo-aspect-ratio"]')
			.val( tma.setting('logodisplaywidth', 54) )
			.change(function() {

				tma.setSetting( 'logodisplaywidth', $(this).val() );
				tma.refreshLogoStyles();

			});

		} else {

			// if logos aren't available, hide logos aspect ratio setting
			$('DIV#config DIV.logos-aspect-ratio').hide();

		}

		/* font-size setting (device - all modules) */

		$('DIV#config DIV.fontsize .less').click(function() {

			var fs = parseInt( tma.localSetting('fontsize', $('BODY').css('fontSize')) );

			if (fs > 5) {
				fs--;
				$('BODY').css('fontSize', fs + 'px');
				tma.setLocalSetting('fontsize', fs);
				$(window).resize();
				$('#body').css('margin-top', $('#head').outerHeight());
			}

		});
		$('DIV#config DIV.fontsize .more').click(function() {

			var fs = parseFloat( tma.localSetting('fontsize', $('BODY').css('fontSize')) );

			fs++;
			$('BODY').css('fontSize', fs + 'px');
			tma.setLocalSetting('fontsize', fs);
			$(window).resize();
			$('#body').css('margin-top', $('#head').outerHeight());

		});

		/* language mode setting (local - all modules) */

		// add languages
		if (this.i18.languages) {
			$.each(this.i18.languages, function(lng, title) {

				$('DIV#config SELECT[name="language-mode"]').append(
						'<option value="' + lng + '">' + title + ' (' + lng + ')</option>'
				);

			});
		}

		// save language button
		$('DIV#config DIV.language-mode .button-save').click(function() {

			tma.setLocalSetting( 'languagemode', $('DIV#config SELECT[name="language-mode"]').val() );

			window.location.reload();

		});

		// reset confirm dialogs
		$('DIV#config DIV.reset-confirm-dialogs BUTTON').click(function() {

			var key;
			var data = amplify.store();
			for ( key in data ) {
				if (key.indexOf('jkq-dialog-autoskip_') === 0) {
					$('DIV#config DIV.reset-confirm-dialogs BUTTON').removeAttr("disabled");
					amplify.store(key, null);
				}
			}
			$('DIV#config DIV.reset-confirm-dialogs BUTTON').attr("disabled", true);

		});

		// apply current setting
		$('DIV#config SELECT[name="language-mode"]')
		.val( tma.localSetting('languagemode', 'pvr') );

		var key;
		var data = amplify.store();
		for ( key in data ) {
			if (key.indexOf('jkq-dialog-autoskip_') === 0) {
				// there is at least one dialog to reset
				$('DIV#config DIV.reset-confirm-dialogs BUTTON').removeAttr("disabled");
				break;
			}
		}

	};

	var descriptionsCache = {};

	/**
	 * reads the module description
	 *
	 * @param {tmaModuleData} moduleData
	 */
	this.getModuleDescription = function( moduleData ) {

		var deferred = $.Deferred();
		var pipe = deferred;

		if (
				(moduleData) &&
				(moduleData.module.i18.module.info)
		) {

			if ( moduleData.module.i18.module.info.match( /\.html$/ ) ) {

				// moduleData.module.i18.module.info is a filename
				var filename = moduleData.path() + 'lng/' + moduleData.module.i18.module.info;

				if (!descriptionsCache[moduleData.name]) {

					// load module description
					$.get(filename).done(function(data) {

						descriptionsCache[moduleData.name] = data;
						deferred.resolve(descriptionsCache[moduleData.name]);

					});

				} else {

					deferred.resolve(descriptionsCache[moduleData.name]);

				}

			} else {

				// moduleData.module.i18.module.info is a text string
				descriptionsCache[moduleData.name] = moduleData.module.i18.module.info;
				deferred.resolve(descriptionsCache[moduleData.name]);

			}

		} else {

			deferred.resolve('');

		}

		return pipe.promise();

	};

	this.installNewModules = function() {

		this.init().done(function() {

			var popup = new jkqWindow();

			popup.showModal();
			popup.setTitle( self.i18.install.title ); //
			$(popup).on('close', function() {
				installation = false;
				if ( $('#config').hasClass('globals-config') ) {
					self.insertConfig();
					$('#config').show();
				}
			});

			var $content = popup.container.find('DIV.jkqWindowContent');

			var template = (
					'<div class="install-popup">' +
						'<p>{text}</p>' +
						'<div class="modules-frame new-modules">' +
							'<div class="modules-list" />' +
						'</div>' +
						'<p class="buttons">' +
							'<button class="jk-but" fn="install" disabled>{install}</button> ' +
							'<button class="jk-but" fn="close">{close}</button> ' +
						'</p>' +
					'</div>'
			).format( self.i18.install );
			$content.append(template);

			var moduleTemplate =
				'<div class="module jk-list-row" module="{name}" data-type="">' +
					'<input type="checkbox">' +
					'<span class="module-name">{caption}</span>' +
					'<button class="jk-but iconset ic-info" title="{info_tooltip}"></button>' +
				'</div>';

			$.each(tma.fn.newModules, function( i, name ) {

				var moduleData = tma.fn.getModuleData(name);
				if (moduleData.manifest) {

					var $md = $(
							moduleTemplate.format({
								name:			moduleData.name,
								caption:		moduleData.name,
								info_tooltip:	self.i18.info_tooltip,
							})
					);

					$content.find('DIV.modules-list')
					.append($md);

					getModuleLanguage(moduleData, function() {

						$md.find('SPAN').text(moduleData.module.i18.module.title);
						$md.attr('data-type', moduleData.manifest.type);
						if (moduleData.module.i18.module.info) $md.find('BUTTON.ic-info').show();

					});

				}

			});

			// info button
			$content.on('click', 'BUTTON.ic-info', function(e) {

				var modulename = $(this).parents('DIV.module').attr('module');
				showModuleInfo(modulename);

			});

			// (de-)select module by checkbox
			$content.on('click', 'DIV.module INPUT', function(e) {

				$(this).parents('DIV.module').toggleClass('module-selected');

				// en-/disable install button
				$content.find('BUTTON[fn="install"]').prop("disabled", $content.find('INPUT:checked').length === 0);

			});

			// (de-)select module by click on name
			$content.on('click', 'DIV.module SPAN', function(e) {

				$(this).parents('DIV.module').find('INPUT').click();

			});

			// button close
			$content.on('click', 'BUTTON[fn="close"]', function(e) {

				popup.close();

			});

			// button install
			$content.on('click', 'BUTTON[fn="install"]', function(e) {

				var usedModules = tma.modulesToLoad();
				$content.find('INPUT:checked').each(function() {
					var modulename = $(this).parents('DIV.module').attr('module');
					usedModules.push(modulename);
				});
				tma.setLocalSetting( 'modules', usedModules.join(',') );

				window.location.reload();

			});

			popup.adjustPos();

		});

	};

}
