/**
 * dashboard module
 */
tma.registerModule( dashboard );
dashboard.prototype = new tmaModule();
function dashboard() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'dashboard';

	this.pageGroup = 'control';

	var self = this;

	this.statusInterval = null;

	//var statusInitialised = 0;

	var seedEPG = '<tr id="chepg"><td>&nbsp;</td></tr>';
	var savedEPG = seedEPG;

    var PLAYMODE_None = 0;             	// No playback started
    var PLAYMODE_Playing = 2;       	// Normal Playback
    var PLAYMODE_TempPlaying = 3;       // Time shifting
    var PLAYMODE_RecPlaying = 4;        // Time shifting in recording service.
    var PLAYMODE_Mp3 = 5;               // MP3
	var PLAYMODE_Media = 8;				// Media file

	/**
	 * initialize module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'appendMenuTab' );

		return this.loadLanguage();

	};

	/**
	 * append tab to menu
	 *
	 * @param {tmaFeature}	f
	 */
	this.appendMenuTab = function( f ) {

		tma.call('addPage', ['MainMenu', this]);

	};

	/**
	 * Display the channels obtained from the PVR in the browser window
	 */
	this.show = function() {

		this.init().done(function() {

			$('#timestamp').hide();

			var template = (
				'<div class="status">' +
						'<div class="status-current"></div>' +
						'<div class="status-rec-items"></div>' +
						'<div class="status-timer-items"></div>' +
				'</div>'
			);

			$(template)
			.appendTo('DIV#content');
//			.find('A')
//			.click(function(e) {
//				e.preventDefault();
//				window.open(this.href);
//			});

			self.renderStatus();

		});

	};

	/**
	 * this page gets removed, so remove config class
	 */
	this.cleanup = function() {

		if (self.statusInterval) {
			window.clearTimeout(self.statusInterval);
			self.statusInterval = null;
		}

		$('#timestamp').show();

	};

	/**
	 * render table of current action
	 *
	 * @param {TmaStatus} status
	 */
	this.renderCurrent = function(status) {

		//=======================================================================================
		//							CURRENT CHANNEL / PLAYBACK
		//=======================================================================================
		//Work out the playing status

		var playStatus = 'Unknown';
		var playDetails = 'Unknown';

		playStatus = self.i18.playstatus[status.playmode].name;    //Current state description, "Live TV", "Normal Playback", etc.

		//DONE - When time-shifting, do not show the file name.
		//DONE - Option to start recording the current channel.
		//TODO - When stop on time-shifting, screen does blank and does not return to live TV.

		var $table = $('<table class="current-status jk-list jk-round"><tbody /></table>');
		var $head = $('<thead><tr><th colspan="2">' + playStatus + ' ' + status.timestamp.slice(-8) + ' </th></tr></thead>').prependTo($table);
		var $tbody = $table.find('TBODY');

		var channel = tma.channel(status.current_logoid);

		var recButton = '';

		//=======================================================================================
		//							CURRENT CHANNEL
		//=======================================================================================
		if (status.playmode === PLAYMODE_None)	//Watching live TV, so show the channel info
		{
			//TODO - For each channel in the dropdown, show the current EPG entry.
			//$tbody.append('<tr><td>' + channel.servicename + '</td><td>[Info] [Prev] [Next] [Rec] [Pause]</td></tr>');

			// jk: create channel selector (see /moduleutils/js/chselector.js included via manifest)
			$chselector = createChannelSelector(
					// blacklist
					[
					 "ffffffffffffff00"
					 ]
			);
			var $row = $('<tr id="chselect"><td colspan="2"></td></tr>');
			$row.find('TD').append($chselector);
			$tbody.append($row);

			// jk: insert this row, when data is available
			//$tbody.append('<tr id="chepg"><td>&nbsp;</td></tr>');
			$tbody.append(savedEPG);

			//$tbody.append('<tr><td colspan="2">[Info] [Prev] [Next] [Rec] [Pause]</td></tr>');


			// jk: find the hidden select element
			var $select = $chselector.find('SELECT[name="channels"]');
			// jk: set current channel (set value and trigger "change" to update the component)
			$select.val(status.current_logoid).change();
			// jk: switch channel on selection change
			$select.change(function() {

				var logoid = $(this).val();

				var ch = tma.channel( logoid );
				if ( ch ) {
					var savedEPG = seedEPG;
					tma.apiChannelSet( ch.chtype, ch.chnum ).done(function() {
						self.renderStatus();
					}).fail(function(xhr, errorType, error) {
						self.renderStatus();
					});
				}
			});  //End channel selector change

			recButton = '<button rcu="record"  class="transport jk-but iconset ic-rec"></button>&nbsp;&nbsp;';

			//TODO - Sometimes no EPG data is returned even when EPG data is on the PVR. TAP problem?  Often isolated to certain channels.
			//Load the current EPG event for the current channel
			if(channel)
			{
				var	dateFrom = new Date(status.timestamp);

				tma.apiGetEpg(
						channel,
						tma.toPvrDate(dateFrom),
						tma.toPvrDate(dateFrom)
				)
				/** @param {TmaEpgEvents} epg */
				.done(function(epg) {

					//console.log (epg.event[0].name);

					if(epg.event[0])
					{
						var	dateStart = new Date(epg.event[0].starttime_local);
						var	dateEnd = new Date(epg.event[0].endtime_local);

						var epgEventRow = '<tr id="chepg"><td class="epg-cell">' +
											tma.date("H:i", dateStart) +
											' - ' +
											tma.date("H:i", dateEnd) +
											' ' + epg.event[0].name +
											//'</td><td class="epg-info-button-cell">' +
											//'<button class="jk-but but-epginfo"><span><b><i>i</i></b></span></button>' +
										'</td></tr>';

						savedEPG = epgEventRow;

						//$epgrow = $('#chepg').replaceWith(epgEventRow);
						$('#chepg').replaceWith(epgEventRow);
						$('#chepg').prop('epgevent', epg.event[0]);
						//$epgrow = $('#chepg').prop('epgevent', epg.event[0]).replaceWith(epgEventRow);

					}


				}); //END epg.done function
			}//END channel object not null


		}
		else						//Watching a recording, show the recording name and details.
		//=======================================================================================
		//							PLAYBACK
		//=======================================================================================
		{
			var tempProgress = (status.duration * status.currentblock) / status.totalblock;

			//DONE - If this is time-shifting, show the channel name
			//TODO - If playing at other than x1 speed, display the speed.
			//TODO - Button to display the INF/INF+ data.

			// jk: double initialisation of channel
			//var channel = tma.channel(status.current_logoid);

			// jk: modified translation
			var tempBody = '<tr><td>';
			if (status.playmode == PLAYMODE_TempPlaying)
			{
				tempBody += '<b>' + channel.servicename + '</b>';
			}
			else
			{
				//tempBody += '<tr><td><b>' + status.filename.slice(0, -4) + '</b>' + ' at <b>' + tma.formatDuration(tempProgress.toFixed(0)) + '</b> of <b>' + tma.formatDuration(status.duration) + '</b></td><td>';
				tempBody += self.i18.play_recording.format({
					filename:	status.filename.replace(/\.[^.]*$/, ''), // jk: remove file ext. with variable length
					played:		tma.formatDuration(tempProgress.toFixed(0)),
					duration:	tma.formatDuration(status.duration),
				});
			}

			tempBody +=  '</td><td>';

			if(status.trickmode == 4) //Playback paused
			{
				tempBody += '<button class="playmode jk-but iconset ic-play"></button>';
			}
			else
			{
				tempBody += '<button class="playmode jk-but iconset ic-pause"></button>';
			}

			tempBody +=  '</td></tr>';

			$tbody.append(tempBody);

			//Add the playback skip buttons
			var buttonCode = '<tr><td class="skip-button-list" colspan="2">' +
								'<button class="jk-but but-skip but-skip1" skip="-180"><span>-3 min</span></button>' +
								'<button class="jk-but but-skip but-skip2" skip="-60" ><span>-1 min</span></button>' +
								'<button class="jk-but but-skip but-skip3" skip="-30" ><span>-30 Sec</span></button>' +
								'<button class="jk-but but-skip but-skip4" skip="-10" ><span>-10 Sec</span></button>' +
								'<button class="jk-but but-skip but-skip5" skip="+10"  ><span>+10 Sec</span></button>' +
								'<button class="jk-but but-skip but-skip6" skip="+30"  ><span>+30 Sec</span></button>' +
								'<button class="jk-but but-skip but-skip7" skip="+60"  ><span>+1 min</span></button>' +
								'<button class="jk-but but-skip but-skip8" skip="+180" ><span>+3 min</span></button>' +
							'</td></tr>';

			$tbody.append(buttonCode);

		}//END Playback

		//Add transport control keys.
		//>> 2, 4, 8, 16, 32, 64, 128.
		var transportCode = '<tr><td class="transport-button-list" colspan="2">' +
								'<table class="pad-table"><tbody><tr><td>' +

									'<button rcu="f1"  class="control-pad-fn1 transport jk-but"><b>F1</b></button>' +
									'<button rcu="f2"  class="control-pad-fn2 transport jk-but"><b>F2</b></button>' +
									'<button rcu="f3"  class="control-pad-fn3 transport jk-but"><b>F3</b></button>' +
									'<button rcu="f4"  class="control-pad-fn4 transport jk-but"><b>F4</b></button>' +

									'&nbsp;&nbsp;' +

									'<button rcu="info"  class="transport jk-but"><i><b>i</b></i></button>' +

									'<br><br>' +

									recButton +

									'<button rcu="prev"  class="transport jk-but iconset ic-prev"></button>' +
									'<button rcu="rew"   class="transport jk-but iconset ic-rewind"></button>' +
									'<button rcu="slow"  class="transport jk-but iconset ic-slow"></button>' +
									'<button rcu="stop"  class="transport jk-but iconset ic-stop"></button>' +
									'<button rcu="pause" class="transport jk-but iconset ic-pause"></button>' +
									'<button rcu="play"  class="transport jk-but iconset ic-play"></button>' +
									'<button rcu="fwd"   class="transport jk-but iconset ic-forward"></button>' +
									'<button rcu="next"  class="transport jk-but iconset ic-next"></button>' +

								'</td>' +
								'<td>' +
									'<button rcu="menu"  class="control-pad transport jk-but">menu</button>' +
									'<button rcu="up"    class="control-pad transport jk-but iconset ic-up"></button>' +
									'<button rcu="guide" class="control-pad transport jk-but">guide</button>' +
								'<br>' +
									'<button rcu="left"  class="control-pad transport jk-but iconset ic-left"></button>' +
									'<button rcu="ok"    class="control-pad transport jk-but">ok</button>' +
									'<button rcu="right" class="control-pad transport jk-but iconset ic-right"></button>' +
								'<br>' +
									'<button rcu="exit"  class="control-pad transport jk-but">exit</button>' +
									'<button rcu="down"  class="control-pad transport jk-but iconset ic-down"></button>' +
									'<button rcu="playlist"  class="control-pad transport jk-but">list</button>' +
							'</td></tr></tbody></table>' +
						'</td></tr>';
		$tbody.append(transportCode);

		$('DIV.status DIV.status-current').html($table);

		//Playback control buttons
		//$('DIV.status BUTTON.ic-stop').attr('skip','stop');
		$('DIV.status BUTTON.ic-pause').attr('skip','pause');
		$('DIV.status BUTTON.ic-play').attr('skip','play');

		$('DIV.status BUTTON.playmode').click(function(e) {

			var skip = $(this).attr('skip');
			//alert('Playback control ' + skip);

			var params = null;

			if(skip == 'stop')
			{
				tma.stopPlayback().done(function() {
					self.renderStatus();
				});
			}

			if(skip == 'pause')
			{
				params = {
						'function':	'files',
						action:		'pause!'
				};

				$.getJSON('/api', params).done(function() {
					// jk: wait for API response
					self.renderStatus();
				});

			}

			if(skip == 'play')
			{
				params = {
						'function':	'files',
						action:		'resume!'
				};

				$.getJSON('/api', params).done(function() {
					// jk: wait for API response
					self.renderStatus();
				});
			}
		});

		$('DIV.status BUTTON.but-skip').click(function(e) {

			var skip = $(this).attr('skip');
			//alert('Skipping ' + skip);

			var params = {
					'function':	'files',
					action:		'jump!',
					//nohide:		'',		//if "nohide" is present the progress bar is not hidden when jumping.
					time:		skip
			};

			//This is sent blind.  Either it works or it doesn't
			$.getJSON('/api', params).done(function() {
				// jk: wait for API response
				self.renderStatus();
			});
		});

		//Transport keys
		$('DIV.status BUTTON.transport').click(function(e) {

			var key = $(this).attr('rcu');
			//alert('pressing ' + key);

			var params = {
					'function':	'remote',
					action:		'send!',
					key:		key
			};

			//This is sent blind.  Either it works or it doesn't
			$.getJSON('/api', params).done(function() {
				// jk: wait for API response
				self.renderStatus();
			});

		});

		// add event listener to display EPG

		$('TABLE.current-status').on('click', 'TD.epg-cell', function(e) {

			//alert('Displaying');

			var $row = $(this).parents('TR');
			var event = $row.prop('epgevent');

			//alert('Displaying ' + epg.name);
			tma.call( 'showEvent', [ event ] );

		});

	};

	/**
	 * render table of recordings in progress
	 *
	 * @param {TmaStatus} status
	 */
	this.renderRecordings = function(status) {

		//=======================================================================================
		//						RECORDINGS IN PROGRESS
		//=======================================================================================
		//If there are recordings in progress, show them
		var rec;
		var recCount = status.activeRecordingCount();
		var recHeading = '';

		var $table = $('<table class="jk-list jk-round rectable"><tbody /></table>');
		var $tbody = $table.find('TBODY');

		if(recCount === 0)		//No recordings in progress
		{
			recHeading = self.i18.no_recordings;
			$tbody.append('<tr><td></td></tr>');
		}
		else
		{
			if(recCount == 1)	//One recording in progress
			{
				recHeading = recCount + ' ' + self.i18.one_recording;
			}
			else				//Multiple recordings in progress
			{
				recHeading = recCount + ' ' + self.i18.many_recordings;
			}

			for(rec in status.recording)
			{
				if(status.recording[rec].rectype !== 0)
				{
					//TODO - Show logo, % elapsed/remaining, stop recording, extend, start chase-play, get info (inf+ too), reuse module from file list if possible
					//TODO - Red progress bar

					var channel = tma.channel(status.recording[rec].logoid);

					var percentComplete = (status.recording[rec].recordedsec / (status.recording[rec].duration * 60)) * 100;
					var tempMins = status.recording[rec].recordedsec / 60;

					var logoline = '';
					if(channel.logo)
					{
						logoline = '<div class="channellogo channellogo-small logo-small-' + channel.logo.logoid + '"></div> ';
					}
					else
					{
						if(channel.lcn)
						{
							logoline = '<b>' + channel.lcn + '</b>';
						}
						else
						{
							logoline = '<b>' + channel.servicename + '</b>';
						}
					}

					var textDescription = self.i18.recording_in_progress.format({
						filename: status.recording[rec].filename.slice(0, -4),
						channel: channel.servicename,
					});

					var recRow = '<tr><td class="rec-logo">' +
									logoline + '</td>'+
									//'<td class="rec-detail"><b>' + status.recording[rec].filename.slice(0, -4) + '</b> ' +
									//	self.i18.on + ' <b>' + channel.servicename + '</b> ' +
									//self.i18.done + ' <b>' + tma.formatDuration(tempMins.toFixed(0)) + '</b> ' +
									//self.i18.of + ' <b>' + tma.formatDuration(status.recording[rec].duration) + '</b></td>' +

								'<td class="rec-detail">' + textDescription +

								'<td class="rec-times">' +
									tma.formatDuration(tempMins.toFixed(0)) + '&nbsp;/&nbsp;' +
									tma.formatDuration(status.recording[rec].duration) + '</td>' +

								'<td class="rec-buttons">' +
									'<button change="less" class="jk-but rec-change"><span>-5</span></button>' +
									'<button class="jk-but rec-stop iconset ic-stop"></button>' +
									'<button change="more"class="jk-but rec-change"><span>+5</span></button>' +
								'</td></tr>';

					$row = $(recRow).appendTo($tbody);
					//$row.attr('slot', status.recording[rec].slot);
					$row.prop('rec', status.recording[rec]);
				}
			}
		}

		var $head = $('<thead><tr><th colspan="4">' + recHeading + '</th></tr></thead>').prependTo($table);
		$('DIV.status .status-rec-items').html($table);

		//Add stop recording button
		$('DIV.status BUTTON.rec-stop').click(function(e) {

			var $row = $(this).parents('TR');
			//var slot = $row.attr('slot');
			/** @returns {TmaStatusRecording} */
			var rec = $row.prop('rec');
			var slot = rec.slot;
			//alert('stopping slot ' + slot);

			// @TODO localisation
			new jkqDialog({
				title: 'Stop Recording in slot ' + slot, //self.i18.delete_confirm_caption,
				text:  'Stop recording "' + rec.getFilename(true) + '" ?',
				buttons: {
					ok:				 tma.i18.respond_yes,
					jkqDialogCancel: tma.i18.respond_cancel
				}
			}).confirm(function(button) {
				if (button == 'ok') {
					tma.stopRecording(slot).done(function(data) {
						self.renderStatus();
					});
				}
			});
		});

		//Add lengthen and shorten buttons
		$('DIV.status BUTTON.rec-change').click(function(e) {

			var $row = $(this).parents('TR');
			var rec = $row.prop('rec');
			//var slot = $row.attr('slot');
			var slot = rec.slot;
			var change = $(this).attr('change');
			//alert(change + ' on slot ' + slot);

			var newDuration = rec.duration;

			if(change == 'more') { newDuration = newDuration + 5; }
			if(change == 'less') { newDuration = newDuration - 5; }

			//Note: Changing the duration to less than the current recorded time stops the recording.

			if(newDuration > (rec.recordedsec / 60))  //Prevent the user from making the duration shorter then the current elapsed recording time
			{
				var params = {
						'function':	'files',
						action:		'duration!',
						slot:		slot,
						duration:	newDuration
				};

				$.getJSON('/api', params);
			}
			self.renderStatus();
		});

	};

	/**
	 * render table of upcoming timers
	 *
	 * @param {TmaTimers} timers
	 */
	this.renderTimers = function(timers) {

		var t;
		var	tCount = 0;

		var	dateNow = new Date(timers.timestamp);		//'Now' on the PVR
		var dateOnlyNow = new Date(dateNow);
		dateOnlyNow = dateOnlyNow.setHours(0,0,0,0);	//'Today' on the PVR

		var table = (
				'<table id="timer-table" class="upcoming-timers jk-list jk-round">' +
					'<thead><tr><th colspan="2">{title}</th></tr></thead>' +
					'<tbody />' +
				'</table>'
		).format({
			title: (timers.timers.length) ? self.i18.upcoming_timers : self.i18.no_timers,
		});

		var $table = $(table);

		var $tbody = $table.find('TBODY');

		if(timers.timers.length)
		{

			for (t in timers.timers)
			{
				/** @returns {TmaTimer} */
				var timer = timers.timers[t];

				if(timer.isrec == 1 && tCount < 7)
				{
					var	dateStart = new Date(timer.starttime);
					var dateOnlyStart = new Date(dateStart);
					dateOnlyStart = dateOnlyStart.setHours(0,0,0,0);
					var daysRemaining = (dateOnlyStart - dateOnlyNow) / 60000 / 24 / 60;
					var minsRemaining = (dateStart - dateNow) / 60000;
					var textRemaining = tma.formatDuration(minsRemaining.toFixed(0));

					if(daysRemaining === 0 || minsRemaining <= 360)   //This the timer is due today, or in the next 6 hours
					{
						// jk: modified translation
						//textRemaining = self.i18._in + ' <b>' + tma.formatDuration(minsRemaining.toFixed(0)) + '</b>';
						textRemaining = self.i18.timer_remaining_mins.format({
							remaining: tma.formatDuration(minsRemaining.toFixed(0)),
						});
					}
					else
					{
						if(daysRemaining == 1)	//If the timer is due tomorrow
						{
							//jk: modified translation
							textRemaining = self.i18.timer_remaining_tomorrow.format({
								starttime: tma.date( "H:i", timer.starttime ),
							});
						}
						else					//If the timer is due after tomorrow
						{
							//TODO - If this is more than 7 days in the future, use the date
							textRemaining = self.i18.timer_remaining_day.format({
								day: tma.i18.days[dateStart.getDay()].name,
								starttime: tma.date( "H:i", timer.starttime ),
							});
						}
					}

					//TODO - Calculate how long until the timer triggers and colour-code the line.

					//If there is a channel logo, show the logo, if there is no logo, but the LCN is not zero, show that, otherwise show the name
					var logoline = '';
					var channel = tma.channel(timer.logoid);
					if(channel.logo)
					{
						logoline = '<div class="channellogo channellogo-small logo-small-' + channel.logo.logoid + '"></div> ';
					}
					else
					{
						if(channel.lcn)
						{
							logoline = '<b>' + channel.lcn + '</b>';
						}
						else
						{
							logoline = '<b>' + channel.servicename + '</b>';
						}
					}

					var timerRow = '<tr>'+
										'<td class="timer-logo">' + logoline + '</td>' +
										'<td class="timer-row">' +  '<b>' + timer.getFilename(true) + '</b> ' + textRemaining + /*' ' + self.i18.on + ' <b>' + timer.chname + */'</b></td></tr>';

					$row = $(timerRow).appendTo($tbody);

					$row.prop('timer', timer);

					tCount++;
				}
			}
		}

		$('DIV.status DIV.status-timer-items').html($table);

		// add event listener to edit timers
		$('TABLE.upcoming-timers TD.timer-row').click(function(e) {

			var $row = $(this).parents('TR');
			/** @returns {TmaTimer} */
			var timer = $row.prop('timer');

			tma.call( 'editTimer', [ timer ] ).done(function(popup) {
				popup.relTimer = timer;
				$(popup).on('close', function() {
					self.renderStatus();
				});
			});
		});

	};

	this.renderStatus = function () {

		//Clear out timer from previous loop
		if (self.statusInterval) {
			window.clearTimeout(self.statusInterval);
			self.statusInterval = null;
		}

		// get current status
		/** @param {TmaStatus} status */
		tma.apiGetStatus().done(function(status) {

			//=======================================================================================
			//							CURRENT CHANNEL / PLAYBACK
			//=======================================================================================
			self.renderCurrent(status);

			//=======================================================================================
			//						RECORDINGS IN PROGRESS
			//=======================================================================================
			self.renderRecordings(status);

		});

		//Get the timers
		tma.apiGetTimers().done(function(timers) {

			//=======================================================================================
			//								UPCOMING TIMERS
			//=======================================================================================
			self.renderTimers(timers);

		});//END Timers

		// periodically check the status
		//if(!self.statusInterval)
		//{
			//console.log('Before ' + self.statusInterval);
			self.statusInterval = window.setTimeout(
				function() {
					self.renderStatus();
				},
				10000
				//30000  //Refresh the status every 30 seconds
			);
			//console.log('After ' + self.statusInterval);
		//}
	};

	this.refresh = function() {
		self.renderStatus();
	};

}
