/**
 * EPG module
 */
tma.registerModule( epgchannelpage );
epgchannelpage.prototype = new tmaModule();
function epgchannelpage() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'epgchannelpage';

	this.pageGroup = 'epg';

	/**
	 * current selected channel
	 */
	this.channel = null;

	var self = this;

	/**
	 * Starttime of the epg-timespan in milliseconds rounded to minutes
	 *
	 * @type {Number}
	 */
	this.starttime = 0;

	/**
	 * Endtime of the epg-timespan in milliseconds rounded to minutes
	 *
	 * @type {Number}
	 */
	this.endtime = 0;

	/**
	 * default query length in minutes
	 */
	this.timeSliceLength = 360; // 6h => 4 slices per day

	/**
	 * total time range ( maxTimeSlices x timeSliceLength )
	 */
	this.maxTimeSlices = 28;	// 7 days

	/* settings */

	var prefs = {

			/**
			 * get the show-description-mode (none|first|*full)
			 *
			 * @return {String}
			 */
			getDescrMode: function() {
				return self.localSetting('descr', 'full');
			},

			/**
			 * set the show-description-mode (none|first|full)
			 *
			 * @param {String}	set
			 */
			setDescrMode: function(set) {
				if (
						(set == 'none') ||
						(set == 'first') ||
						(set == 'full')
				) {
					self.setLocalSetting( 'descr', set );
				}
			},

			/**
			 * get the show-text-mode (none|*first|full)
			 *
			 * @return {String}
			 */
			getTextMode: function() {
				return self.localSetting('text', 'first');
			},

			/**
			 * set the show-text-mode (none|first|full)
			 *
			 * @param {String}	set
			 */
			setTextMode: function(set) {
				if (
						(set == 'none') ||
						(set == 'first') ||
						(set == 'full')
				) {
					self.setLocalSetting( 'text', set );
				}
			},

			/**
			 * get the show-text-if-no-description-mode (true|*false)
			 *
			 * @return {String}
			 */
			getTextDepMode: function() {
				return self.localSetting('textdep', false) ;
			},

			/**
			 * set the show-text-if-no-description-mode (true|false)
			 *
			 * @param {Boolean}	set
			 */
			setTextDepMode: function(set) {
				if (set !== undefined) {
					self.setLocalSetting( 'textdep', set === true );
				}
			},

			/**
			 * get the show-genre-mode (none|*show)
			 *
			 * @return {String}
			 */
			getGenreMode: function() {
				return self.localSetting('genre', 'show');
			},

			/**
			 * set the show-genre-mode (none|show)
			 *
			 * @param {String} set
			 */
			setGenreMode: function(set) {
				if (
						(set == 'none') ||
						(set == 'show')
				) {
					self.setLocalSetting( 'genre', set );
				}
			},

			/**
			 * get the genre-coloring (*on|off)
			 *
			 * @return {Boolean}
			 */
			getGenreColoring: function() {
				return parseInt( self.localSetting( 'genre-coloring', 1 ) ) == 1;
			},

			/**
			 * set the genre-coloring (on/off)
			 *
			 * @param {Boolean}	set		new genre-coloring setting
			 */
			setGenreColoring: function(set) {
				self.setLocalSetting( 'genre-coloring', (set) ? 1 : 0 );
			},

	};

	/**
	 * initialize module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'appendMenuTab' );

		return this.loadLanguage();

	};

	/**
	 * append tab to menu
	 *
	 * @param {tmaFeature}	f
	 */
	this.appendMenuTab = function( f ) {

		tma.call('addPage', ['MainMenu', this]);

	};

	/**
	 * Display the channels obtained from the PVR in the browser window
	 */
	this.show = function() {

		this.init().done(function() {

			// load genre list
			tma.apiEpgGenre().done(function() {

				self.showEPG();

				tma.call('addConfigItem', ['MainMenu', self]);

				self.insertConfig();

			});

		});

	};

	/**
	 * this page gets removed, so hide the channels config
	 */
	this.cleanup = function() {

		// remove event handling
		$(tma).off('tmaTimerAdded.epgchannelpage');
		$(tma).off('tmaTimerModified.epgchannelpage');
		$(tma).off('tmaTimerDeleted.epgchannelpage');
		$(window).off('resize.epgchannelpage');
		$(tma).off('genre_colors_change.epgchannelpage');

	};

	/**
	 * creates a reservation bar for a timer, if it is in the EPG range
	 *
	 * @param {TmaTimer}	timer		tma-timer object
	 * @returns {jqObject} or false
	 */
	this.createTimerMarker = function(timer) {

		var $events = $('.epgch-event');

		/**
		 * Valid days for recurring reservation
		 * default: every day is valid
		 */
		var recurringValidDays = [1,1,1,1,1,1,1];

		if (timer.reservationtype() == 2) {
			// weekend, only sunday and saturday is valid
			recurringValidDays = [1,0,0,0,0,0,1];
		} else if (timer.reservationtype() == 4) {
			// weekdays, only monday to friday is valid
			recurringValidDays = [0,1,1,1,1,1,0];
		}

		//*jshint loopfunc: true */
		$events.each(function() {

			var $event = $(this);
			var event = $event.prop('event');

			var event_start_date= new Date(event.starttime_local);
			var event_end_date	= new Date(event.endtime_local);
			var event_start = event_start_date.getTime();
			var event_end	= event_end_date.getTime();
			var event_duration = (event_end - event_start);

			var $cell = $event.find('.epgch-clm-time');
			var $timebox = $cell.find('.epgch-clm-timebox');

			var cellH = $cell.height();
			var timeboxOffs = $timebox.position();

			var timer_start = new Date(timer.starttime).getTime();
			var timer_end	= new Date(timer.endtime).getTime();

			var loop = 0;
			var day = 0;
			while (timer_start < self.endtime) {

				var dayOfWeek = parseInt(tma.date('w', timer_start));

				if (
						(recurringValidDays[dayOfWeek]) &&
						(
								((timer_start	>  event_start) && (timer_end	<  event_end)) ||
								((timer_start	<= event_start) && (timer_end	>  event_start)) ||
								((timer_start	<  event_end  ) && (timer_end	>= event_end))
						)
				) {

					var timer_duration = Math.min(timer_end, event_end) - Math.max(timer_start, event_start);
					var h = Math.min(cellH, Math.round(cellH * timer_duration / event_duration) );
					var top = Math.max(
							- timeboxOffs.top,
							Math.round( - timeboxOffs.top + cellH * (timer_start - event_start) / event_duration )
					);

					$(
							(
									'<div class="epgch-mark {typeclass}{typeext}" style="top:{top}px;height:{height}px;" />'
							).format({
								typeclass:	timer.isrec ? 'epgch-mark-rec' : 'epgch-mark-play',
										typeext:	(loop) ? '-ff' : '',
												top:		top,
												height:		h,
							})
					)
					.attr('timerid', timer.id)
					.prop('timer', timer)
					/*.click(function(e) {

						// edit reservation

						// prevent opening EPG details
						e.stopPropagation();

						// hope, that any module handles the 'editTimer' event
						tma.call( 'editTimer', [ $(this).prop('timer') ] );

					})*/

					.appendTo($timebox);

				}

				loop++;
				if (!timer.reservationtype()) {
					// one time
					break;
				} else if (timer.reservationtype() == 3) {
					// weekly
					timer_start += 7*24*3600000;	// next week
					timer_end	+= 7*24*3600000;	// next week
					day			+= 7;				// next week
				} else {
					timer_start += 24*3600000;		// next day
					timer_end	+= 24*3600000;		// next day
					day++;							// next day
				}

			}

		});

	};

	this.renderTimers = function() {

		if (tma.timers) {
			if (this.channel) {
				$('.epgch-events-box DIV.epgch-mark').remove();
				/** @param {TmaTimer} timer */
				$.each(tma.timers.timers, function(ix, timer) {

					if (timer.logoid == self.channel.logoid) {

						self.createTimerMarker(timer);

					}

				});
			}
		}

	};

	/**
	 * process the result of the EGP query
	 *
	 * @param {TmaEpgEvents}	epg			EPG events list
	 * @param {TmaChannel}		channel		tma channel object
	 */
	this.processEpgResult = function(epg, channel) {

		var $container = $('.epgch-events');

		var evtTmpl =	'<tr class="epgch-event" eventid="{eventid}" timestamp="{timestamp}">' +
							'<td class="epgch-clm-time">' +
								'<div class="epgch-clm-timebox">' +
									'<span class="epgch-evt-date">{date}</span>' +
									'<span class="epgch-evt-time">{start} - {end}</span>' +
									//'<span class="epgch-evt-duration">{duration} min</span>' +
									'<span class="epgch-evt-duration">{dur_fmt}</span>' +			//DMC Format duration
								'</div>' +
							'</td>' +
							'<td class="epgch-clm-name" has-descr="{hasdescr}">' +
								'{genre}' +
								'<span class="epgch-evt-title">{name}</span>' +
								'<span class="epgch-evt-descr">{description}</span>' +
								'<span class="epgch-evt-text">{text}</span>' +
							'</td>' +
						'</tr>';

		if (epg.event) {

			var day = 99;

			$.each(epg.event, function(ix, event) {

				var $event = $( evtTmpl.format({
					eventid:	event.eventid,
					timestamp:	(new Date(event.starttime_local)).getTime(),
					date:			tma.date(self.i18.shortdate_fmt, event.starttime_local),
					start:			tma.date('H:i', event.starttime_local),
					end:			tma.date('H:i', event.endtime_local),
					duration:		event.duration,
					dur_fmt: 		tma.formatDuration(event.duration), //DMC Format duration
					name:			event.name,
					genre:			getGenreAndParentalString(event, prefs.getGenreColoring() ),
					description:	event.description,
					hasdescr:		(event.description) ? 'y' : 'n',
					text:			event.text.replace(/(?:\r\n|\r|\n)/g, '<br>'),
				}) )

				.appendTo($container)
				.prop('event', event);

				var event_start_date= new Date(event.starttime_local);
				var event_end_date	= new Date(event.endtime_local);
				var event_start = event_start_date.getTime();
				var event_end	= event_end_date.getTime();

				// new day?
				if (event_start_date.getDate() > day) {
					$event.addClass('epgch-new-day');
				}
				day = event_start_date.getDate();

				// current?
				if (
						(event_start < self.starttime) &&
						(event_end	 > self.starttime)
				) {

					var percent = parseInt((self.starttime - event_start) * 100 / (event_end - event_start));

					var currBarTmpl =	'<span class="epgch-evt-curr">' +
											'<span class="epgch-evt-curr-part" style="width: {percent}%;" />' +
										'</span>';

					$(currBarTmpl.format({
						percent: percent,
					}))
					.appendTo($event.find('.epgch-clm-name'));

				}

			});

			// display timers
			this.renderTimers();

		}

	};

	/**
	 * load and display EPG data one channel
	 *
	 * @param {TmaChannel}	channel		tma channel object
	 * @returns {jqXHR}
	 */
	this.updateEpg = function(channel) {

		this.channel = channel;

		$('.epgch-events .epgch-event').remove();

		return tma.apiGetEpg(
				channel,
				tma.toPvrDate(this.starttime),
				tma.toPvrDate(this.endtime)
		)
		.done(function(epg) {

			self.processEpgResult(epg, channel);

		});

	};

	this.setTimespan = function() {

		this.starttime = (tma.config.mock) ? new Date('2013/10/13 18:06:02') : new Date();
		this.starttime = Math.round(this.starttime / 60000) * 60000;
		var minutes = this.maxTimeSlices * this.timeSliceLength;
		this.endtime = this.starttime + minutes * 60000;

	};

	/**
	 * load and display an EPG grid
	 *
	 * currently the EPG data is loaded for all channels of the selected favourites group (=> channels page)
	 * the time range is currently fix 3 hours from now
	 */
	this.showEPG = function () {

		var favGroup = tma.localSetting('defaultfavourites', '');

		this.channels = tma.getChannels(favGroup);

		this.setTimespan();

		var $stickyContent = $('DIV.sticky-content');
		var $content = $('DIV#content');

		// channel selector
		$stickyContent.append(createChannelSelector(
				// blacklist
				[
				 //'00c0000103fb283d', // ARD for test
				 //'00c0000103f32b66', // ZDF for test
				 "ffffffffffffff00",
				 "ffffffffffffff02",
				 "ffffffffffffff04",
				 ]
		));

		var $template = $(
				(
					'<div class="epgch-events-box" descr-mode="{descrMode}" text-mode="{textMode}"' +
					' textdep="{textdep}" genre-mode="{genreMode}">' +
						'<table class="jk-list jk-round">' +
							'<thead>' +
								'<tr><th colspan="2">{date}</th></tr>' +
							'</thead>' +
							'<tbody class="epgch-events" />' +
						'</table>' +
					'</div>'
				).format({
					date:		tma.date(this.i18.list_date_fmt, this.starttime),
					descrMode:	prefs.getDescrMode(),
					textMode:	prefs.getTextMode(),
					genreMode:	prefs.getGenreMode(),
					textdep:	prefs.getTextDepMode() ? 'y' : 'n',
				})
		).appendTo($content)
		.css('padding-top', $stickyContent.outerHeight());

		var $select = $stickyContent.find('SELECT[name="channels"]');

		// first get status to update current channel and rec channels
		/** @param {TmaStatus} status */
		tma.apiGetStatus().done(function(status) {

			// then get timers to set reservation-bars
			tma.apiGetTimers().done(function(timers) {

				$select.change(function() {

					// compute timestamp of current top event
					var topEventStart = '';
					$('TBODY.epgch-events TR').each(function() {

						var $tr = $(this);
						var top = $tr.position().top;

						if (top > 0) {
							topEventStart = $tr.attr('timestamp');
							return false; // break
						}

					});

					var channel = tma.channel($(this).val());
					if (channel) {
						self.updateEpg(channel).done(function() {

							if (topEventStart) {

								// set scroll pos to event with starttime closest to topEventStart
								var offs =
									$('DIV.epgch-events-box TBODY').position().top -
									$('DIV.epgch-events-box TABLE').position().top;
								var minDist = 99999999999999999;
								var closestScrollTop = 0;
								$('TBODY.epgch-events TR').each(function() {

									var $tr = $(this);
									var timestamp = parseInt($tr.attr('timestamp'));
									var top = $tr.position().top;

									if (Math.abs(timestamp - topEventStart) < minDist) {
										minDist = Math.abs(timestamp - topEventStart);
										closestScrollTop  = top - offs;
									}

								});
								$('DIV.epgch-events-box').scrollTop(closestScrollTop);

							}

						});
					}

				})
				.val(status.current_logoid).change();

			});

		});

		this.createEventHandler();

	};

	this.createEventHandler = function() {

		// show timer form
		$('.epgch-events').on('click', '.epgch-mark', function(e) {

			// prevent opening EPG details
			e.stopPropagation();

			// hope, that any module handles the 'editTimer' event
			tma.call( 'editTimer', [ $(this).prop('timer') ] );

		});

		// show event details
		$('.epgch-events').on('click', '.epgch-event', function(e) {

			var $event = $(this);

			// check, if any module handles the 'showEvent' event
			// and prevent reopening of the same info
			if (
					(tma.fn.showEvent) &&
					!$event.hasClass('jk-mark')
			) {

				tma.fn.showEvent( $event.prop('event') ).done(function(popup) {
					popup.$relatedEvent = $event;
					$(popup).on('close', function() {
						this.$relatedEvent.removeClass('jk-mark');
					});
				});

				$event.addClass('jk-mark');

			}

		});

		// handle added timer
		$(tma).on('tmaTimerAdded.epgchannelpage', function(e, timer) {

			// add reservation
			self.createTimerMarker(timer);

		});

		// handle modified timer
		$(tma).on('tmaTimerModified.epgchannelpage', function(e, timer) {

			// remove prev reservetion
			$('DIV.epgch-mark[timerid="' + timer.id + '"]').remove();
			// add modified reservation
			self.createTimerMarker(timer);

		});

		// handle deleted timer
		$(tma).on('tmaTimerDeleted.epgchannelpage', function(e, timer) {

			// remove prev reservetion
			$('DIV.epgch-mark[timerid="' + timer.id + '"]').remove();

		});

	};

	function onGenreColorChanged() {

		//console.log('epgchannelspage.onGenreColorChanged');
		// genre color configuration has changed
		$('.epgch-event H4[genre-value]').each(function() {
			var genre = tma.genres.genre( parseInt($(this).attr('genre-value')) );
			if (genre) {
				$(this).attr('style', genre.style( prefs.getGenreColoring() ));
			}
		});

	}

	this.insertConfig = function () {

		var template = (
			'<h3>{label_heading}</h3>' +
			'<div class="setting">' +
				'<span class="label">{description.label}:</span>' +
				'<input type="radio" name="description" value="none" id="descr0">' +
				'<label for="descr0">{description.none}</label> ' +
				'<input type="radio" name="description" value="first" id="descr1">' +
				'<label for="descr1">{description.first}</label> ' +
				'<input type="radio" name="description" value="full" id="descr2">' +
				'<label for="descr2">{description.full}</label> ' +
			'</div>' +
			'<div class="setting">' +
				'<span class="label">{text.label}:</span>' +
				'<input type="radio" name="text" value="none" id="text0">' +
				'<label for="text0">{text.none}</label> ' +
				'<input type="radio" name="text" value="first" id="text1">' +
				'<label for="text1">{text.first}</label> ' +
				'<input type="radio" name="text" value="full" id="text2">' +
				'<label for="text2">{text.full}</label> ' +
				'<input type="checkbox" name="textdep" value="y" id="textdep">' +
				'<label for="textdep">{text.depondescr}</label> ' +
			'</div>' +
			'<div class="setting">' +
				'<span class="label">{genre.label}:</span>' +
				'<input type="radio" name="genre" value="none" id="genre0">' +
				'<label for="genre0">{genre.none}</label> ' +
				'<input type="radio" name="genre" value="show" id="genre1">' +
				'<label for="genre1">{genre.show}</label> ' +
			'</div>'
		).format(this.i18.config);

		var $config = $(template);

		/* description */
		$config.find('INPUT[name="description"]').click(function() {

			prefs.setDescrMode( $(this).val() );
			$('.epgch-events-box').attr('descr-mode', $(this).val());
			self.renderTimers();

		})
		.filter('[value="' +  prefs.getDescrMode() + '"]' )
		.prop('checked', true);

		/* text */
		$config.find('INPUT[name="text"]').click(function() {

			prefs.setTextMode( $(this).val() );
			$('.epgch-events-box').attr('text-mode', $(this).val());
			self.renderTimers();

		})
		.filter('[value="' +  prefs.getTextMode() + '"]' )
		.prop('checked', true);

		/* text depending on description */
		$config.find('INPUT[name="textdep"]').click(function() {

			$('.epgch-events-box').attr('textdep', $(this).prop('checked') ? 'y' : 'n');
			prefs.setTextDepMode( $(this).prop('checked') );
			self.renderTimers();

		})
		.prop('checked', prefs.getTextDepMode());

		/* show/hide genre */
		$config.find('INPUT[name="genre"]').click(function() {

			prefs.setGenreMode( $(this).val() );
			$('.epgch-events-box').attr('genre-mode', $(this).val());
			self.renderTimers();

		})
		.filter('[value="' +  prefs.getGenreMode() + '"]' )
		.prop('checked', true);

		var $colorSettings = tma.fn.getModule('genrecolours').createConfig(
				prefs.getGenreColoring(),
				function(isEnabled) { // onEnableChange
					prefs.setGenreColoring(isEnabled);
					onGenreColorChanged();
				}
		);

		$(tma).on('genre_colors_change.epgchannelpage', onGenreColorChanged);

		$('DIV#config')
		.append($config)
		.append($colorSettings);

	};

}
