/**
 * EPG event info modul
 */
tma.registerModule( eventinfo );
eventinfo.prototype = new tmaModule();
function eventinfo() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'eventinfo';

	var self = this;

	/* settings */

	var prefs = {

			/**
			 * get the pre-padding time in minutes
			 *
			 * @return {Number}
			 */
			getPrePadding: function() {
				return parseInt(self.setting('pretime'	, '0'));
			},

			/**
			 * set the pre-padding time in minutes
			 *
			 * @param {Number}	set
			 */
			setPrePadding: function(set) {
				self.setSetting( 'pretime', parseInt(set) );
			},

			/**
			 * get the post-padding time in minutes
			 *
			 * @return {Number}
			 */
			getPostPadding: function() {
				return parseInt(self.setting('posttime'	, '0'));
			},

			/**
			 * set the post-padding time in minutes
			 *
			 * @param {Number}	set
			 */
			setPostPadding: function(set) {
				self.setSetting( 'posttime', parseInt(set) );
			},

			/**
			 * get the genre-coloring (*on|off)
			 *
			 * @return {Boolean}
			 */
			getGenreColoring: function() {
				return parseInt( self.localSetting( 'genre-coloring', 1 ) ) == 1;
			},

			/**
			 * set the genre-coloring (on/off)
			 *
			 * @param {Boolean}	set		new genre-coloring setting
			 */
			setGenreColoring: function(set) {
				self.setLocalSetting( 'genre-coloring', (set) ? 1 : 0 );
			},

	};

	/**
	 * initialize module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'showEvent' );
		tma.fn.addFeature( this, 'findEvent' );

		return $.Deferred().resolve();

	};

	/**
	 * Show EPG event popup
	 *
	 * @param {tmaFeature}	f
	 * @param {TmaEpgEvent}	event		tma EPG event object
	 */
	this.showEvent = function (f, event) {

		/**
		 * since this contains asynchronous calls, pause the chain
		 * and decide with the result of the call, whether to stop,
		 * or to continue
		 */
		f.pause();


		this.init().done(function() {

			var popup = self.createPopup(event);

			// return popup instance
			f.stop(popup);

		});

	};

	/**
	 * Show EPG event popup by EventID in time range
	 *
	 * @param {tmaFeature}	f
	 * @param {TmaEpgEvent}	options		Object containing eventid, channel, starttime, endtime
	 */
	this.findEvent = function (f, options) {

		/**
		 * since this contains asynchronous calls, pause the chain
		 * and decide with the result of the call, whether to stop,
		 * or to continue
		 */
		f.pause();

		this.init().done(function() {

			return tma.apiGetEpg(
					options.channel,
					tma.toPvrDate(options.starttime),
					tma.toPvrDate(options.endtime)
			)

			.done(
					/** @param {TmaEpgEvents} epg */
					function(epg) {

						var event = epg.getEvent( options.eventid );
						if (event) {

							var popup = self.createPopup(event, options.channel);

							// return popup instance
							f.stop(popup);

						}

					}
			);

		});

	};

	function onGenreColorChanged() {

		//console.log('eventinfo.onGenreColorChanged');
		// genre color configuration has changed
		$('.eventinfo-frame H4[genre-value]').each(function() {
			var genre = tma.genres.genre( parseInt($(this).attr('genre-value')) );
			if (genre) {
				$(this).attr('style', genre.style( prefs.getGenreColoring() ));
			}
		});

	}

	/**
	 * create window
	 *
	 * @param {TmaEpgEvent}	event		tma EPG event object
	 * @returns {jkqWindow}
	 */
	this.createPopup = function(event) {

		var popup = new jkqWindow();
		popup.width = 500;

		popup.show();
		popup.setTitle( event.channel.servicename );
		popup.closeOnOutsideClick();

		var $content = popup.container.find('DIV.jkqWindowContent');

		var event_start 	= new Date(event.starttime_local);
		var event_end		= new Date(event.endtime_local);
		var event_duration	= parseInt( (event_end.getTime() - event_start.getTime()) / 60000);

		var template = (
				'<div class="eventinfo-frame">' +
					'<div class="eventinfo-top-bar">' +
						'<div class="sign-rec" />' +
						'<button class="jk-but iconset ic-clock" title="{i18.edit_timer_tooltip}" />' +
						'<button class="jk-but iconset ic-rec" title="{i18.create_timer_tooltip}" />' +
						'<button class="jk-but iconset ic-gear" />' +
					'</div>' +
					'<div class="config eventinfo-config">' +
						'<div class="setting setting-less-more pretime">' +
							'<span class="label">{i18.config.pretime}</span>' +
							'<input type="text" name="pretime" value="0" disabled> <span>min.</span> ' +
							'<button class="lessmore less">&#x25C0;</button>' +
							'<button class="lessmore more">&#x25B6;</button>' +
						'</div>' +
						'<div class="setting setting-less-more posttime">' +
							'<span class="label">{i18.config.posttime}</span>' +
							'<input type="text" name="posttime" value="0" disabled> <span>min.</span> ' +
							'<button class="lessmore less">&#x25C0;</button>' +
							'<button class="lessmore more">&#x25B6;</button>' +
						'</div>' +
					'</div>' +
					'<div class="eventinfo">' +
						'{eventDate}' +
						'<h1>{eventName}</h1>' +
						'{channelLogo}' +
						'{eventGenreParental}' +
						'<h2>{eventDescription}</h2>' +
						'<p>{eventText}</p>' +
					'</div>' +
				'</div>'
		).format( $.extend(
				{},
				self,
				{
					eventName:			event.name,
					eventDescription:	event.description,
					eventText:			event.text.replace(/(?:\r\n|\r|\n)/g, '<br>'),
					eventDate:			getEventDateString(event_start, event_end, event_duration),
					eventGenreParental:	getGenreAndParentalString( event, prefs.getGenreColoring() ),
					channelLogo:		(event.channel.haslogo() ? '<div class="channellogo logo-' + event.channel.logoid + '"/>' : ''),
				}
		) );

		var $template = $(template).appendTo($content);

		// is "edittimer" available
		if (tma.fn.editTimer) {

			// assign timer to event, if available
			event.timer = self.findTimer(event);

			// show a record sign, if there is a timer with the eventid of this event
			if (event.timer) {
				$template.addClass('has-timer');
			}

			// edit the timer of this event
			$template.find('DIV.eventinfo-top-bar BUTTON.ic-clock').click( function() {

				tma.fn.editTimer( event.timer );

			});

			// create a new timer for this event
			$template.find('DIV.eventinfo-top-bar BUTTON.ic-rec').click( function() {

				// create a new timer
				tma.fn.editTimer(
						new TmaTimer({
							index:		-1,		// new timer
							isrec:		1,		// record
							starttime:	tma.toPvrDate(event_start.getTime() - (prefs.getPrePadding() * 60000) ),
							duration:	event_duration + prefs.getPrePadding() + prefs.getPostPadding(),
							filename:	event.name + tma.config.recextension,
							logoid:		event.channel.logoid,
							eventid1:	event.eventid,
							eventid2:	event.eventid,
							serviceid:	event.serviceid,
						})
				);

			});

			// handle modified timer
			$(tma).on('tmaTimerModified.eventinfo' + popup.ctx.id, function() {
				popup.close();
			});

			// handle added timer
			$(tma).on('tmaTimerAdded.eventinfo' + popup.ctx.id, function(timer) {
				event.timer = timer;
				popup.container.find('.eventinfo-frame').addClass('has-timer');
				popup.close();
			});

			// handle deleted timer
			$(tma).on('tmaTimerDeleted.eventinfo' + popup.ctx.id, function() {
				event.timer = null;
				popup.container.find('.eventinfo-frame').removeClass('has-timer');
				popup.close();
			});

			$(popup).on('close', function() {
				// remove timer handlers on close
				$(tma).off('tmaTimerModified.eventinfo' + this.ctx.id);
				$(tma).off('tmaTimerAdded.eventinfo' + this.ctx.id);
				$(tma).off('tmaTimerDeleted.eventinfo' + this.ctx.id);
			});

		}

		/* config */
		$template.on('click', 'BUTTON.ic-gear', function() {
			$template.find('DIV.eventinfo-config').slideToggle();
		});

		// config pre-padding
		var $pretime = $template.find('DIV.eventinfo-config DIV.pretime INPUT');
		$pretime.val( prefs.getPrePadding() );

		$template.find('DIV.eventinfo-config DIV.pretime BUTTON.less').click(function() {

			var val = parseInt( $pretime.val() );
			if (val > 0) {
				$pretime.val( (val - 1) );
				prefs.setPrePadding( val - 1 );
			}

		});
		$template.find('DIV.eventinfo-config DIV.pretime BUTTON.more').click(function() {

			var val = parseInt( $pretime.val() );

			$pretime.val( (val + 1) );
			prefs.setPrePadding( val + 1 );

		});

		// config post-padding setting
		var $posttime = $template.find('DIV.eventinfo-config DIV.posttime INPUT');
		$posttime.val( prefs.getPostPadding() );

		$template.find('DIV.eventinfo-config DIV.posttime BUTTON.less').click(function() {

			var val = parseInt( $posttime.val() );
			if (val > 0) {
				$posttime.val( (val - 1) );
				prefs.setPostPadding( val - 1 );
			}

		});
		$template.find('DIV.eventinfo-config DIV.posttime BUTTON.more').click(function() {

			var val = parseInt( $posttime.val() );

			$posttime.val( (val + 1) );
			prefs.setPostPadding( val + 1 );

		});

		// Genre colouring

		var $colorSettings = tma.fn.getModule('genrecolours').createConfig(
				prefs.getGenreColoring(),
				function(isEnabled) { // onEnableChange
					prefs.setGenreColoring(isEnabled);
					onGenreColorChanged();
				}
		);

		$(tma).on('genre_colors_change.eventinfo', onGenreColorChanged);
		$(popup).on('close', function() {
			$(tma).off('genre_colors_change.eventinfo');
		});

		$template.find('DIV.eventinfo-config').append($colorSettings);

		popup.adjustPos({
			hAlign:	'right',
			effect:	'slide-rtl'
		});

		return popup;

	};

	/**
	 * searches a timer with a specific eventid
	 *
	 * @param {Object}	event	tma-Event object
	 * @returns {Object} tma-Timer object, or false if not found
	 */
	this.findTimer = function(event) {

		var found = false;

		$.each(tma.timers.timers, function(nr, timer) {

			if (timer.serviceid == event.serviceid) {

				var timer_start = new Date(timer.starttime).getTime();
				var timer_end	= new Date(timer.endtime).getTime();
				var event_start = new Date(event.starttime_local);
				var event_end	= new Date(event.endtime_local);

				if (timer.eventid2 == event.eventid) {
					// found by corresponding eventid
					found = timer;
					return false; // break
				} else if ((timer_start <= event_start) && (timer_end >= event_end)) {
					// found: event is fully covered
					found = timer;
					return false; // break
				} else if (
						((timer_start <= event_start) && (timer_end > event_start))  ||
						((timer_start < event_end) && (timer_end >= event_end))
				) {
					// found: event is partially covered
					found = timer;
					return false; // break
				}
			}

		});

		return found;

	};

}
