/**
 * RecInf modul
 */
tma.registerModule( recfileinfo );
function recfileinfo() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'recfileinfo';

	var self = this;

	/**
	 * initialize module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'showRecInf' );

		return $.Deferred().resolve();

	};

	/**
	 * display standard recording file info
	 *
	 * @param {tmaFeature}	f
	 * @param {Object}		file		tma recfile object
	 */
	this.showRecInf = function(f, file) {

		/**
		 * since this contains an asynchronous call, pause the chain
		 * and decide with the result of the call, whether to stop,
		 * or to continue
		 */
		f.pause();

		this.init().done(function() {

			if (tma.debug) tma.log('recfileinfo.showRecInf: init.done');

			var channel = tma.channel(file.infdata.header.logoid);

			var popup = new jkqWindow();
			popup.width = 500;

			popup.show();
			popup.setTitle( (channel) ? channel.servicename : file.infdata.service.servicename );
			popup.closeOnOutsideClick();

			var $content = popup.container.find('DIV.jkqWindowContent');

			var recStart = new Date(file.infdata.header.starttime);
			var recDuration = file.infdata.header.durationmin * 60000 + file.infdata.header.durationsec * 1000;
			var recEnd = new Date(recStart.getTime() + recDuration);
			var recBlocks = parseInt(file.size / tma.config.blockfactor);

			var eventStart 		= new Date(file.infdata.eventinfo.starttime_utc + ' UTC');
			var eventEnd		= new Date(file.infdata.eventinfo.endtime_utc + ' UTC');
			var eventDuration	= parseInt( (eventEnd.getTime() - eventStart.getTime()) / 60000);

			var calcStart		= Math.max(recStart.getTime(), eventStart.getTime());
			var calcEnd			= Math.min(recEnd.getTime(), eventEnd.getTime());
			var calcDuration	= parseInt((calcEnd - calcStart) / 60000);

			var startPercent	= parseInt( (calcStart - recStart) * 1000 / recDuration ) / 10;
			var durationPercent	= parseInt( (calcEnd - calcStart) * 1000 / recDuration ) / 10;

			var partial = (eventDuration != calcDuration);

			var startBlock = parseInt(recBlocks * startPercent / 100);

			$(
					self.templates.content.render({
						i18:				self.i18,
						eventDate:			getEventDateString(eventStart, eventEnd, eventDuration, calcDuration),
						recbar:				{
												partial:			partial,
												startBlock:			startBlock,
												startPercent:		startPercent,
												durationPercent:	durationPercent,
											},
						eventName:			file.infdata.eventinfo.name,
						eventDescription:	file.infdata.eventinfo.description,
						eventText:			file.infdata.exteventinfo.text.replace(/(?:\r\n|\r|\n)/g, '<br>'),
						eventGenreParental:	getGenreAndParentalString(file.infdata.eventinfo),
						channelLogo:		((channel) && (channel.logo) ? '<div class="channellogo logo-' + channel.logo.logoid + '"/>' : ''),
					})
			).appendTo($content)
			// play event
			.on('click', 'DIV.eventbar', function() {

				var startposition = $(this).attr('startblock');

				if (tma.fn.playFile) {
					tma.fn.playFile( tma.files.directory, file, startposition );
				}

			});

			if (tma.debug) {
				popup.adjustPos();
			} else {
				popup.adjustPos({
					hAlign:	'right',
					effect:	'slide-rtl'
				});
			}

			// return this popup instance
			f.stop(popup);

		});

	};

}
