/**
 * Inf+ module
 */
tma.registerModule( recfileinfplus );
function recfileinfplus() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'recfileinfplus';

	var self = this;

	/**
	 * Tap ID
	 */
	this.TapId = 1145914122;

	/**
	 * override setup to check if Inf+ TAP is running
	 */
	this.setup = function() {

		// if not prooved in manifest
		//if (!tma.taps.find(this.TapId)) return false;

		// register tmaFeatures
		tma.addApiExtension( this, 'apiLoadInfPlus' );
		tma.fn.addFeature( this, 'showRecInf' );

		return $.Deferred().resolve();

	};

	/**
	 * get Inf+ info
	 *
	 * @param {String}		dir			path
	 * @param {String}		file		filename
	 * @param {String}		fileHex	hex representation of the filename
	 * @returns jqXHR
	 */
	this.apiLoadInfPlus = function (dir, file, fileHex) {

		var data =
			"dir="	    + dir + "\n" +
			"file="	    + file + "\n" +
			"file_hex=" + fileHex;

		/*var data = {
			dir		: dir,
			file	: file,
			file_hex: fileHex
		};*/

		var url = '/api?function=getlive&tap=' + this.TapId + '!';

		/**
		 * this construct allows us to switch to fail(),
		 * if the api returns an error_code <br>
		 * see: return deferred.promise().fail()
		 * http://stackoverflow.com/a/5112734
		 */
		var deferred = $.Deferred();

		$.ajax({
			type: "POST",
			url: url,
			data: data,
			dataType: 'json'
		}).done(function(data, status, xhr) {

			if (data.error_code === 0) {
				deferred.resolve(data, status, xhr);
			} else {
				// switch to fail()
				// TODO think about standardised errors
				deferred.reject(
						xhr,
						'tma-error',
						'loadInfPlus: POST ' + url + ' returns error_code ' + data.error_code
				);
			}

		}).fail(function(xhr, errorType, error) {
			deferred.reject(xhr, errorType, error);
		});

		return deferred.promise().fail(function(xhr, errorType, error) {

			tma.log({
				title: errorType + " in recfileinfplus.loadInfPlus()",
				details: {
					error: error,
					xhrReadyState: xhr.readyState,
					xhrStatus: xhr.status,
					parameter: '',
					directory: dir,
					file: file,
					filehex: fileHex,
				}
			});

		});

	};

	/**
	 * returns the html string for a recording bar
	 *
	 * @param {Boolean}	partial
	 * @param {Number}	startBlock
	 * @param {Number}	startPercent
	 * @param {Number}	durationPercent
	 * @returns {String}
	 */
	this.recBar = function (partial, startBlock, startPercent, durationPercent) {

		var s =	'<div class="recbar">' +
		'<div class="eventbar" partial="' + partial + '" ' +
		'startblock="' + startBlock + '" ' +
		'style="left:' + startPercent + '%;width:' + durationPercent + '%;" ' +
		'title="' + this.i18.event_bar_title + '">' +
		'&nbsp;&#x25B6;</div>' +
		'</div>';

		return s;

	};

	/**
	 * display Inf+ recording file info
	 *
	 * @param {tmaFeature}	f
	 * @param {Object}		file		tma recfile object
	 */
	this.showRecInf = function(f, file) {

		/**
		 * since this contains asynchronous calls, pause the chain
		 * and decide with the result of the call, whether to stop,
		 * or to continue
		 */
		f.pause();

		this.init().done(function() {

			var channel = tma.channel(file.infdata.header.logoid);

			var recStart = new Date(file.infdata.header.starttime);
			var recDuration = file.infdata.header.durationmin * 60000 + file.infdata.header.durationsec * 1000;
			var recEnd = new Date(recStart.getTime() + recDuration);
			var recBlocks = parseInt(file.size / tma.config.blockfactor);

			//self.apiLoadInfPlus(tma.files.directory, f.name, f.name_hex).done(function(data) {
			// test tma api extension
			tma.apiLoadInfPlus(tma.files.directory, file.name, file.name_hex).done(function(data) {

				var popup = new jkqWindow();
				popup.width = 500;

				popup.show();
				popup.setTitle(
						((channel) ? channel.servicename : file.infdata.service.servicename) + ' - ' +
						$( getEventDateString(recStart, recEnd, file.infdata.header.durationmin) ).text()
				);
				popup.closeOnOutsideClick();

				var $content = popup.container.find('DIV.jkqWindowContent');

				var $infplusContent = $('<div class="infplus" />').appendTo($content);

				// calculate start and duration of each event in the recording and show info
				$.each(data.events, function(nr, event) {

					var eventStart	= new Date(event.starttime);
					var eventEnd	= new Date(event.endtime);

					var calcStart		= Math.max(recStart.getTime(), eventStart.getTime());
					var calcEnd			= Math.min(recEnd.getTime(), eventEnd.getTime());
					var calcDuration	= parseInt((calcEnd - calcStart) / 60000);

					var startPercent	= parseInt( (calcStart - recStart) * 1000 / recDuration ) / 10;
					var durationPercent	= parseInt( (calcEnd - calcStart) * 1000 / recDuration ) / 10;

					var partial = (event.duration != calcDuration);

					var startBlock = parseInt(recBlocks * startPercent/ 100);

					$(	'<div class="fileinfo">' +
							getEventDateString(eventStart, eventEnd, event.duration, calcDuration) +
							self.recBar(partial, startBlock, startPercent, durationPercent) +
							'<h1>' + event.name + '</h1>' +
							'<div class="eventinfo" eventid="' + event.eventid + '">' +
							((channel) && (channel.logo) ? '<div class="channellogo logo-' + channel.logo.logoid + '"/>' : '') +
							getGenreAndParentalString(event) +
							'<h2>' + event.description + '</h2>' +
							'<p>' + event.text.replace(/(?:\r\n|\r|\n)/g, '<br>') + '</p>' +
							'</div>' +
							'</div>'
					).appendTo($infplusContent);

				});

				$infplusContent.find('DIV.eventinfo[eventid="' + file.infdata.eventinfo.eventid + '"]')
				.show()
				.parents('DIV.fileinfo').css('background-color', '#FFFFFF');

				// accordion
				$infplusContent.on('click', 'DIV.fileinfo', function() {

					var $eventinfo = $(this).find('DIV.eventinfo');
					if ($eventinfo.is(':hidden')) {
						// open hidden infos only
						$infplusContent.find('DIV.eventinfo').filter(':visible')
						.slideUp()
						.parents('DIV.fileinfo').css('background-color', '#F0F0F0');
						$eventinfo.slideDown()
						.parents('DIV.fileinfo').css('background-color', '#FFFFFF');
					}

				});

				// play event
				$infplusContent.on('click', 'DIV.eventbar', function() {

					var startposition = $(this).attr('startblock');

					if (tma.fn.playFile) {
						tma.fn.playFile( tma.files.directory, file, startposition );
					}

				});

				popup.adjustPos({
					hAlign:	'right',
					effect:	'slide-rtl'
				});

				/**
				 * this method handled the feature, so
				 * stop the feature chain
				 * and return this popup instance
				 */
				f.stop(popup);

			}).fail(function(xhr, status, err) {

				/**
				 * this method can't handle the feature, so
				 * continue the feature chain
				 */
				f.run();

			});

		}).fail(function(xhr, status, err) {

			/**
			 * this method can't handle the feature, so
			 * continue the feature chain
			 */
			f.run();

		});

	};

}
