/**
 * recordings module
 */
tma.registerModule( recordingspage );
recordingspage.prototype = new tmaModule();
function recordingspage() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'recordingspage';

	this.pageGroup = 'files';

	// available table columns
	var tableColumns = {
			filename:	true,
	        filefn:		true,
	        filesize:	true,
	        filedate:	true,
	        duration:	true,
	        channel:	true
	        };

	var self = this;

	/**
	 * js timer, checks status periodically (set by this.playFile)
	 */
	this.statusInterval = null;

	/**
	 * setup module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'appendMenuTab' );
		tma.fn.addFeature( this, 'playFile' );
		tma.fn.addFeature( this, 'startTap' );

		return this.loadLanguage();

	};

	/**
	 * append tab to menu
	 *
	 * @param {tmaFeature}	f
	 */
	this.appendMenuTab = function( f ) {

		tma.call('addPage', ['MainMenu', this]);

	};

	/**
	 * display the content
	 *
	 * @returns jQueryDeferred
	 */
	this.show = function() {

		return this.init().done(function() {

			if (tma.files === null) {
				// load the files
				var dir = $('DIV.directory A[href]').last().attr('href') || "/DataFiles";
				self.loadFiles(dir);
			} else {
				// show existing files
				self.showFiles(tma.files);
			}

			tma.call('addConfigItem', ['MainMenu', self]);

			self.insertConfig();

		});

	};

	/**
	 * reload data and display the content
	 */
	this.refresh = function() {

		// refresh the files
		var dir = $('DIV.directory A[href]').last().attr('href') || "/DataFiles";
		self.loadFiles(dir);

	};

	/**
	 * this page gets removed
	 */
	this.cleanup = function() {

		// stop status checking
		if (self.statusInterval) {
			window.clearTimeout(self.statusInterval);
			self.statusInterval = null;
		}

		// hide and clear config frame, if exists
		$('DIV#config').removeClass('recordings-config');

	};

	/**
	 * get a new directory content
	 *
	 * @returns jQueryDeferred
	 */
	this.loadFiles = function (directory) {

		// delete all child nodes of content and display a pending image
		$('#content').append('<img src="css/img/wait.gif">');
		$("#timestamp").text(this.i18.files_loading);

		return tma.apiGetFiles(directory).always(function() {

			// remove pending image
			$('#content').empty();

		}).done(function(files) {

			self.showFiles(files);

		}).fail(function(xhr, errorType, error) {

			$("#timestamp").append( tma.i18.error.ajax.format(errorType, error) );

		});

	};

	/**
	 * display path breadcrumbs
	 */
	this.renderBreadcrumbs = function() {

		var path = tma.files.directory.replace(/^\//, '').split('/');
		var $dirDiv = $(
				'<div class="directory">' +
					'<select name="mount">' +
						'<option value="/mnt/hd/DataFiles">DataFiles</option>' +
						'<option value="/mnt/hd/MediaFiles">MediaFiles</option>' +
						'<option value="/mnt/hd/MP3Files">MP3Files</option>' +
						'<option value="/mnt/hd/PhotoFiles">PhotoFiles</option>' +
						'<option value="/mnt/hd/ProgramFiles">ProgramFiles</option>' +
					'</select><br>' +
				'</div>'
		)
		.appendTo('DIV#content')
		.on('click', 'A', function(e) {
			e.preventDefault();
			self.loadFiles( $(this).attr('href') );
		}).on('change', 'SELECT', function(e) {
			self.loadFiles( $(this).val() );
		});

		var mount = '';
		var mount_depth = 1;
		$.each(path, function(nr, value) {

			mount += '/' + value;
			$('<span>/</span>').appendTo($dirDiv);

			if ( (nr == 1) && (value == 'hd')) {
				mount_depth = 2;
			}

			if ( nr >= mount_depth ) {

				$('<a href="' + mount + '">' + value + '</a>').appendTo($dirDiv);

				if ( nr == mount_depth ) {
					$dirDiv.find('SELECT').val(mount);
				}

			} else {

				$('<span>' + value + '</span>').appendTo($dirDiv);

			}

		});

	};

	/**
	 * display the file list of the current files object
	 *
	 * @param {TmaFiles}	files
	 */
	this.showFiles = function (files) {

		// delete all child nodes of content
		$('#content').empty();

		$("#timestamp"	).text( new Date(tma.files.timestamp).toLocaleString() );

		var driveTmpl = (
				'<div class="diskspace">' +
					'<span>{disk_space}</span>' +
					'<div class="diskspace-used" />' +
					'<div class="diskspace-free" />' +
					'<span class="diskspace-total" />' +
				'</div>'
		).format(this.i18);

		// show disk space
		$(driveTmpl).appendTo('DIV#content');

		this.setFilesStatus();

		if (!tma.files) return;

		// sort files
		var sortKey = this.localSetting('sort_key', 'name');
		var sortDir = this.localSetting('sort_dir', 'asc');
		tma.sortFiles(sortKey, sortDir);

		// display path breadcrumbs
		this.renderBreadcrumbs();

		// create Table
		var $table = $('<table/>', {
			'class': "files jk-list jk-round"
		});

		var head = '<thead><tr>';
		var row  = '<tr>';

		var userCols = this.localSetting( 'columns', tableColumns );
		$.each(userCols, function(clmClass) {
			if (
					(clmClass != 'channel') ||
					( tma.files.directory.indexOf('/mnt/hd/DataFiles') === 0 )
			) {
				head += '<th class="' + clmClass + '" can-sort-dir="asc"><a/></th>';
				row  += '<td class="' + clmClass + '"><span/></td>';
			}
		});

		head +=	'</tr></thead>';
		row  +=	'</tr>';

		// create table head with sort options
		var $head = $(head).appendTo($table);
		$head.find('TH.filename A').text(this.i18.columns.filename);
		$head.find('TH.filesize A').text(this.i18.columns.filesize);
		$head.find('TH.filedate A').text(this.i18.columns.filedate);
		if ( tma.files.directory.indexOf('/mnt/hd/ProgramFiles') === 0 ) {
			$head.find('TH.duration A').text('Version');
		} else {
			$head.find('TH.duration A').text(this.i18.columns.duration);
		}
		$head.find('TH.channel  A').text(this.i18.columns.channel);
		$head.find('TH.filefn   A').remove();

		$head.find('TH.filename').attr('sort-key', 'name');
		$head.find('TH.filesize').attr('sort-key', 'size');
		$head.find('TH.filedate').attr('sort-key', 'sortdate');
		$head.find('TH.duration').attr('sort-key', 'infdata.header.durationmin');
		$head.find('TH.channel' ).attr('sort-key', 'infdata.service.servicename');
		$head.find('TH.filefn'	).removeAttr('can-sort-dir');

		// set the current sort state
		$head.find('TH[sort-key="' + sortKey + '"]').attr({
			'sort-dir':		sortDir,
			'can-sort-dir': (sortDir == 'asc') ? 'desc' : 'asc'
		});

		// set sort event handler
		$head.on('click', 'A', function(e) {
			e.preventDefault();
			var sDir = $(this).parent().attr('can-sort-dir');
			var sKey = $(this).parent().attr('sort-key');
			self.setLocalSetting('sort_key', sKey);
			self.setLocalSetting('sort_dir', sDir);
			self.showFiles(files);
		});

		var $tbody = $('<tbody/>').appendTo($table);

		// create file list
		var channelDisplay = parseInt( this.localSetting('channel_display', 3) );
		var channel;

		/** @param {TmaFile} f */
		files.eachFile(function(i, f) {

			$row = $(row).appendTo($tbody);

			$row.attr('attr', f.attr);
			$row.attr('inode', f.inode);
			$row.prop('file', f);

			var removeExt = (f.infpresent || f.tappresent);
			$row.find('TD.filename SPAN').text( (f.getName(removeExt)) );

			if (f.attr == 208) {
				// size
				if (		f.size /       1024 /*1KB*/ < 0.1 ) {
					$row.find('TD.filesize SPAN').text( ( f.size ).toLocaleString() + ' B');
				} else if (	f.size /    1048576 /*1MB*/ < 0.1 ) {
					$row.find('TD.filesize SPAN').text( (parseInt(f.size * 10 /       1024 /*1KB*/) / 10 ).toLocaleString() + ' KB');
				} else if (	f.size / 1073741824 /*1GB*/ < 0.1 ) {
					$row.find('TD.filesize SPAN').text( (parseInt(f.size * 10 /    1048576 /*1MB*/) / 10 ).toLocaleString() + ' MB');
				} else {
					$row.find('TD.filesize SPAN').text( (parseInt(f.size * 10 / 1073741824 /*1GB*/) / 10 ).toLocaleString() + ' GB');
				}

				// date
				if ( (f.infpresent) && (f.infdata.header.starttime)) {
					$row.find('TD.filedate SPAN').text( tma.date( self.i18.recdate_fmt, f.infdata.header.starttime ) );
				} else {
					$row.find('TD.filedate SPAN').text( tma.date( self.i18.recdate_fmt, f.timestamp ) );
				}

				if (f.infpresent) {

					$row.addClass('playable');
					$row.addClass('infpresent');

					// duration
					$row.find('TD.duration SPAN').text( tma.formatDuration(f.infdata.header.durationmin) );

					// channel
					channel = tma.channel(f.infdata.header.logoid);
					if ( channel ) {

						$row.find('TD.channel').html(
								'<div class="channellogo channellogo-small logo-small-' + channel.logoid + '" />' +
								'<span class="channelname">' +
								((channel) ? channel.servicename : f.infdata.service.servicename) +
								'</span>'
						);
					}

					// info button
					$('<button class="jk-but info">i</button>').appendTo($row.find('TD.filefn > SPAN'));

				} else if ( f.tappresent ) {

					$row.addClass('startable');
					$row.addClass('tappresent');
					$row.attr('tapid', f.tap.tapid);

					if (tma.taps.find(f.tap.tapid, tma.files.directory + '/' + f.name)) {
						$row.addClass('running');
					}

					// author
					$row.find('TD.channel SPAN').text( f.tap.author );

					// name
					$row.find('TD.filename SPAN').text( f.tap.name );

					// version
					$row.find('TD.duration SPAN').text( f.tap.version );

				} else if ( f.name.match(/\.rec$|\.mpg$|\.mkv$|\.ts$|\.vob$|\.m2ts$|\.divx$/) ) {

					$row.addClass('playable');

				} else if ( f.name.match(/\.txt$|\.log|\.prn$|\.csv$|\.json$|\.ini$|\.sh$|\.lng$/) ) {

					$row.addClass('textfile');

				}

				// more functions opener
				$('<button class="jk-but iconset ic-gear morefn" title="' + self.i18.menu.more_functions + '"></button>').appendTo($row.find('TD.filefn > SPAN'));

			}

		});

		// hide logo or channel name
		if ( (channelDisplay & 1) === 0 ) $table.find('TD.channel > DIV').hide();
		if ( (channelDisplay & 2) === 0 ) $table.find('TD.channel > SPAN').hide();

		// hide deselected columns
		$.each(userCols, function(clmClass, display) {
			if (!display) $table.find( ( 'TD.{0},TH.{0}' ).format( clmClass ) ).hide();
		});

		// add event listener for folders to change directories
		$table.find('TR[attr="224"]').on('click', function() {

			// 'this' is the clicked element
			var f = $(this).prop('file');
			var directory = tma.files.directory + '/' + f.name;
			self.loadFiles( directory );

		});

		// add event listener to show infos
		$table.on('click', 'BUTTON.info', function(e) {

			var $row = $(this).parents('TR');

			if (tma.debug) tma.log('recordingspage.info.click');

			// check, if any module handles the 'showRecInf' event
			// and prevent reopening of the same info
			if (
					(tma.fn.showRecInf) &&
					!$row.hasClass('jk-mark')
			) {

				if (tma.debug) tma.log('recordingspage.info.click: showRecInf available');

				var f = $row.prop('file');

				if (tma.debug) tma.log('recordingspage.info.click: f=' + f.name);

				tma.fn.showRecInf( f ).done(function(popup) {

					popup.file = f;
					$(popup).on('close', function() {
						$('TABLE.files TR[inode="' + this.file.inode + '"]').removeClass('jk-mark');
					});

				});

				$row.addClass('jk-mark');

			}

		});

		// add event listener to show "more functions"
		$table.on('click', 'BUTTON.morefn', function() {

			// remove previous moreFn's
			$('DIV.morefn').fadeOut(function() {
				$(this).remove();
			});

			$(this).parents('TR').addClass('jk-mark');

			var $moreFn = $(
					('<div class="morefn">' +
						// close moreFn
						'<button class="jk-but morefn_close"><span>&#x25B2;</span></button>' +
						// play file
						'<button class="jk-but play"><i class="fa fa-play" /><span>{play}</span></button>' +
						// start TAP
						'<button class="jk-but start"><i class="fa fa-play" /><span>{start}</span></button>' +
						// stop
						'<button class="jk-but stop"><i class="fa fa-stop" /><span>{stop}</span></button>' +
						// stop recording
						'<button class="jk-but stoprec"><i class="fa fa-stop" /><span>{stoprec}</span></button>' +
						// delete
						'<button class="jk-but iconset ic-delete"><span>{del}</span></button>' +
						// rename
						'<button class="jk-but iconset ic-rename"><span>{rename}</span></button>' +
						// move
						'<button class="jk-but iconset ic-move"><span>{move}</span></button>' +
						// open
						'<button class="jk-but open"><i class="fa fa-file-text-o" /><span>{open}</span></button>' +
					'</div>').format(self.i18.menu)
			);

			/** @returns {TmaFile} */
			var file = $(this).parents('TR').prop('file');

			tma.call('fileCreateMoreFn', [$moreFn, file]);

			$moreFn.appendTo( $(this).parent() )
			.fadeIn();

		});

		// add event listener to play files
		$table.on('click', 'BUTTON.play', function() {

			var $tr = $(this).parents('TR');
			/** @returns {TmaFile} */
			var f = $tr.prop('file');

			// start playback
			if ( $tr.is('.playable') ) {

				var startposition = 0;
				if (f.infpresent) {
					startposition = f.infdata.bookmarks.startposition || 0;
				}

				self.playFile(null, tma.files.directory, f, startposition);

			}

		});

		// add event listener to start TAPs
		$table.on('click', 'BUTTON.start', function() {

			var $tr = $(this).parents('TR');
			/** @returns {TmaFile} */
			var f = $tr.prop('file');

			// start TAP
			if ( $tr.is('.startable') ) {

				self.startTap(null, tma.files.directory, f);

			}

		});

		// add event listener to stop playback or tap
		$table.on('click', 'BUTTON.stop', function() {

			var $tr = $(this).parents('TR');

			if ($tr.is('.play')) {
				tma.stopPlayback().done(function() {

					$tr.removeClass('play');

					if (self.statusInterval) {
						window.clearTimeout(self.statusInterval);
						self.statusInterval = null;
					}

				});
			}

			if ($tr.is('.running')) {

				/** @returns {TmaFile} */
				var f = $tr.prop('file');
				// find running TAP
				var tap = tma.taps.find(f.tap.tapid, tma.files.directory + '/' + f.name);
				if (tap) {
					tma.apiStopTap(tap).done(function() {
						$tr.removeClass('running');
					});
				} else {
					$tr.removeClass('running');
				}
			}

		});

		// add event listener to stop recording
		$table.on('click', 'BUTTON.stoprec', function() {

			var $tr = $(this).parents('TR');
			var slot = parseInt( $tr.attr('slot') );
			tma.stopRecording(slot).done(function(data) {

				if (data.return_code == 1) {
					$tr.removeClass('rec');
				}

			});

		});

		// add event listener to delete files
		$table.on('click', 'BUTTON.ic-delete', function() {

			/** @returns {TmaFile} */
			var f = $(this).parents('TR').prop('file');

			self.deleteFile(null, tma.files.directory, f);
		});

		// add event listener to rename files
		$table.on('click', 'BUTTON.ic-rename', function() {

			/** @returns {TmaFile} */
			var f = $(this).parents('TR').prop('file');

			self.renameFile(null, tma.files.directory, f);
		});

		// add event listener to move files
		$table.on('click', 'BUTTON.ic-move', function() {

			/** @returns {TmaFile} */
			var f = $(this).parents('TR').prop('file');
			var root = $('DIV.directory SELECT').val();

			// open a directory selector...
			if (tma.featureExist('selectDirectory')) {
				tma.fn.selectDirectory( root, function(selectedPath) {
					self.moveFile(null, tma.files.directory, f, selectedPath);
				});
			}

		});

		// add event listener to open files
		$table.on('click', 'BUTTON.open', function() {

			/** @returns {TmaFile} */
			var f = $(this).parents('TR').prop('file');

			// open a text file in a new window
			window.open(tma.files.directory + '/' + f.name,'_blank');

		});

		// close "more functions"
		$table.on('click', 'DIV.morefn BUTTON', function() {
			$(this).parent().fadeOut(function() {
				$(this).remove();
			});
			$(this).parents('TR').removeClass('jk-mark');
		});

		$('DIV#content').append($table);

	};

	/**
	 * sets sizes and displayed values (used/free/total) of the current drive
	 *
	 * @param {Number}	freeGB		free disk space in Gigabytes
	 * @param {Number}	totalGB		total disk space in Gigabytes
	 */
	this.setDiskSpace = function(freeGB, totalGB) {

		var usedGB = totalGB - freeGB;

		$('DIV.diskspace DIV.diskspace-used')
		.css('width', (parseInt(usedGB * 200 / totalGB) / 10) + 'em')
		.text(usedGB + ' GB');

		$('DIV.diskspace DIV.diskspace-free')
		.css('width', (parseInt(freeGB * 200 / totalGB) / 10) + 'em')
		.text(freeGB + ' GB');

		// set total space
		$('DIV.diskspace SPAN.diskspace-total')
		.text(totalGB + ' GB ' + this.i18.disk_total);

	};

	/**
	 * loads the status via API and updates diskspace and file's play/rec state
	 */
	this.setFilesStatus = function () {

		/** @param {TmaStatus} status */
		tma.apiGetStatus().done(function(status) {

			self.setDiskSpace(status.freedisk >> 10, status.totaldisk >> 10);

			$('TABLE.files TBODY TR.playable').removeClass('rec play');

			if (self.statusInterval) {
				window.clearTimeout(self.statusInterval);
				self.statusInterval = null;
			}

			if (status.absolutefilename) {
				// file is playing
				var file = tma.splitPath(status.absolutefilename);
				if (file) {
					if (tma.files.directory == file.directory) {
						$('TABLE.files TR').filter(function() {
							/** @returns {TmaFile} */
							var f = $(this).prop('file');
							return (f) &&
							(
									(f.getName() + '.inf' == status.getFilename()) ||
									(f.getName() == status.getFilename())
							);
						}).addClass('play');
					}
				}

			}

			if (tma.files.directory == '/mnt/hd/DataFiles') {

				status.eachRecording( function(nr, recording) {

					$('TABLE.files TR').filter(function() {
						var f = $(this).prop('file');
						return (f) && (f.name_hex == recording.filename_hex);
					}).addClass('rec').attr('slot', recording.slot);

				});

			}

			// remove external mount points
			$('DIV.directory SELECT OPTION[external]').remove();

			if (status.drive_count > 0) {
				var $dirSelect = $('DIV.directory SELECT');
				$.each(status.drives, function(nr, drive) {

					$('<option value="' + drive.mountpath + '" external="true">' + drive.mountpath + '</option>').appendTo($dirSelect);

					if ( tma.files.directory.indexOf(drive.mountpath) === 0 ) {
						// this is the current drive
						$dirSelect.val(drive.mountpath);

						self.setDiskSpace(drive.free_mb >> 10, drive.size_mb >> 10);

					}

				});
			}

			// Tap status
			if ( tma.files.directory.indexOf('/mnt/hd/ProgramFiles') === 0 ) {
				tma.apiGetTaps().done(function() {

					$('TABLE.files TBODY TR.startable').removeClass('running');

					/** @param {TmaTap} tap */
					tma.taps.eachTap(function(i, tap) {

						if ( tap.file_name.replace(/\/[^/]+$/, '') == tma.files.directory ) {
							$('TABLE.files TBODY TR[tapid="' + tap.tapid + '"]').addClass('running');
						}

					});

				});
			}

			// periodically check the status
			self.statusInterval = window.setTimeout(
				function() {
					self.setFilesStatus();
				},
				5000
			);

		});

	};

	/**
	 * displays an alert-box with a file operation error message
	 *
	 * @param {String} fileAction	Name of the file operation
	 * @param {Object} data			API result data
	 */
	this.fileError = function(fileAction, data) {

		var errmsg = '';
		if ( (data.mpg_status !== 0) && (data.mpg_errno) ) {
			errmsg += '<i>{recext}</i>: {mpg_error}<br>';
		}
		if ( (data.inf_status !== 0) && (data.inf_errno) ) {
			errmsg += '<i>inf</i>: {inf_error}<br>';
		}
		if ( (data.nav_status !== 0) && (data.nav_errno) ) {
			errmsg += '<i>nav</i>: {mpg_error}<br>';
		}

		new jkqAlert({
			title: tma.i18.error.dialog_title,
			text: (
					'{message}' +
					'<span style="display: block; text-align: left; margin-top: 0.5em">' +
						errmsg +
					'</span>'
			).format({
				message:	fileAction,
				recext:		tma.config.recextension,
				mpg_error:	(
						tma.i18.error.files.errno[data.mpg_errno] ?
								tma.i18.error.files.errno[data.mpg_errno] :
									'errno = ' + data.mpg_errno
				),
				inf_error:	(
						tma.i18.error.files.errno[data.inf_errno] ?
								tma.i18.error.files.errno[data.inf_errno] :
									'errno = ' + data.nav_errno
				),
				nav_error:	(
						tma.i18.error.files.errno[data.nav_errno] ?
								tma.i18.error.files.errno[data.nav_errno] :
									'errno = ' + data.inf_errno
				),
			})
		}).confirm();

	};

	/**
	 * Feature handler: rename a file
	 *
	 * @param {tmaFeature}	f
	 * @param {String}		directory
	 * @param {TmaFile}		file				tma file object
	 * @param {String}		newDirectory
	 */
	this.moveFile = function (f, directory, file, newDirectory) {

		if (tma.files.directory != newDirectory) {

			tma.apiRenameFile(
					directory, file.name, file.name_hex,
					newDirectory, file.name
			).done(function(data) {

				if (
						(data.mpg_status === 0) &&
						(data.inf_status === 0) &&
						(data.nav_status === 0)
				) {

					$('TABLE.files TR[inode="' + file.inode + '"]').remove();

				} else {

					self.fileError(tma.i18.error.files.move, data);

				}

			});
		}

	};

	/**
	 * Feature handler: delete a file
	 *
	 * @param {tmaFeature}	f
	 * @param {String}		directory
	 * @param {TmaFile}		file				tma file object
	 */
	this.deleteFile = function(f, directory, file) {

		new jkqDialog({
			title: self.i18.delete_confirm_caption,
			text: self.i18.delete_confirm.format(file.getName(true)),
			buttons: {
				ok:				 tma.i18.respond_yes,
				jkqDialogCancel: tma.i18.respond_cancel
			}
		}).confirm(function(button) {

			if (button == 'ok') {

				tma.apiDeleteFile(directory, file.name, file.name_hex).done(function(data) {

					if (
							(data.mpg_status === 0) &&
							(data.inf_status === 0) &&
							(data.nav_status === 0)
					) {

						$('TABLE.files TR[inode="' + file.inode + '"]').remove();

						self.setFilesStatus();

					} else {

						self.fileError(tma.i18.error.files.del, data);

					}

				});

			}

		});

	};

	/**
	 * Feature handler: rename a file
	 *
	 * @param {tmaFeature}	f
	 * @param {String}		directory
	 * @param {TmaFile}		file				tma file object
	 */
	this.renameFile = function(f, directory, file) {

		new jkqInputDialog({
			title: self.i18.rename_dialog_caption,
			text: self.i18.rename_dialog_text.format( file.getName(file.infpresent) ),
			inputValue: file.getName(file.infpresent).replace(/"/, '&quot;')
		}).confirm(function() {

			tma.apiRenameFile(
					directory, file.name, file.name_hex,
					directory, this.value() + ((file.infpresent) ? tma.config.recextension : '')
			).done(function(data) {

				if (
						(data.mpg_status === 0) &&
						(data.inf_status === 0) &&
						(data.nav_status === 0)
				) {

					$('TABLE.files TR[inode="' + file.inode + '"] TD.filename SPAN').text( file.getName( file.infpresent || file.tappresent) );

				} else {

					self.fileError(tma.i18.error.files.rename, data);

				}

			});

		});

	};

	/**
	 * Feature handler: play a file
	 *
	 * @param {tmaFeature}	f
	 * @param {String}		directory
	 * @param {TmaFile}		file				tma file object
	 * @param {Number}		startposition
	 */
	this.playFile = function (f, directory, file, startposition) {

		tma.apiPlayFile(directory, file.name, file.name_hex, startposition).done(function(playObject) {

			if (playObject.status === 0) {

				$('TABLE.files TR').removeClass('play')
					.filter('TR[inode="' + file.inode + '"]').addClass('play');

				// periodically check the status
				self.statusInterval = window.setTimeout(
					function() {
						self.setFilesStatus();
					},
					2000
				);

			} else {

				new jkqAlert({
					title:	tma.i18.error.dialog_title,
					text:	tma.i18.error.files.play
				}).confirm();

			}

		});

	};

	/**
	 * Feature handler: start a TAP
	 *
	 * @param {tmaFeature}	f
	 * @param {String}		directory
	 * @param {TmaFile}		file
	 */
	this.startTap = function (f, directory, file, startposition) {

		tma.apiStartTap(directory + '/' + file.name).done(function(result) {

			if (result.return_code === 1 && result.error === 0) {

				$('TABLE.files TR[tapid="' + file.tap.tapid + '"]').addClass('running');

			} else {

				new jkqAlert({
					title:	tma.i18.error.dialog_title,
					text:	tma.i18.error.files.tap_start
				}).confirm();

			}

		}).fail(function() {

			new jkqAlert({
				title:	tma.i18.error.dialog_title,
				text:	tma.i18.error.files.tap_start
			}).confirm();

		});

	};

	this.insertConfig = function () {

		// create column config
		var $colSetting = createColumnsConfig(
				this.localSetting( 'columns', tableColumns ),
				this.i18.config.label_columns,
				$.extend({filefn:	'Buttons'}, this.i18.columns),
				'#content TABLE.files',
				function(cfg) {
					self.setLocalSetting('columns', cfg );
				}
		);

		// logo/text
		var $channelSetting = createChannelConfig(
				this.localSetting('channel_display', 3),
				this.i18.config.channel_display.caption,
				this.i18.config.channel_display,
				'#content TABLE.files',
				function(cfg) {
					self.setLocalSetting('channel_display', cfg );
				}
		);

		$('DIV#config')
		.append($colSetting)
		.append($channelSetting)
		.addClass('recordings-config');


	};

}
