/**
 * edit timer popup module
 */
tma.registerModule( timeredit );
timeredit.prototype = new tmaModule();
function timeredit() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'timeredit';

	var self = this;

	/**
	 * initialize module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'editTimer' );

		return $.Deferred().resolve();

	};

	/**
	 * display the reservation form
	 *
	 * @param {tmaFeature}	f
	 * @param {TmaTimer}	origTimer		tma timer object
	 * @param {Object}		modifications	object containing modifications to the original timer, e.g.
	 * 										{
	 *											starttime:	'2014/08/15 16:00:00',
	 *											duration:	15,
	 *											endtime:	'2014/08/15 16:15:00',
	 *										}
	 * @returns {jQueryDeferred}
	 */
	this.editTimer = function(f, origTimer, modifications) {

		var modTimer = $.extend({}, origTimer, modifications);

		/**
		 * since this contains asynchronous calls, pause the chain
		 * and decide with the result of the call, whether to stop,
		 * or to continue
		 */
		f.pause();

		this.init().done(function() {

			// init the window
			var popup = new jkqWindow();
			popup.showModal();
			popup.setTitle(
					(origTimer.index < 0) ? self.i18.caption_new : self.i18.caption_edit
			);

			var $content = popup.container.find('DIV.jkqWindowContent');

			$content.append(self.templates.content.render({
				i18: self.i18,
				frequencies: tma.i18.frequency,
				channels: tma.getChannels(''),
			}));

			if (origTimer.index < 0) {
				$content.find('#timerbuttonupdate SPAN').text( self.i18.button_create );
				$content.find('#timerbuttondelete').remove();
			}

			//DMC - Filter the generic AV option
			$content.find('#timerservice OPTION[value="ffffffffffffff00"]').remove();

			//Start time toolbar
			$('DIV#starttoolbar > BUTTON').button({text: true}).click(function() {
				var amount = parseInt( $(this).text() );
				self.adjustTime( "#timerstart", amount, 1 );
			});

			//Duration time toolbar
			$('DIV#durtoolbar > BUTTON').button({text: true}).click(function() {
				var amount = parseInt( $(this).text() );
				self.adjustTime( "#timerduration", amount, 1 );
			});

			//End time toolbar
			$('DIV#endtoolbar > BUTTON').button({text: true}).click(function() {
				var amount = parseInt( $(this).text() );
				self.adjustTime( "#timerend", amount, 2 );
			});

			popup.adjustPos({
				hAlign: 'right',
				effect: 'slide-rtl'
			});

			/**
			 * this could be an alternative time selection
			 * instead of slider access
			 * set controlType: dtpSpinner in dtpOptions
			var dtpSpinner =  {
					create: function(tp_inst, obj, unit, val, min, max, step){
						$('<input class="ui-timepicker-input" value="'+val+'" style="width:50%">')
						.appendTo(obj)
						.spinner({
							min: min,
							max: max,
							step: step,
							change: function(e,ui){ // key events
								// don't call if api was used and not key press
								if(e.originalEvent !== undefined)
									tp_inst._onTimeChange();
								tp_inst._onSelectHandler();
							},
							spin: function(e,ui){ // spin events
								tp_inst.control.value(tp_inst, obj, unit, ui.value);
								tp_inst._onTimeChange();
								tp_inst._onSelectHandler();
							}
						});
						return obj;
					},
					options: function(tp_inst, obj, unit, opts, val){
						if(typeof(opts) == 'string' && val !== undefined)
							return obj.find('.ui-timepicker-input').spinner(opts, val);
						return obj.find('.ui-timepicker-input').spinner(opts);
					},
					value: function(tp_inst, obj, unit, val){
						if(val !== undefined)
							return obj.find('.ui-timepicker-input').spinner('value', val);
						return obj.find('.ui-timepicker-input').spinner('value');
					}
			};
			 */

			// setup the datepickers
			var dtpOptions = {
					//controlType: dtpSpinner,
					controlType: 'slider',
					addSliderAccess: true,
					sliderAccessArgs: { touchonly: false },
					firstDay: 1,
					timeFormat: "HH:mm",
					dateFormat: "yy/mm/dd",
					monthNames: [],
					monthNamesShort: [],
					dayNames: [],
					dayNamesShort: [],
					dayNamesMin: [],
					timeText: self.i18.dtp_time_text,
					hourText: self.i18.dtp_hour_text,
					minuteText: self.i18.dtp_minute_text,
					closeText: self.i18.dtp_close_text,
					currentText: self.i18.dtp_current_text,
					timeOnlyTitle: self.i18.dtp_time_only_title,
			};

			$.each(tma.i18.months, function(i, month) {
				dtpOptions.monthNames.push(month.name);
				dtpOptions.monthNamesShort.push(month.shortname);
			});

			$.each(tma.i18.days, function(i, day) {
				dtpOptions.dayNames.push(day.name);
				dtpOptions.dayNamesShort.push(day.shortname);
				dtpOptions.dayNamesMin.push(day.shortname.slice(0, 2));
			});

			//Setup the start date field
			$("#timerstart").datetimepicker($.extend({}, dtpOptions, {
				onClose: function (selectedDateTime) {
					$("#timerstart").val( tma.toPvrDate( new Date(selectedDateTime) ) );
					self.refreshTimerEditTimes(1);
				}
			}));

			$("#timerstarticon,#timerstarttext").click(function () {
				$("#timerstart").val( function(i, value) {
					return tma.date( 'Y/m/d H:i', new Date( value ) );
				} ).datetimepicker("show");
			}).css( 'cursor', 'pointer' );

			//Setup the duration field
			$("#timerduration").timepicker($.extend({}, dtpOptions, {
				onClose: function (selectedDateTime) {
					$("#timerduration").val( tma.toPvrDate( new Date('2000/01/01 ' + selectedDateTime) ) );
					self.refreshTimerEditTimes(1);
				}
			}));

			$("#timerdurationicon,#timerdurationtext").click(function () {
				$("#timerduration").val( function(i, value) {
					return tma.date( 'H:i', new Date( value ) );
				} ).datetimepicker("show");
			}).css( 'cursor', 'pointer' );

			// Setup the end date field
			$("#timerend").datetimepicker($.extend({}, dtpOptions, {
				onClose: function (selectedDateTime) {
					$("#timerend").val( tma.toPvrDate( new Date(selectedDateTime) ) );
					self.refreshTimerEditTimes(2);
				}
			}));

			$("#timerendicon,#timerendtext").click(function () {
				$("#timerend").val( function(i, value) {
					return tma.date( 'Y/m/d H:i', new Date( value ) );
				} ).datetimepicker("show");
			}).css( 'cursor', 'pointer' );

			$('#timertypediv').buttonset();

			self.resetTimerFields(modTimer);

			// JKIT - update the logo on selection change
			$('#timerservice').change(function() {

				var channel = tma.channel( $(this).val() );
				if (channel) {
					// remove old logo class
					// http://stackoverflow.com/questions/2644299/jquery-removeclass-wildcard
					// and add current
					$('#timerservicelogo').removeClass(function (index, css) {
					    return (css.match (/\blogo-\S+/g) || []).join(' ');
					}).addClass('logo-' + channel.logoid);

				}

			});

			//Make the abort button hide the timer edit window
			$('#timerbuttonabort').click(function() {
				popup.close();
			});

			//Make the reset button reset the timer to its initial settings
			$('#timerbuttonreset').click(function() {
				self.resetTimerFields(modTimer);
			});

			//Make the create/update button do its thing
			$('#timerbuttonupdate').click(function(e) {

				e.stopPropagation();

				var dlgOptions = {};

				if (origTimer.index < 0) {
					dlgOptions = {
							title: self.i18.create_dialog_title,
							text: self.i18.create_dialog_text,
							autoSkipText: tma.i18.respond_auto_skip,
							autoSkipButton: 'tmaCreateTimer',
							buttons: {
								tmaCreateTimer:	 tma.i18.respond_yes,
								jkqDialogCancel: tma.i18.respond_cancel
							}
					};
				} else {
					dlgOptions = {
							title: self.i18.modify_dialog_title,
							text: self.i18.modify_dialog_text,
							autoSkipText: tma.i18.respond_auto_skip,
							autoSkipButton: 'tmaModifyTimer',
							buttons: {
								tmaModifyTimer:	 tma.i18.respond_yes,
								jkqDialogCancel: tma.i18.respond_cancel
							}
					};
				}

				new jkqDialog(dlgOptions).confirm(function(button) {

					if (button == dlgOptions.autoSkipButton) {

						self.saveTimer(origTimer).done(function() {
							popup.close();
						});

					}
				});

			});

			// add event listener to delete timers
			$('#timerbuttondelete').click(function() {

				new jkqDialog({
					title: tma.i18.timer.delete_confirm_caption,
					text:  tma.i18.timer.delete_confirm_text.format(origTimer.getFilename(true)),
					autoSkipText: tma.i18.respond_auto_skip,
					autoSkipButton: 'tmaDeleteTimer',
					buttons: {
						tmaDeleteTimer:	 tma.i18.respond_yes,
						jkqDialogCancel: tma.i18.respond_cancel
					}
				}).confirm(function(button) {

					if (button == 'tmaDeleteTimer') {

						tma.apiDeleteTimer(origTimer).done(function() {

							popup.close();

						}).fail(function(xhr, errorType, error) {

							new jkqAlert({
								title:	tma.i18.error.dialog_title,
								text:	tma.i18.error.timer.del + '<br><br>' + error
							}).confirm();

						});

					}

				});

			});

			/**
			 * this method handled the feature, so
			 * stop the feature chain
			 * and return this popup instance
			 */
			f.stop(popup);

		});

	};

	/**
	 * changing start/duration/end with the toolbar
	 */
	this.adjustTime = function (dateNodeSelector, amount, refreshMode) {

		var $dateNode = $(dateNodeSelector);
		var st = new Date( $dateNode.val() );

		var et = new Date(st.getTime() + (amount * 60000));

		$dateNode.val( tma.toPvrDate(et) );

		this.refreshTimerEditTimes(refreshMode);

	};

	/**
	 * Extract the values from the input fields and reformat them for the API.
	 *
	 * @param {TmaTimer}	origTimer	tma timer object: modify; null: new
	 * @returns {jqXHR}
	 */
	this.saveTimer = function (origTimer) {

		var updateTimer = new TmaTimer();

		jQuery.extend(updateTimer,	origTimer);

		var ttype = $("#timertype1").is(":checked") ? 1 : 0;

		updateTimer.isrec = ttype;

		updateTimer.filename = $("#timername").val() + tma.config.recextension;

		var channel = tma.channel( $("#timerservice").val() );

		updateTimer.servicetype = channel.chtype;
		updateTimer.serviceindex = channel.chnum;

		updateTimer.reservationtype( $("#timerfrequency").val() );

		updateTimer.starttime = $("#timerstart").val();
		updateTimer.endtime = $("#timerend").val();

		var td = new Date( $("#timerduration").val() );
		var tdur = (((td.getHours() * 60 ) + td.getMinutes()));

		updateTimer.duration = tdur;

		return tma.apiTimerUpdate(origTimer, updateTimer).fail(function(xhr, errorType) {

				new jkqAlert({
					title:	tma.i18.error.dialog_title,
					text:	tma.i18.error.timer.create_modify
				}).confirm();

		});

	};

	/**
	 * Initialise the field values for display
	 *
	 * @param {TmaTimer}	timer	tma timer object to edit, empty for a new timer
	 */
	this.resetTimerFields = function (timer) {

		var channelid = '';

		// set defaults
		if	(!timer.filename)	timer.filename	= this.i18.default_filename + tma.config.recextension;
		if	(!timer.logoid)		timer.logoid	= $('#timerservice').val();
		  //The default start time is in 5 minutes
		if	(!timer.starttime)	timer.starttime = tma.toPvrDate(new Date(new Date() - tma.timeOffset + 300000));
		//The default duration is 2 hours
		if	(!timer.duration)	timer.duration	= 120;

		//Set the timer type
		if(timer.isrec == 1) {
			$("#timertype1").prop("checked", true);
		} else {
			$("#timertype0").prop("checked", true);
		}

		//Set the name of the timer
		$("#timername").val(timer.getFilename(true));

		//Set the service
		$('#timerservice').val(timer.logoid);
		channelid = timer.logoid;

		//Set the timer frequency
		$('#timerfrequency').val(timer.reservationtype().toString());

		//Set the start time
		$("#timerstart").val(timer.starttime);

		//Set the duration
		var td = new Date("2000/01/01 02:00:00");
		td.setHours(timer.duration / 60);
		td.setMinutes(timer.duration % 60);
		$("#timerduration").val( tma.toPvrDate(td) );

		//Calculate the end timer
		this.refreshTimerEditTimes(1);

		if (channelid) {
			$('#timerservicelogo').addClass('logo-' + channelid);
		}

	   //Force the UI to update
		$("#timertypediv").buttonset('refresh');

	};

	this.refreshTimerEditTimes = function (refreshMode) {

		var st = new Date( $("#timerstart").val() );
		var td = new Date();
		var et = new Date();

		if ( refreshMode == 1 ) {		//Start + Duration = End

			td = new Date( $("#timerduration").val() );

			var msec = (((td.getHours() * 60 ) + td.getMinutes()) * 60000);

			if( msec < 60000 ) {msec = 60000;}			//Do not allow timers less than 1 minute
			if( msec > 86100000 ) {msec = 86100000;}	//Do not allow timers longer than 23:55 (firmware limit)

			et = new Date(st.getTime() + msec);

		} else {					//Duration = End - Start

			et = new Date( $("#timerend").val() );

			var diff = (et.getTime() - st.getTime()) / 60000;  //Difference in minutes

			if ( diff < 1 )  //If the end date is before the start date, force a 1 minute timer.
			{
				diff = 1;
				et = new Date(st.getTime() + 60000);
			}

			if ( diff > 1435 )  //If the end date more than 23:55 after the start date.
			{
				diff = 1435;
				et = new Date(st.getTime() + (60000 * 1435));
			}

			td = new Date("2000/01/01 00:00:00");
			td.setHours(diff / 60);
			td.setMinutes(diff % 60);

		}


		$("#timerstarttext").text(tma.date(this.i18.timerdate_fmt, st));

		$("#timerduration").val( tma.toPvrDate(td) );
		//$("#timerdurationtext").text(tma.date("H:i", td));
		var dur_min = (et.getTime() - st.getTime()) / 60000;
		$("#timerdurationtext").text(tma.formatDuration(dur_min));  //DMC format duration

		$("#timerend").val( tma.toPvrDate(et) );
		$("#timerendtext").text(tma.date(this.i18.timerdate_fmt, et));

	};

}
