/**
 * timers module
 */
tma.registerModule( timerspage );
timerspage.prototype = new tmaModule();
function timerspage() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'timerspage';

	this.pageGroup = 'timers';

	// available table columns
	var tableColumns = {
			timertype:	true,
			filename:	true,
			starttime:	true,
			endtime:	true,
			duration:	true,
			channel:	true,
			frequency:	true,
			timerfn:	true
	};

	var self = this;

	/**
	 * initialise module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'appendMenuTab' );

		return this.loadLanguage();

	};

	/**
	 * append tab to menu
	 *
	 * @param {tmaFeature}	f
	 */
	this.appendMenuTab = function( f ) {

		tma.call('addPage', ['MainMenu', this]);

	};

	/**
	 * Display the timers, obtained from the PVR in the browser window
	 */
	this.show = function() {

		this.init().done(function() {

			$('#content').html('<img src="css/img/wait.gif">');

			tma.apiGetTimers().done(function() {

				self.showTimers();

				self.insertConfig();

				tma.call('addConfigItem', ['MainMenu', self]);

			}).fail(function(xhr, errorType, error) {

				$("#timestamp").text( tma.i18.error.ajax.format(errorType, error) );
				// remove pending image
				$('#content').empty();

			});

		});

	};

	/**
	 * refresh the timers, obtained from the PVR in the browser window
	 */
	this.refresh = function() {

		$('#content').html('<img src="css/img/wait.gif">');

		tma.apiGetTimers().done(function() {

			self.showTimers();

		}).fail(function(xhr, errorType, error) {

			$("#timestamp").text( tma.i18.error.ajax.format(errorType, error) );
			// remove pending image
			$('#content').empty();

		});

	};

	/**
	 * this page gets removed, so hide the channels config
	 */
	this.cleanup = function() {

		// remove events
		$(tma).off('tmaTimerDeleted.timerspage');
		$(tma).off('tmaTimerAdded.timerspage');
		$(tma).off('tmaTimerModified.timerspage');

	};

	this.showTimers = function () {

		// delete all child nodes of content
		$('#content').empty();

		$("#timestamp"	).text( new Date(tma.timers.timestamp).toLocaleString() );

		// Settings
		var timersFilter = parseInt( this.setting('timersfilter', 0) );
		var channelDisplay = parseInt( this.localSetting('channel_display', 3) );
		var userCols = this.localSetting( 'columns', tableColumns );

		// sort files
		var sortKey = this.localSetting('sort_key', 'starttime');
		var sortDir = this.localSetting('sort_dir', 'asc');
		tma.sortTimers(sortKey, sortDir);

		// create Table
		var $table = $('<table/>', {
			'class': "timers jk-list jk-round"
		});

		var head = '<thead><tr>';
		var row  = '<tr>';

		$.each(userCols, function(clmClass) {
			head += '<th class="' + clmClass + '" can-sort-dir="asc"><a/></th>';
			row  += '<td class="' + clmClass + '"><span/></td>';
		});

		head +=	'</tr></thead>';
		row  +=	'</tr>';

		// create table head
		var $head = $(head).appendTo($table);
		$head.find('TH.timertype A'	).attr('title', this.i18.columns.timertype);
		$head.find('TH.filename A'	).text(this.i18.columns.filename);
		$head.find('TH.starttime A'	).text(this.i18.columns.starttime);
		$head.find('TH.endtime A'	).text(this.i18.columns.endtime);
		$head.find('TH.duration A'	).text(this.i18.columns.duration);
		$head.find('TH.channel A'	).text(this.i18.columns.channel);
		$head.find('TH.frequency A'	).text(this.i18.columns.frequency);

		$head.find('TH.timertype'	).removeAttr('can-sort-dir');
		$head.find('TH.filename'	).attr('sort-key', 'filename');
		$head.find('TH.starttime'	).attr('sort-key', 'starttime');
		$head.find('TH.endtime'		).attr('sort-key', 'endtime');
		$head.find('TH.duration'	).attr('sort-key', 'duration');
		$head.find('TH.channel' 	).attr('sort-key', 'chname');
		$head.find('TH.frequency' 	).attr('sort-key', 'reservationtype');
		$head.find('TH.timerfn'		).removeAttr('can-sort-dir');

		// set the current sort state
		$head.find('TH[sort-key="' + sortKey + '"]').attr({
			'sort-dir':		sortDir,
			'can-sort-dir': (sortDir == 'asc') ? 'desc' : 'asc'
		});

		// set sort event handler
		$head.on('click', 'A', function(e) {
			e.preventDefault();
			var sDir = $(this).parent().attr('can-sort-dir');
			var sKey = $(this).parent().attr('sort-key');
			self.setLocalSetting('sort_key', sKey);
			self.setLocalSetting('sort_dir', sDir);
			self.showTimers();
		});

		var $tbody = $('<tbody/>').appendTo($table);

		$.each(tma.timers.timers, function() {

			/** @returns {TmaTimer} */
			var timer = this;

			$row = $(row).appendTo($tbody);
			$row.prop('timer', timer);
			$row.attr('type', timer.isrec ? 'rec' : 'play');
			$row.attr('timerid', timer.id);

			//$row.find('TD.timertype SPAN'	).text( timer.isrec ? self.i18.type_rec : self.i18.type_play );
			$row.find('TD.timertype SPAN'	).addClass( timer.isrec ? 'fa fa-circle' : 'fa fa-play-circle' );
			$row.find('TD.filename SPAN'	).text( timer.getFilename(true) );
			$row.find('TD.starttime SPAN'	).text( tma.date( self.i18.timerdate_fmt, timer.starttime ) );
			$row.find('TD.endtime SPAN'		).text( tma.date( "H:i", timer.endtime ) );
			$row.find('TD.frequency SPAN'	).text( tma.i18.frequency[timer.reservationtype()].name );
			$row.find('TD.duration SPAN'	).text( tma.formatDuration(timer.duration) );

			// channel
			$row.find('TD.channel').html(
					'<div class="channellogo channellogo-small logo-small-' + timer.logoid + '" />' +
					'<span class="channelname">' + timer.chname + '</span>'
			);

			$('<button class="jk-but iconset ic-delete" title="' + self.i18.delete_tooltip + '"></button>')
			.appendTo($row.find('TD.timerfn > SPAN'));

			$('<button class="jk-but iconset ic-edit" title="' + self.i18.edit_tooltip + '"></button>')
			.appendTo($row.find('TD.timerfn > SPAN'));

		});

		// filter
		if (timersFilter == 1) $table.find('TR[type="play"]').hide();
		if (timersFilter == 2) $table.find('TR[type="rec"]').hide();

		// hide logo or channel name
		if ( (channelDisplay & 1) === 0 ) $table.find('TD.channel > DIV').hide();
		if ( (channelDisplay & 2) === 0 ) $table.find('TD.channel > SPAN').hide();

		// hide deselected columns
		$.each(userCols, function(clmClass, display) {
			if (!display) $table.find( ( 'TD.{0},TH.{0}' ).format( clmClass ) ).hide();
		});

		$('DIV#content')
		.append($table)
		.append('<p class="add-timer"><button class="jk-but iconset ic-plus add-timer"><span>' + this.i18.new_tooltip + '</span></button></p>')
		.find('BUTTON.add-timer').click( function(e) {
			// stop event propagation, needed for 'closeOnOutsideClick'
			e.stopPropagation();

			tma.call('editTimer', [ new TmaTimer({
				index:		-1,		// new timer
				isrec:		1,		// record
			})  ] );
		});

		$('DIV#content P.add-timer').width( $('DIV#content TABLE.timers').width() );

		// add event listener to delete timers
		$('TABLE.timers BUTTON.ic-delete').click(function() {

			self.deleteTimer( $(this).parents('TR').prop('timer') );

		});

		// add event listener to edit timers
		$('TABLE.timers BUTTON.ic-edit').click(function(e) {

			var $row = $(this).parents('TR');
			var timer = $row.prop('timer');
			tma.call( 'editTimer', [ timer ] ).done(function(popup) {

				popup.relTimer = timer;
				$(popup).on('close', function() {

					var closedPopup = this;
					// remove current mark
					$('TABLE.timers TBODY TR').filter(function() {
						return ($(this).prop('timer') == closedPopup.relTimer);
					}).removeClass('jk-mark');

				});

				$row.addClass('jk-mark');

			});

		});

		if (tma.fn.findEvent) {

			// add event listener to view timers event
			$('TABLE.timers TD.filename').click(function(e) {

				var $row = $(this).parents('TR');
				/** @returns {TmaTimer} */
				var timer = $row.prop('timer');
				var options = {
					channel: 	tma.channel(timer.logoid),
					eventid:	timer.eventid2,
					starttime:	timer.starttime,
					endtime:	timer.endtime,
				};

				tma.call( 'findEvent', [ options ] ).done(function(popup) {

					popup.relTimerId = timer.id;
					$(popup).on('close', function() {

						var closedPopup = this;
						// remove current mark
						$('TABLE.timers TR[timerid="{0}"]'.format(closedPopup.relTimerId)).removeClass('jk-mark');

					});

					$row.addClass('jk-mark');

				});

			});

		}

		// handle modified or added timer
		$(tma).on('tmaTimerModified.timerspage tmaTimerAdded.timerspage', function() {

			// reload timers
			tma.apiGetTimers().done(function() {
				self.showTimers();
			});

		});

		// handle deleted timer
		/** @param {TmaTimer} deletedTimer */
		$(tma).on('tmaTimerDeleted.timerspage', function(e, deletedTimer) {

			// remove reservation
			$( 'TABLE.timers TR[timerid="{0}"]'.format(deletedTimer.id) ).remove();

		});

	};

	/**
	 * deletes a reservation
	 *
	 * @param {TmaTimer}	timer
	 */
	this.deleteTimer = function(timer) {

		new jkqDialog({
			title: tma.i18.timer.delete_confirm_caption,
			text:  tma.i18.timer.delete_confirm_text.format( timer.getFilename(true) ),
			autoSkipText: tma.i18.respond_auto_skip,
			autoSkipButton: 'tmaDeleteTimer',
			buttons: {
				tmaDeleteTimer:	 tma.i18.respond_yes,
				jkqDialogCancel: tma.i18.respond_cancel
			}
		}).confirm(function(button) {

			if (button == 'tmaDeleteTimer') {

				tma.apiDeleteTimer(timer).fail(function(xhr, errorType, error) {

					new jkqAlert({
						title:	tma.i18.error.dialog_title,
						text:	tma.i18.error.timer.del + '<br><br>' + error
					}).confirm();

				});

			}

		});

	};

	this.insertConfig = function () {

		// create column config
		var $colSetting = createColumnsConfig(
				this.localSetting( 'columns', tableColumns ),
				this.i18.config.label_columns,
				$.extend({timerfn:	'Buttons'}, this.i18.columns),
				'#content TABLE.timers',
				function(cfg) {
					self.setLocalSetting('columns', cfg );
				}
		);

		// logo/text
		var $channelSetting = createChannelConfig(
				this.localSetting('channel_display', 3),
				this.i18.config.channel_display.caption,
				this.i18.config.channel_display,
				'#content TABLE.timers',
				function(cfg) {
					self.setLocalSetting('channel_display', cfg );
				}
		);

		var $filterSetting = $(
			'<div class="setting">' +
				'<span class="label">' + this.i18.config.filter.caption + '</span>' +
				'<input type="radio" name="timerfilter" id="timerfilter0" value="0"><label for="timerfilter0">' + this.i18.config.filter.all + '</label> ' +
				'<input type="radio" name="timerfilter" id="timerfilter1" value="1"><label for="timerfilter1">' + this.i18.config.filter.rec + '</label> ' +
				'<input type="radio" name="timerfilter" id="timerfilter2" value="2"><label for="timerfilter2">' + this.i18.config.filter.play + '</label> ' +
			'</div>'
		);
		$filterSetting.find('INPUT[name="timerfilter"]').click(function() {

			var timersFilter = $('DIV#config INPUT[name="timerfilter"]:checked').val();
			self.setSetting( 'timersfilter',  timersFilter);

			$('TABLE.timers TR[type]').show();
			if (timersFilter == 1) $('TABLE.timers TR[type="play"]').hide();
			if (timersFilter == 2) $('TABLE.timers TR[type="rec"]').hide();

		}).filter('[value="' +  this.setting('timersfilter', 0) + '"]' ).prop('checked', true);

		$('DIV#config')
		.append($colSetting)
		.append($channelSetting)
		.append($filterSetting);

	};

}
