/**
 * tmsremote module
 */
tma.registerModule( tmsremote );
tmsremote.prototype = new tmaModule();
function tmsremote() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'tmsremote';

	this.pageGroup = 'control';

	var self = this;

	/**
	 * Tap ID
	 */
	this.TapId = 2383036976;

	var tmsrInitialized = false;

	var wasConnected = false;

	/**
	 * turns a language object member into a Mustache template lambda function,
	 * that replaces {xyz} by lambda parameter.
	 */
	function Mustachify( itemName, i18obj ) {

		// init
		if(!i18obj) i18obj = self.i18;

		var itemPath = itemName.split('.');
		if (itemPath.length == 1) {

			if (i18obj[itemName]) {

				var text = i18obj[itemName].replace(/{([^\}]+)}/, '{0}');

				i18obj[itemName] = function() {
					return function(val /*, render*/) {
						return text.format( val );
					};
				};

			}

		} else if (itemPath.length > 1) {

			 Mustachify( itemPath.slice(1).join('.'), i18obj[ itemPath[0] ]);

		}

	}

	/**
	 * override setup
	 */
	this.setup = function() {

		// if not prooved in manifest
		//if (!tma.taps.find(this.TapId)) return false;

		// register tmaFeatures
		tma.fn.addFeature( this, 'appendMenuTab' );

		return this.loadLanguage().done(function() {
			Mustachify( 'config.tv_interval' );
		});

	};

	/**
	 * append tab to menu
	 *
	 * @param {tmaFeature}	f
	 */
	this.appendMenuTab = function( f ) {

		tma.call('addPage', ['MainMenu', this]);

	};

	/**
	 * Display the channels obtained from the PVR in the browser window
	 */
	this.show = function() {

		this.init().done(function() {

			$("#timestamp").hide();

			self.initTmsr();

			self.showPage();

			tma.call('addConfigItem', ['MainMenu', self]);

			self.insertConfig();

			if (wasConnected) {
				self.connect();
			}

		});

	};

	/**
	 * this page gets removed, so hide the channels config
	 */
	this.cleanup = function() {

		$("#timestamp").show();

		$('BODY').off('keypress.tmsremote').off('keyup.tmsremote');
		$(window).off('resize.tmsremote');

		// disconnect
		if ($('BUTTON[name="quit"]').is(':visible')) {
			tmsr.quit();
			wasConnected = true;
		} else {
			wasConnected = false;
		}

	};

	/**
	 * establishes the mouse or touch events for remote buttons
	 *
	 * @param {jQueryObject} $remoteFrame
	 */
	this.setupButtonEvents = function($remoteFrame) {

		/** @param {jQueryEvent} e */
		function getPos(e) {

			if (e.originalEvent.touches || e.originalEvent.changedTouches) {
				// convert coordinates
				var touchstart = e.originalEvent.touches[0] || e.originalEvent.changedTouches[0];
				return {
					x: touchstart.pageX,
					y: touchstart.pageY,
				};
			} else {
				return {
					x: e.pageX,
					y: e.pageY
				};
			}
		}

		//http://stackoverflow.com/questions/8503453/click-event-called-twice-on-touchend-in-ipad
		//http://stackoverflow.com/questions/2915833/how-to-check-browser-for-touchstart-support-using-js-jquery
		var startevent	= ('ontouchstart'	in document.documentElement) ? 'touchstart'	: 'mousedown';
		var stopevent	= ('ontouchend'		in document.documentElement) ? 'touchend'	: 'mouseup';
		var moveevent	= ('ontouchmove'	in document.documentElement) ? 'touchmove'	: 'mousemove';

		var log;
		var keyDownX = 0;
		var keyDownY = 0;
		var active = false;

		$remoteFrame

		/** @param {jQueryEvent} e */
		.on(startevent, '*[keycode]', function(e) {

			var keyCode = parseInt($(this).attr('keyCode'));
			var pos = getPos(e);
			keyDownX = pos.x;
			keyDownY = pos.y;
			active = true;
			tmsr.keyDown(keyCode);
			log = 'down: ' + pos.x + 'x' + pos.y;
			//console.log(log);

		})

		/** @param {jQueryEvent} e */
		.on(stopevent, '*[keycode]', function(e) {

			var keyCode = parseInt($(this).attr('keyCode'));
			tmsr.keyUp();
			var pos = getPos(e);
			log = 'up  : ' + pos.x + 'x' + pos.y;
			//console.log(log);
			active = false;

		})

		/** @param {jQueryEvent} e */
		.on(moveevent, function(e) {

			if (active) {
				var pos = getPos(e);
				log = 'move: ' + pos.x + 'x' + pos.y + ' sc: ' + $remoteFrame.scrollTop();
				//console.log(log);
				if (
						(Math.abs(pos.x - keyDownX) > 5) ||
						(Math.abs(pos.y - keyDownY) > 5)
				) {
					tmsr.keyCancel();
					log = 'cancel';
					//console.log(log);
					active = false;
				}
			}

		});

	};

	this.showPage = function() {

		var $content = $('DIV#content');

		var $template = $( this.templates.content.render(this.i18) )
		.appendTo($content);

		var $remoteFrame = $template.find('.remote-frame')
		.append( self.templates[prefs.getRemoteLayout()].render() );

		this.setupButtonEvents($remoteFrame);

		// connect
		$('BUTTON[name="connect"]').click(function(e) {

			self.connect();

		});

		// disconnect
		$('BUTTON[name="quit"]').click(function() {
			tmsr.quit();
		});

		// refresh TV screen image
		$('BUTTON[name="refreshTv"]').click(function() {

			tmsr.sendCommand(tmsr.Cmd_VideoFrame);

		});

		$('BUTTON[name="hideOsd"]')
		.click(function() {

			// toggle
			var hideOsd = ($(this).attr('hide-osd') == '1') ? 0 : 1;
			$(this).attr('hide-osd', hideOsd ? '1' : '0');
			tmsr.sendCommand(tmsr.Cmd_HideOsd, hideOsd);

		})
		// init
		.attr('hide-osd', prefs.getHideOsd() ? '1' : '0');

		// fullscreen
		$('BUTTON[name="fullscreen"]').click(function() {

			$('BODY').toggleClass('webc-fullscreen');
			$(this).find('I').toggleClass('fa-chevron-up fa-bars');
			self.resizeRemoteFrame();

		});

		$(window).on('resize.tmsremote', function() {
			self.resizeRemoteFrame();
		});
		this.resizeRemoteFrame();

		this.setKeyCodes(prefs.getRemoteCode());

	};

	// adjust vertical size of remote-frame and size of TV screen
	this.resizeRemoteFrame = function() {

		var $remoteFrame = $('.remote-frame');
		var offs = $remoteFrame.offset();
		var maxH = $(window).innerHeight() - offs.top - 5;
		$remoteFrame.css('max-height', maxH);

		// init TV screen size
		var tvscreenMargin = 25;

		var maxW =
			$('.tmsr').width() -
			$('.remote-frame').outerWidth(true) -
			tvscreenMargin;

		maxH -= $('.tmsr-connect').outerHeight() + 2 /* tvscreen border */;
		var w = maxW;
		var h = parseInt(maxW * prefs.getAspectRatio() / 720);
		if (h > maxH) {
			h = maxH;
			w = parseInt(h * 720 / prefs.getAspectRatio());
		}

		$('.tvscreen').width(w).height(h)
		.css({lineHeight: parseInt(h) + 'px'});

	};

	this.initTmsr = function() {

		if (!tmsrInitialized) {

			tmsrInitialized = true;

			tmsr.loadKeymap('/tmsremote/');

			tmsr.setScreenshotIntervall( prefs.getTvRefresh() );

			$(tmsr).on('ws_open', function (e) {

				$('.tmsr-connect').addClass('is-connected').removeClass('is-disconnected');

				// keyboard control
				tmsr.startKeyboardControl();

			});

			$(tmsr).on('ws_close', function (e) {

				$('.tmsr-connect').addClass('is-disconnected').removeClass('is-connected');

				// direct mode off
				$('SPAN.direct-mode').attr('mode', 'off');

				tmsr.stopKeyboardControl();

				$('CANVAS').remove();

			});

			$(tmsr).on('ws_message', function (e, msgNr) {

				switch (msgNr) {

				case tmsr.MID_DirectOn:

					$('SPAN.direct-mode').attr('mode', 'on');
					break;

				case tmsr.MID_DirectOff:

					$('SPAN.direct-mode').attr('mode', 'off');
					break;

				case tmsr.MID_AuthFailed:

					new jkqAlert({
						title:	tma.i18.error.dialog_title,
						text:	self.i18.error.auth_failed
					}).confirm();
					break;

				}

			});

			$(tmsr).on('ws_authenticated', function (e) {

				if (prefs.getHideOsd()) {
					tmsr.sendCommand(tmsr.Cmd_HideOsd, 1);
				}

			});

		}

	};

	this.insertConfig = function () {

		var $config = $(this.templates.config.render( this.i18 ));

		$config.find('INPUT[name="port"]')
		.val( prefs.getPort() )
		.blur(function() {

			prefs.setPort( $(this).val() );

		});

		$config.find('INPUT[name="password"]')
		.val( prefs.getPassword() )
		.blur(function() {

			prefs.setPassword( $(this).val() );

		});

		$config.find('SELECT[name="remoteCode"]')
		.val( prefs.getRemoteCode() )
		.change(function() {

			var code = $(this).val();
			prefs.setRemoteCode( code );
			self.setKeyCodes(code);

		});

		$config.find('SELECT[name="remoteLayout"]')
		.val( prefs.getRemoteLayout() )
		.change(function() {

			var layout = $(this).val();
			prefs.setRemoteLayout( layout );
			var remote = self.templates[layout].render();

			if (remote.match(/^<svg/)) {

				// special inline svg trick

				//import contents of the svg document into this document
				var importedSVGRootElement = document.importNode($(remote).get(0), true);

				// hack for correct scaling in IE 11
				if (window.ActiveXObject || "ActiveXObject" in window) {
					importedSVGRootElement.setAttribute('preserveAspectRatio', 'xMidYMin slice');
				} else {
					importedSVGRootElement.setAttribute('preserveAspectRatio', 'xMidYMin meet');
				}

				//append the imported SVG root element to the appropriate HTML element
				$('.remote-frame').empty().append(importedSVGRootElement);

			} else {
				$('.remote-frame').empty().append( remote );
			}

			self.setKeyCodes(prefs.getRemoteCode());

		});

		$config.find('input[name="hideOsd"]')
		.prop('checked', prefs.getHideOsd() )
		.change(function() {

			var hideOsd = $(this).is(':checked') ? 1 : 0;
			prefs.setHideOsd( hideOsd );
			$('BUTTON[name="hideOsd"]').attr('hide-osd', hideOsd ? '1' : '0');

			tmsr.sendCommand(tmsr.Cmd_HideOsd, hideOsd);

		});

		$config.find('SELECT[name="aspectRatio"]')
		.val( prefs.getAspectRatio() )
		.change(function() {

			prefs.setAspectRatio( $(this).val() );
			$('DIV#config').slideUp(function() {
				self.resizeRemoteFrame();
			});

		});

		$config.find('SELECT[name="tvrefresh"]')
		.val( prefs.getTvRefresh() )
		.change(function() {

			var interval = parseInt( $(this).val() );
			prefs.setTvRefresh( interval );
			tmsr.setScreenshotIntervall( interval );

		});

		$('DIV#config').append($config);

	};

	this.setKeyCodes = function(codeTable) {

		var code = tmsr.keyCodes[codeTable];

		$.each(code, function(key, code) {

			$('.remote-frame .' + key).attr('keyCode', code);

		});

	};

	this.connect = function() {

		var $tvCanvas = $('<CANVAS id="tv" width="720" height="576" />').appendTo('.tvscreen');

		if($tvCanvas.get(0).getContext){

			var $osdCanvas = $('<CANVAS id="osd" width="720" height="576" />').appendTo('.tvscreen');

			tmsr.connect(
					window.location.hostname,
					prefs.getPort(),
					prefs.getPassword(),
					$osdCanvas.get(0).getContext('2d'),
					$tvCanvas.get(0).getContext('2d')
			);

		} else {

			alert('Error: canvas feature not available!');

		}

	};

	/* Settings */

	var prefs = {
			/**
			 * get the port setting
			 *
			 * @return {Number}
			 */
			getPort: function() {
				return parseInt(self.setting( 'port', 1413 ));
			},

			/**
			 * set the port setting
			 *
			 * @param {Number}	set		new port setting
			 */
			setPort: function(set) {
				if (parseInt(set)) {
					self.setSetting( 'port', parseInt(set) );
				}
			},

			/**
			 * get the password setting
			 *
			 * @return {String}
			 */
			getPassword: function() {
				return self.localSetting( 'password', '' );
			},

			/**
			 * set the password setting
			 *
			 * @param {String}	set		new password setting
			 */
			setPassword: function(set) {
				self.setLocalSetting( 'password', set );
			},

			/**
			 * get the remote_layout setting
			 *
			 * @return {String}
			 */
			getRemoteLayout: function() {
				return self.setting( 'remote_layout', 'srp2410' );
			},

			/**
			 * set the remote_layout setting
			 *
			 * @param {String}	set		new remote layout setting
			 */
			setRemoteLayout: function(set) {
				var layouts = ['srp2410', 'srp2401', 'crp2401', 'tf5000'];
				if (layouts.indexOf(set) >= 0) {
					self.setSetting( 'remote_layout', set );
				}
			},

			/**
			 * get the remote_code setting
			 *
			 * @return {String}
			 */
			getRemoteCode: function() {
				return self.setting( 'remote_code', 'RT_2100' );
			},

			/**
			 * set the remote_code setting
			 *
			 * @param {String}	set		new remote code setting
			 */
			setRemoteCode: function(set) {
				var codes = ['RT_2100', 'RT_2100', 'RT_7100PLUS', 'RT_7260PLUS'];
				if (codes.indexOf(set) >= 0) {
					self.setSetting( 'remote_code', set );
				}
			},

			/**
			 * get the hide_osd setting
			 *
			 * @return {Boolean}
			 */
			getHideOsd: function() {
				return parseInt(self.localSetting( 'hide_osd', 0 )) > 0;
			},

			/**
			 * set the hide_osd setting
			 *
			 * @param {Boolean}	set		new hide_osd setting
			 */
			setHideOsd: function(set) {
				self.setLocalSetting( 'hide_osd', (set) ? 1 : 0 );
			},

			/**
			 * get the aspect ratio setting
			 *
			 * @return {Number} divisor of 720 (405=16/9, 540=4/3)
			 */
			getAspectRatio: function() {
				return parseInt(self.localSetting( 'aspect_ratio', 405));
			},

			/**
			 * set the aspect ratio setting
			 *
			 * @param {Number}	set		new aspect ratio setting, divisor of 720 (405=16/9, 540=4/3)
			 */
			setAspectRatio: function(set) {
				set = parseInt(set);
				if ((set == 405) || (set == 540)) {
					self.setLocalSetting( 'aspect_ratio', set );
				}
			},

			/**
			 * get the TV picture refresh intervall setting
			 *
			 * @return {Number}
			 */
			getTvRefresh: function() {
				return parseInt(self.localSetting( 'tvrefresh', 0));
			},

			/**
			 * set the TV picture refresh intervall setting
			 *
			 * @param {Number}	set
			 */
			setTvRefresh: function(set) {
				self.setLocalSetting( 'tvrefresh', parseInt(set) );
			},

	};

}
