
/**
 * create a table-columns config element
 *
 * @param {Object}		currentCfg		current config; type: {clmClass: bool display, ... }
 * @param {String}		caption			caption of the settings entry
 * @param {Object}		columnCaptions	type: {clmClass: string caption, ... }
 * @param {String}		target		selector, to access the data table
 * @param {Function}	onCfgChange		callback: function(newColumnsCfg)
 * @returns {jQueryObject} div.setting
 */
function createColumnsConfig(currentCfg, caption, columnCaptions, target, onCfgChange) {

	// create Table
	var $table = $('<table/>', {
		'class': "jk-list jk-round columns"
	});

	var buttons =
		'<div>' +
			'<button class="jk-but shift-left"	><span>&#x25C0;</span></button>' +
			'<input type="checkbox" class="show-hide">' +
			'<button class="jk-but shift-right"	><span>&#x25B6;</span></button>' +
		'</div>';

	// create table head and config row
	var head = '<thead><tr>';
	var row  = '<tbody><tr class="colset">';

	$.each(currentCfg, function(clmClass, display) {
		head += '<th class="' + (display ? 'used' : 'unused') + '" colname="' + clmClass + '">' + columnCaptions[clmClass] + '</th>';
		row  += '<td class="' + (display ? 'used' : 'unused') + '" colname="' + clmClass + '">' + buttons + '</td>';
	});

	head +=	'</tr></thead>';
	row  +=	'</tr></tbody>';

	$(head).appendTo($table);
	$(row ).appendTo($table);

	// check used columns
	$table.find('TD.used INPUT.show-hide,TD[colname="filefn"] INPUT.show-hide').prop('checked', true);

	function triggerCfgChange() {

		var cells = {};
		$table.find( 'INPUT.show-hide' ).each(function() {
			cells[ $(this).parents('TD').attr('colname') ] = $(this).is(':checked');
		});

		if (onCfgChange) onCfgChange(cells);

	}

	$table.find( 'BUTTON.shift-right' ).click(function() {

		var col = $(this).parents('TD').attr('colname');

		$(
				('TD[colname="{0}"],TH[colname="{0}"]').format( col )
		).each(function() {
			$(this).insertAfter( $(this).next() );
		});

		triggerCfgChange();

		$( target ).find(
				('TD.{0},TH.{0}').format( col )
		).each(function() {
			$(this).insertAfter( $(this).next() );
		});

	});

	$table.find( 'BUTTON.shift-left' ).click(function() {

		var col = $(this).parents('TD').attr('colname');

		$table.find(
				('TD[colname="{0}"],TH[colname="{0}"]').format( col )
		).each(function() {
			$(this).insertBefore( $(this).prev() );
		});

		triggerCfgChange();

		$( target ).find(
				('TD.{0},TH.{0}').format( col )
		).each(function() {
			$(this).insertBefore( $(this).prev() );
		});

	});

	$table.find( 'INPUT.show-hide').click(function() {

		var col = $(this).parents('TD').attr('colname');

		$(
				('TD[colname="{0}"],TH[colname="{0}"]' ).format( col )
		).toggleClass('used unused');

		triggerCfgChange();

		$( target ).find( ( 'TD.{0},TH.{0}' ).format( col ) ).toggle();

	});

	var $colSetting = $('<div class="setting"><span>' + caption + '</span></div>')
	.append($table);

	return $colSetting;

}
