/**
 * Plugin: Test for click-/touchable osd elements
 */

/* Array of bytes to base64 string decoding */

function base64DecToArr (sBase64, nBlocksSize) {

	function b64ToUint6 (nChr) {

		return nChr > 64 && nChr < 91 ?
				nChr - 65
				: nChr > 96 && nChr < 123 ?
						nChr - 71
						: nChr > 47 && nChr < 58 ?
								nChr + 4
								: nChr === 43 ?
										62
										: nChr === 47 ?
												63
												:
													0;

	}

	var
	sB64Enc = sBase64.replace(/[^A-Za-z0-9\+\/]/g, ""), nInLen = sB64Enc.length,
	nOutLen = nBlocksSize ? Math.ceil((nInLen * 3 + 1 >> 2) / nBlocksSize) * nBlocksSize : nInLen * 3 + 1 >> 2,
	ta = new ArrayBuffer(nOutLen);
	taBytes = new Uint8Array(ta);

	for (var nMod3, nMod4, nUint24 = 0, nOutIdx = 0, nInIdx = 0; nInIdx < nInLen; nInIdx++) {
		nMod4 = nInIdx & 3;
		nUint24 |= b64ToUint6(sB64Enc.charCodeAt(nInIdx)) << 18 - 6 * nMod4;
		if (nMod4 === 3 || nInLen - nInIdx === 1) {
			for (nMod3 = 0; nMod3 < 3 && nOutIdx < nOutLen; nMod3++, nOutIdx++) {
				taBytes[nOutIdx] = nUint24 >>> (16 >>> nMod3 & 24) & 255;
			}
			nUint24 = 0;

		}
	}

	return ta;
}

function base64EncFromArr(arrayBuffer) {
	var base64    = '';
	var encodings = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';

	var bytes         = new Uint8Array(arrayBuffer);
	var byteLength    = bytes.byteLength;
	var byteRemainder = byteLength % 3;
	var mainLength    = byteLength - byteRemainder;

	var a, b, c, d;
	var chunk;

	// Main loop deals with bytes in chunks of 3
	for (var i = 0; i < mainLength; i = i + 3) {
		// Combine the three bytes into a single integer
		chunk = (bytes[i] << 16) | (bytes[i + 1] << 8) | bytes[i + 2];

		// Use bitmasks to extract 6-bit segments from the triplet
		a = (chunk & 16515072) >> 18; // 16515072 = (2^6 - 1) << 18
		b = (chunk & 258048)   >> 12; // 258048   = (2^6 - 1) << 12
		c = (chunk & 4032)     >>  6; // 4032     = (2^6 - 1) << 6
		d = chunk & 63;               // 63       = 2^6 - 1

		// Convert the raw binary segments to the appropriate ASCII encoding
		base64 += encodings[a] + encodings[b] + encodings[c] + encodings[d];
	}

	// Deal with the remaining bytes and padding
	if (byteRemainder == 1) {
		chunk = bytes[mainLength];

		a = (chunk & 252) >> 2; // 252 = (2^6 - 1) << 2

		// Set the 4 least significant bits to zero
		b = (chunk & 3)   << 4; // 3   = 2^2 - 1

		base64 += encodings[a] + encodings[b] + '==';
	} else if (byteRemainder == 2) {
		chunk = (bytes[mainLength] << 8) | bytes[mainLength + 1];

		a = (chunk & 64512) >> 10; // 64512 = (2^6 - 1) << 10
		b = (chunk & 1008)  >>  4; // 1008  = (2^6 - 1) << 4

		// Set the 2 least significant bits to zero
		c = (chunk & 15)    <<  2; // 15    = 2^4 - 1

		base64 += encodings[a] + encodings[b] + encodings[c] + '=';
	}

	return base64;
}

$(function() {

	// insert a base 64 input area
	$('TEXTAREA#log').after('<p>base64 input:<br><textarea id="base64" /><br><button id="insertb64">insert</button></p>');
	$('TEXTAREA#base64').css({
		width: 720,
		height: 200
	});

	$('BUTTON#insertb64').click(function() {

		var lines = $('TEXTAREA#base64').val().split('\n');

		for ( var i = 0; i < lines.length; i++) {
			tmsr.decodeMsgData(base64DecToArr(lines[i]), osdCtx, tvCtx);
		}

	});

});

$(tmsr).on('ws_message', function (e, msgNr) {

	switch (msgNr) {

		case tmsr.MID_OSDframeComplete:

			var r = tmsr.getUpdatedRegion();
			if (r.width > 0) {

				osdCtx.strokeStyle = "#00FF00";
				osdCtx.strokeRect(
						r.left,
						r.top,
						r.width,
						r.height
				);
			}

			log('  (' +
					r.left + ',' +
					r.top + '),(' +
					r.right + ',' +
					r.bottom + ')');

			break;

	}

});

$(tmsr).on('ws_osdblock', function (e, data) {

    log(base64EncFromArr(data));

});
