
var logtext = '';

function log(msg) {
	//$("#log").append(msg + "\n");
	logtext += msg + "\n";
}

$(tmsr).on('log', function (e, msg) {
	log(msg);
});

function create_canvas() {

	var $tvCanvas = $('<CANVAS id="tv" width="720" height="576" />').css({
		width: 	$('DIV#tvscreen').width(),
		height: $('DIV#tvscreen').height(),
	}).appendTo('DIV#tvscreen');

	if($tvCanvas.get(0).getContext){

		var $osdCanvas = $('<CANVAS id="osd" width="720" height="576" />').css({
			width: 	$('DIV#tvscreen').width(),
			height: $('DIV#tvscreen').height(),
		}).appendTo('DIV#tvscreen');

		return {
			tv:		$tvCanvas.get(0).getContext('2d'),
			osd:	$osdCanvas.get(0).getContext('2d'),
		};

	}

	return false;

}

$(tmsr).on('ws_open', function (e) {

	$('DIV#disconnected').hide();
	$('DIV#connected').show();

	// keyboard control
	tmsr.startKeyboardControl();

});

$(tmsr).on('ws_close', function (e) {

	$('DIV#disconnected').show();
	$('DIV#connected').hide();
	
	// direct mode off
	$('SPAN.direct-mode').attr('mode', 'off');

	tmsr.stopKeyboardControl();

	$('CANVAS').remove();

});

$(tmsr).on('ws_message', function (e, msgNr) {

	switch (msgNr) {

		case tmsr.MID_DirectOn:

			$('SPAN.direct-mode').attr('mode', 'on');
			break;

		case tmsr.MID_DirectOff:

			$('SPAN.direct-mode').attr('mode', 'off');
			break;

	}

});

function setKeyCodes(codeTable) {

	var code = tmsr.keyCodes[codeTable];

	$.each(code, function(key, code) {

		$('.remote .' + key).attr('keyCode', code);

	});

}

function setRemotePos() {

	var offs = $('DIV#tvscreen').offset();
	$('DIV.remote').css({
		top: offs.top,
		left: offs.left + parseInt($('DIV#tvscreen').width()) + 20
	});

}

function setTvSize(sizeStr) {

	var size = sizeStr.split('x');
	if (size.length == 2) {

		$('DIV#tvscreen').css({
			width: 	parseInt(size[0]),
			height: parseInt(size[1]),
		});

		$('DIV#tvscreen CANVAS').css({
			width: 	parseInt(size[0]),
			height: parseInt(size[1]),
		});

		setRemotePos();

	}

}

/**
 * establishes the mouse or touch events for remote buttons
 *
 * @param {jQueryObject} $remoteFrame
 */
function setupButtonEvents($remoteFrame) {

	/** @param {jQueryEvent} e */
	function getPos(e) {

		if (e.originalEvent.touches || e.originalEvent.changedTouches) {
			// convert coordinates
			var touchstart = e.originalEvent.touches[0] || e.originalEvent.changedTouches[0];
			return {
				x: touchstart.pageX,
				y: touchstart.pageY,
			};
		} else {
			return {
				x: e.pageX,
				y: e.pageY
			};
		}
	}

	//http://stackoverflow.com/questions/8503453/click-event-called-twice-on-touchend-in-ipad
	//http://stackoverflow.com/questions/2915833/how-to-check-browser-for-touchstart-support-using-js-jquery
	var startevent	= ('ontouchstart'	in document.documentElement) ? 'touchstart'	: 'mousedown';
	var stopevent	= ('ontouchend'		in document.documentElement) ? 'touchend'	: 'mouseup';
	var moveevent	= ('ontouchmove'	in document.documentElement) ? 'touchmove'	: 'mousemove';

	var keyDownX = 0;
	var keyDownY = 0;
	var active = false;

	$remoteFrame

	/** @param {jQueryEvent} e */
	.on(startevent, '*[keycode]', function(e) {

		var keyCode = parseInt($(this).attr('keyCode'));
		var pos = getPos(e);
		keyDownX = pos.x;
		keyDownY = pos.y;
		keyScrollTop = $remoteFrame.scrollTop();
		active = true;
		tmsr.keyDown(keyCode);

	})

	/** @param {jQueryEvent} e */
	.on(stopevent, '*[keycode]', function(e) {

		var keyCode = parseInt($(this).attr('keyCode'));
		tmsr.keyUp();
		var pos = getPos(e);
		active = false;

	})

	/** @param {jQueryEvent} e */
	.on(moveevent, function(e) {

		if (active) {
			$('PRE').append(log + '\n');
			if (
					(Math.abs(pos.x - keyDownX) > 5) ||
					(Math.abs(pos.y - keyDownY) > 5)
			) {
				tmsr.keyCancel();
				active = false;
			}
		}

	});

}

$(function() {

	// load keycontrol.ini
	tmsr.loadKeymap('');

	//get the SVG document using XMLHTTPRequest
	$.get(
			"TF5000-Remote.svg",
			function(svgDoc){

				//import contents of the svg document into this document
				var importedSVGRootElement = document.importNode(svgDoc.documentElement,true);

				// hack for correct scaling in IE 11
				if (window.ActiveXObject || "ActiveXObject" in window) {
					importedSVGRootElement.setAttribute('preserveAspectRatio', 'xMidYMin slice');
				} else {
					importedSVGRootElement.setAttribute('preserveAspectRatio', 'xMidYMin meet');
				}

				//append the imported SVG root element to the appropriate HTML element
				$("DIV.tf5000").append(importedSVGRootElement);

			},
			"xml"
	).always(function() {

		// features - browser
		$('P.browserid').append(navigator.userAgent);

		// remote buttons
		/*$('DIV.remote').on('click', '*[keycode]', function() {

			if ( $('DIV#connected').is(':visible') ) {
				var keyCode = parseInt($(this).attr('keyCode'));

				tmsr.sendKey(keyCode);
			}

		});*/

		setupButtonEvents($('DIV.remote'));

		setRemotePos();

		$('DIV#disconnected INPUT[name="host"]').val(window.location.hostname);

		// connect
		$('FORM#connect').submit(function(e) {

			e.preventDefault();

			$('INPUT#hideOsd').attr('checked', false);

			var ctx = create_canvas();
			if (ctx) {

				var host		= $('INPUT[name="host"]').val();
				var port		= $('INPUT[name="port"]').val();
				var password	= $('INPUT[name="password"]').val();

				tmsr.connect(host, port, password, ctx.osd, ctx.tv);

			} else {

				alert('Error: canvas feature not available!');

			}

		});

		// disconnect
		$('BUTTON#quit').click(function() {
			$('BODY').off('keypress').off('keyup');
			tmsr.quit();
		});

		// remote selection
		$('SELECT[name="remoteTypes"]').on('change', function() {

			var type = $(this).val();
			$('DIV.remote').hide().filter('.' + type ).show();

		}).val('srp2410').change();

		// remote code selection
		$('SELECT[name="remoteCodes"]').on('change', function() {

			setKeyCodes( $(this).val() );

		}).val('RT_2100').change();

		// refresh TV screen image
		$('BUTTON#refreshTv').click(function() {

			tmsr.sendCommand(tmsr.Cmd_VideoFrame);

		});

		// screen size selection
		$('SELECT[name="viewSize"]').on('change', function() {

			setTvSize( $(this).val() );

		});

		// toggle OSD visibility on toppy
		$('INPUT#hideOsd').click(function() {

			var value = $(this).is(':checked') ? 1 : 0;

			tmsr.sendCommand(tmsr.Cmd_HideOsd, value);

		});

		// refresh log
		$("#refreshlog").click(function() {
			$("#log").val(logtext);
			logtext = '';
		});

	});

});

